@@ -390,7 +390,7 @@ DMGNAME="${LyxBase}${LyXGitCommitHash:+-}${LyXGitCommitHash}"
 DMGSIZE="550m"
 
 # Check for existing SDKs
-SDKs=$(echo ${DEVELOPER_SDKS}/MacOSX1[01]*sdk)
+SDKs=$(echo ${DEVELOPER_SDKS}/MacOSX[12][0-9]*sdk)
 case $SDKs in
 *${SDKROOT}*)
 	;;
@@ -716,7 +716,7 @@ build_lyx() {
 		echo CPPFLAGS="${CPPFLAGS}"
 		export CPPFLAGS
 		echo CONFIGURE_OPTIONS="${LyXConfigureOptions}" ${QtInstallDir:+"--with-qt-dir=${QtInstallDir}"}
-		export PATH="${QtInstallDir}/libexec:$PATH"
+		export PATH="${QtInstallDir}/bin:${QtInstallDir}/libexec:$PATH"
 		"${LyxSourceDir}/configure"\
 			--prefix="${LyxAppPrefix}" --with-version-suffix="-${LyXVersionSuffix}"\
 			${QtInstallDir:+"--with-qt-dir=${QtInstallDir}"} \
@@ -895,6 +895,17 @@ convert_universal() {
 	for arch in ${ARCH_LIST} ; do
 		rm -f "${BUNDLE_PATH}"/*-${arch}
 	done
+	if [ -d "${QtInstallDir}/lib/QtCore.framework/Versions/${QtFrameworkVersion}" -a "yes" = "${qt_deployment}" ]; then
+		echo Verify runtime path configuration
+		for file in "${LyxAppPrefix}"/Contents/MacOS/*lyx* $(find "${LyxAppPrefix}"/Contents -name '*.dylib' -print) ; do
+			echo Check "${file}"
+			BAD_FRAMEWORKS=$(otool -L "${file}" | grep 'Qt.*framework' | grep -v '@rpath' | sort -u)
+			if [ -n "${BAD_FRAMEWORKS}" ]; then
+				echo Bad runtime path configuration in "${file}" for frameworks "${BAD_FRAMEWORKS}"
+				exit 1
+			fi
+		done
+	fi
 }
 
 # -------------------------
@@ -964,12 +975,12 @@ copy_dictionaries() {
 	fi
 	if [ -d "${DictionarySourceDir}" -a "yes" = "${hunspell_dictionaries}" ]; then
 		HunSpellResources="${LyxAppPrefix}/Contents/Resources"
-		( cd "${DictionarySourceDir}" && find dicts -name .svn -prune -o -type f -print | cpio -pmdv "${HunSpellResources}" )
+		( cd "${DictionarySourceDir}" && find dicts -name .svn -prune -o -type f -print | cpio -pmdv --insecure "${HunSpellResources}" )
 		deduplicate "${HunSpellResources}"/dicts
 	fi
 	if [ -d "${DictionarySourceDir}" -a "yes" = "${thesaurus_deployment}" ]; then
 		MyThesResources="${LyxAppPrefix}/Contents/Resources"
-		( cd "${DictionarySourceDir}" && find thes -name .svn -prune -o -type f -print | cpio -pmdv "${MyThesResources}" )
+		( cd "${DictionarySourceDir}" && find thes -name .svn -prune -o -type f -print | cpio -pmdv --insecure "${MyThesResources}" )
 		deduplicate "${MyThesResources}"/thes
 	fi
 }
