@@ -39,8 +39,8 @@ std::string citationStyleToString(CitationStyle const &, bool const latex = fals
 /// Transforms the information about authors into a <authorgroup> (directly written to a XMLStream).
 /// Type: "author" or empty means author of the entry (article, book, etc.); "book" means author of the book
 /// (but not necessarily of this entry in particular).
-void authorsToDocBookAuthorGroup(docstring const & authorsString, XMLStream & xs, Buffer const & buf,
-								 std::string type);
+void authorsToDocBookAuthorGroup(docstring const & authorsString, XMLStream & xs,
+                                 Buffer const & buf, std::string const & type);
 
 
 /// Class to represent information about a BibTeX or
@@ -68,10 +68,10 @@ public:
 	/// otherwise, it will be translated to the buffer language.
 	docstring const getAuthorOrEditorList(Buffer const * buf = nullptr,
 					      size_t const max_key_size = 128,
-					      bool full = false, bool forceshort = false) const;
+					      bool amp = false, bool full = false, bool forceshort = false) const;
 	/// Same for a specific author role (editor, author etc.)
 	docstring const getAuthorList(Buffer const * buf, docstring const & author, size_t const max_key_size,
-				      bool const full = false, bool const forceshort = false,
+				      bool const amp = false, bool const full = false, bool const forceshort = false,
 				      bool const allnames = false, bool const beginning = true) const;
 	///
 	docstring const getYear() const;
@@ -81,7 +81,8 @@ public:
 	/// \param vector of pointers to crossref/xdata information
 	docstring const & getInfo(BibTeXInfoList const & xrefs,
 				  Buffer const & buf, CiteItem const & ci,
-				  docstring const & format = docstring()) const;
+				  docstring const & format = docstring(),
+				  bool const for_xhtml = false) const;
 	/// \return formatted BibTeX data for a citation label
 	docstring const getLabel(BibTeXInfoList const & xrefs,
 		Buffer const & buf, docstring const & format,
@@ -117,7 +118,7 @@ public:
 	docstring const & key() const { return bib_key_; }
 	/// numerical key for citing this entry. currently used only
 	/// by XHTML output routines.
-	docstring citeNumber() const { return cite_number_; }
+	docstring const & citeNumber() const { return cite_number_; }
 	///
 	void setCiteNumber(docstring const & num) { cite_number_ = num; }
 	/// a,b,c, etc, for author-year. currently used only by XHTML
@@ -126,7 +127,7 @@ public:
 	///
 	void setModifier(char c) { modifier_ = c; }
 	///
-	docstring entryType() const { return entry_type_; }
+	docstring const & entryType() const { return entry_type_; }
 	///
 	bool isBibTeX() const { return is_bibtex_; }
 private:
@@ -236,10 +237,11 @@ public:
 	/// output any richtext tags marked in the citation format and escape < and >
 	/// elsewhere, and the general output context.
 	docstring const getInfo(docstring const & key, Buffer const & buf,
-			CiteItem const & ci, docstring const & format = docstring()) const;
+				CiteItem const & ci, docstring const & format = docstring(),
+				bool const for_xhtml = false) const;
 	/// \return formatted BibTeX data for citation labels.
 	/// Citation labels can have more than one key.
-	docstring const getLabel(std::vector<docstring> keys, Buffer const & buf,
+	docstring const getLabel(std::vector<docstring> const & keys, Buffer const & buf,
 				 std::string const & style, CiteItem const & ci) const;
 	/// Is this a reference from a bibtex database
 	/// or from a bibliography environment?
