@@ -506,6 +506,7 @@ Converters::RetVal Converters::convert(Buffer const * buffer,
 				&& !bp.useBibtopic()
 				&& bp.multibib == "child";
 		runparams.includeall = includeall;
+		runparams.use_memindex = bp.use_memindex;
 	}
 
 	// Some converters (e.g. lilypond) can only output files to the
@@ -966,7 +967,7 @@ Converters::RetVal Converters::runLaTeX(Buffer const & buffer, string const & co
 
 
 
-void Converters::buildGraph()
+void Converters::buildGraph(bool notex)
 {
 	// clear graph's data structures
 	G_.init(theFormats().size());
@@ -974,6 +975,8 @@ void Converters::buildGraph()
 	// so, for each of them, we create an arrow on the graph, going from
 	// the one to the other
 	for (auto const & cvt : converterlist_) {
+		if (notex && cvt.latex())
+			continue;
 		int const from = theFormats().getNumber(cvt.from());
 		int const to   = theFormats().getNumber(cvt.to());
 		LASSERT(from >= 0, continue);
