@@ -32,6 +32,7 @@
 #include "Text.h"
 #include "TextMetrics.h"
 #include "TocBackend.h"
+#include "FuncStatus.h"
 
 #include "support/debug.h"
 #include "support/docstream.h"
@@ -79,7 +80,7 @@ DocIterator bruteFind(Cursor const & c, int x, int y)
 		int xo;
 		int yo;
 		Inset const * inset = &it.inset();
-		CoordCache::Insets const & insetCache = c.bv().coordCache().getInsets();
+		CoordCache::Insets const & insetCache = c.bv().coordCache().insets();
 
 		// FIXME: in the case where the inset is not in the cache, this
 		// means that no part of it is visible on screen. In this case
@@ -93,8 +94,8 @@ DocIterator bruteFind(Cursor const & c, int x, int y)
 		Point const o = insetCache.xy(inset);
 		inset->cursorPos(c.bv(), it.top(), c.boundary(), xo, yo);
 		// Convert to absolute
-		xo += o.x_;
-		yo += o.y_;
+		xo += o.x;
+		yo += o.y;
 		double d = (x - xo) * (x - xo) + (y - yo) * (y - yo);
 		// '<=' in order to take the last possible position
 		// this is important for clicking behind \sum in e.g. '\sum_i a'
@@ -197,11 +198,11 @@ void CursorData::setCursorSelectionTo(DocIterator dit)
 		// otherwise the cursor is already normal
 		if (i == anchor_.depth())
 			// dit is a proper extension of the anchor_
-			dit.cutOff(i - 1);
+			dit.resize(i);
 		else if (i + 1 < dit.depth()) {
 			// one has dit[i] != anchor_[i] but either dit[i-1] == anchor_[i-1]
 			// or i == 0. Remove excess.
-			dit.cutOff(i);
+			dit.resize(i + 1);
 			if (dit[i] > anchor_[i])
 				// place dit after the inset it was in
 				++dit.pos();
@@ -647,7 +648,7 @@ void CursorData::recordUndoInset(Inset const * inset) const
 
 void CursorData::recordUndoFullBuffer() const
 {
-	buffer()->undo().recordUndoFullBuffer(*this);
+	buffer()->undo().recordUndoFullBuffer();
 }
 
 
@@ -972,15 +973,15 @@ bool Cursor::popForward()
 void Cursor::getPos(int & x, int & y) const
 {
 	Point p = bv().getPos(*this);
-	x = p.x_;
-	y = p.y_;
+	x = p.x;
+	y = p.y;
 }
 
 
 Row const & Cursor::textRow() const
 {
 	CursorSlice const & cs = innerTextSlice();
-	ParagraphMetrics const & pm = bv().parMetrics(cs.text(), cs.pit());
+	ParagraphMetrics const & pm = bv().textMetrics(cs.text()).parMetrics(cs.pit());
 	return pm.getRow(pos(), boundary());
 }
 
@@ -1201,8 +1202,7 @@ void Cursor::getSurroundingPos(pos_type & left_pos, pos_type & right_pos) const
 	right_pos = -1;
 
 	Row const & row = textRow();
-	double dummy = 0;
-	Row::const_iterator cit = row.findElement(pos(), boundary(), dummy);
+	Row::const_iterator cit = row.findElement(pos(), boundary());
 	// Handle the case of empty row
 	if (cit == row.end()) {
 		if (row.isRTL())
@@ -1292,7 +1292,9 @@ bool Cursor::posVisToNewRow(bool movingLeft)
 	// if moving left in an LTR paragraph or moving right in an
 	// RTL one, move to previous row
 	if (par_is_LTR == movingLeft) {
-		if (row.pos() == 0) { // we're at first row in paragraph
+		if (row.start_boundary())
+			boundary(true);
+		else if (row.pos() == 0) { // we're at first row in paragraph
 			if (pit() == 0) // no previous paragraph! don't move
 				return false;
 			// move to last pos in previous par
@@ -1332,12 +1334,13 @@ void Cursor::posVisToRowExtremity(bool left)
 	LYXERR(Debug::RTL, "entering extremity: " << pit() << "," << pos() << ","
 		<< (boundary() ? 1 : 0));
 
+	// FIXME: no need for metrics here!
 	TextMetrics const & tm = bv_->textMetrics(text());
 	// Looking for extremities is like clicking on the left or the
 	// right of the row.
-	int x = tm.origin().x_ + (left ? 0 : textRow().width());
-	bool b = false;
-	pos() = tm.getPosNearX(textRow(), x, b);
+	int x = tm.origin().x + (left ? 0 : textRow().width());
+	auto [p, b] = tm.getPosNearX(textRow(), x);
+	pos() = p;
 	boundary(b);
 
 	LYXERR(Debug::RTL, "leaving extremity: " << pit() << "," << pos() << ","
@@ -1416,11 +1419,7 @@ bool Cursor::atFirstOrLastRow(bool up)
 	TextMetrics const & tm = bv_->textMetrics(text());
 	ParagraphMetrics const & pm = tm.parMetrics(pit());
 
-	int row;
-	if (pos() && boundary())
-		row = pm.pos2row(pos() - 1);
-	else
-		row = pm.pos2row(pos());
+	int const row = pm.getRowIndex(pos(), boundary());
 
 	if (up) {
 		if (pit() == 0 && row == 0)
@@ -1536,14 +1535,14 @@ void Cursor::insert(MathAtom const & t)
 }
 
 
-void Cursor::insert(MathData const & ar)
+void Cursor::insert(MathData const & md)
 {
 	LATTEST(inMathed());
 	macroModeClose();
 	if (selection())
 		cap::eraseSelection(*this);
-	cell().insert(pos(), ar);
-	pos() += ar.size();
+	cell().insert(pos(), md);
+	pos() += md.size();
 	// FIXME audit setBuffer calls
 	inset().setBuffer(bv_->buffer());
 }
@@ -1552,13 +1551,13 @@ void Cursor::insert(MathData const & ar)
 int Cursor::niceInsert(docstring const & t, Parse::flags f, bool enter)
 {
 	LATTEST(inMathed());
-	MathData ar(buffer());
-	asArray(t, ar, f);
-	if (ar.size() == 1 && (enter || selection()))
-		niceInsert(ar[0]);
+	MathData md(buffer());
+	asMathData(t, md, f);
+	if (md.size() == 1 && (enter || selection()))
+		niceInsert(md[0]);
 	else
-		insert(ar);
-	return ar.size();
+		insert(md);
+	return md.size();
 }
 
 
@@ -1571,18 +1570,25 @@ void Cursor::niceInsert(MathAtom const & t)
 	// If possible, enter the new inset and move the contents of the selection
 	if (t->isActive()) {
 		idx_type const idx = prevMath().asNestInset()->firstIdx();
-		MathData ar(buffer());
-		asArray(safe, ar);
-		prevMath().cell(idx).insert(0, ar);
+		MathData md(buffer());
+		asMathData(safe, md);
+		prevMath().cell(idx).insert(0, md);
 		editInsertedInset();
-	} else if (t->asMacro() && !safe.empty()) {
-		MathData ar(buffer());
-		asArray(safe, ar);
-		docstring const name = t->asMacro()->name();
+	} else if (t->asMacroInset()) {
+		MathData md(buffer());
+		if (!safe.empty())
+			asMathData(safe, md);
+		docstring const name = t->asMacroInset()->name();
 		MacroData const * data = buffer()->getMacro(name);
-		if (data && data->numargs() - data->optionals() > 0) {
-			plainInsert(MathAtom(new InsetMathBrace(buffer(), ar)));
+		int npar = data ? data->numargs() - data->optionals() : 0;
+		if (npar > 0) {
+			plainInsert(MathAtom(new InsetMathBrace(buffer(), md)));
 			posBackward();
+			md.clear();
+			for (int i = 1; i < npar; ++i) {
+				plainInsert(MathAtom(new InsetMathBrace(buffer(), md)));
+				posBackward();
+			}
 		}
 	}
 }
@@ -1730,7 +1736,7 @@ void Cursor::handleNest(MathAtom const & a)
 	Parse::flags const f = im && im->currentMode() != InsetMath::MATH_MODE
 		? Parse::TEXTMODE : Parse::NORMAL;
 	MathAtom t = a;
-	asArray(cap::grabAndEraseSelection(*this), t.nucleus()->cell(idx), f);
+	asMathData(cap::grabAndEraseSelection(*this), t.nucleus()->cell(idx), f);
 	insert(t);
 	editInsertedInset();
 }
@@ -1769,7 +1775,7 @@ bool Cursor::macroModeClose(bool cancel)
 	InsetMathUnknown * p = activeMacro();
 	p->finalize();
 	MathData selection(buffer());
-	asArray(p->selection(), selection);
+	asMathData(p->selection(), selection);
 	docstring const s = p->name();
 	--pos();
 	cell().erase(pos());
@@ -1788,12 +1794,21 @@ bool Cursor::macroModeClose(bool cancel)
 	if (in && in->interpretString(*this, s))
 		return true;
 	bool const user_macro = buffer()->getMacro(name, *this, false);
-	MathAtom atom = user_macro ? MathAtom(new InsetMathMacro(buffer(), name))
-				   : createInsetMath(name, buffer());
+
+	MathAtom atom;
+	FuncRequest fr(LFUN_MATH_INSERT, "\\" + name);
+	FuncStatus status;
+	getStatus(fr, status);
+	// exit if the inset cannot be used e.g. in the current math hull
+	if (!status.enabled())
+		return false;
+	else
+		atom = user_macro ? MathAtom(new InsetMathMacro(buffer(), name))
+					   : createInsetMath(name, buffer());
 
 	// try to put argument into macro, if we just inserted a macro
 	bool macroArg = false;
-	InsetMathMacro * atomAsMacro = atom.nucleus()->asMacro();
+	InsetMathMacro * atomAsMacro = atom.nucleus()->asMacroInset();
 	InsetMathNest * atomAsNest = atom.nucleus()->asNestInset();
 	if (atomAsMacro) {
 		// macros here are still unfolded (in init mode in fact). So
@@ -1809,7 +1824,7 @@ bool Cursor::macroModeClose(bool cancel)
 	}
 
 	// insert remembered selection into first argument of a non-macro
-	else if (atomAsNest && atomAsNest->nargs() > 0)
+	else if (atomAsNest && atomAsNest->isActive())
 		atomAsNest->cell(atomAsNest->firstIdx()).append(selection);
 
 	MathWordList const & words = mathedWordList();
@@ -1818,7 +1833,9 @@ bool Cursor::macroModeClose(bool cancel)
 		|| (it != words.end() && (it->second.inset == "font"
 		                          || it->second.inset == "oldfont"
 		                          || it->second.inset == "textsize"
-		                          || it->second.inset == "mbox"));
+		                          || it->second.inset == "mbox"
+		                          || it->second.inset == "intertext"
+		                          || it->second.name == "underbar"));
 	bool ert_macro = !user_macro && it == words.end() && atomAsMacro;
 
 	if (in && in->currentMode() == Inset::TEXT_MODE
@@ -1882,10 +1899,10 @@ docstring Cursor::macroName()
 void Cursor::pullArg()
 {
 	// FIXME: Look here
-	MathData ar = cell();
+	MathData md = cell();
 	if (popBackward() && inMathed()) {
 		plainErase();
-		cell().insert(pos(), ar);
+		cell().insert(pos(), md);
 		resetAnchor();
 	}
 }
@@ -1907,7 +1924,7 @@ void Cursor::normalize()
 		odocstringstream os;
 		otexrowstream ots(os);
 		TeXMathStream wi(ots, false, true, TeXMathStream::wsDefault);
-		inset().asInsetMath()->write(wi);
+		inset().asInsetMath()->writeMath(wi);
 		lyxerr << to_utf8(os.str()) << endl;
 		pos() = lastpos();
 	}
@@ -2004,7 +2021,7 @@ bool Cursor::upDownInMath(bool up)
 		}
 	}
 
-	// try to find an inset that knows better then we,
+	// try to find an inset that knows better than we do
 	if (inset().idxUpDown(*this, up)) {
 		//lyxerr << "idxUpDown triggered" << endl;
 		// try to find best position within this inset
@@ -2148,7 +2165,7 @@ bool Cursor::mathBackward(bool word)
 }
 
 
-bool Cursor::upDownInText(bool up, bool & updateNeeded)
+bool Cursor::upDownInText(bool up)
 {
 	LASSERT(text(), return false);
 
@@ -2158,6 +2175,9 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 	getPos(xo, yo);
 	xo = beforeDispatchPosX_;
 
+	// Is a full repaint necessary?
+	bool updateNeeded = false;
+
 	// update the targetX - this is here before the "return false"
 	// to set a new target which can be used by InsetTexts above
 	// if we cannot move up/down inside this inset anymore
@@ -2197,11 +2217,7 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 	// first get the current line
 	TextMetrics & tm = bv_->textMetrics(text());
 	ParagraphMetrics const & pm = tm.parMetrics(pit());
-	int row;
-	if (pos() && boundary())
-		row = pm.pos2row(pos() - 1);
-	else
-		row = pm.pos2row(pos());
+	int const row = pm.getRowIndex(pos(), boundary());
 
 	if (atFirstOrLastRow(up)) {
 		// Is there a place for the cursor to go ? If yes, we
@@ -2226,17 +2242,18 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 			dummy.pos() = dummy.pos() == 0 ? dummy.lastpos() : 0;
 			dummy.pit() = dummy.pit() == 0 ? dummy.lastpit() : 0;
 
-			updateNeeded |= bv().checkDepm(dummy, *this);
-			updateTextTargetOffset();
-			if (updateNeeded)
+			if (bv().checkDepm(dummy, *this)) {
+				updateNeeded = true;
 				forceBufferUpdate();
+			}
+			updateTextTargetOffset();
 		}
-		return false;
+		return updateNeeded;
 	}
 
 	// with and without selection are handled differently
 	if (!selection()) {
-		int yo1 = bv().getPos(*this).y_;
+		int yo1 = bv().getPos(*this).y;
 		Cursor old = *this;
 		// To next/previous row
 		// FIXME: the y position is often guessed wrongly across styles and
@@ -2257,6 +2274,7 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 			++dummy.pos();
 		if (bv().checkDepm(dummy, old)) {
 			updateNeeded = true;
+			forceBufferUpdate();
 			// Make sure that cur gets back whatever happened to dummy (Lgb)
 			operator=(dummy);
 		}
@@ -2291,19 +2309,18 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 		}
 
 		Row const & real_next_row = tm.parMetrics(pit()).rows()[next_row];
-		bool bound = false;
-		top().pos() = tm.getPosNearX(real_next_row, xo, bound);
-		boundary(bound);
+		tie(pos(), boundary()) = tm.getPosNearX(real_next_row, xo);
 		// When selection==false, this is done by TextMetrics::editXY
 		setCurrentFont();
 
-		updateNeeded |= bv().checkDepm(*this, old);
+		if (bv().checkDepm(*this, old)) {
+			updateNeeded = true;
+			forceBufferUpdate();
+		}
 	}
 
-	if (updateNeeded)
-		forceBufferUpdate();
 	updateTextTargetOffset();
-	return true;
+	return updateNeeded;
 }
 
 
@@ -2402,6 +2419,9 @@ Font Cursor::getFont() const
 
 	// on boundary, so we are really at the character before
 	pos_type pos = sl.pos();
+	// the error message below is temporary for a possible bug around pos value
+	LYXERR(Debug::DEBUG, "cursor position = " << pos << " sl address = " <<
+	       &sl << " sl empty? " << sl.empty());
 	if (pos > 0 && boundary())
 		--pos;
 
@@ -2450,7 +2470,7 @@ bool notifyCursorLeavesOrEnters(Cursor const & old, Cursor & cur)
 	// but stop if the inset claims the cursor to be invalid now
 	for (size_type j = i; j < old.depth(); ++j) {
 		Cursor inset_pos = old;
-		inset_pos.cutOff(j);
+		inset_pos.resize(j + 1);
 		if (old[j].inset().notifyCursorLeaves(inset_pos, cur))
 			return true;
 	}
@@ -2458,7 +2478,7 @@ bool notifyCursorLeavesOrEnters(Cursor const & old, Cursor & cur)
 	// notify everything on top of the common part in new cursor,
 	// but stop if the inset claims the cursor to be invalid now
 	for (; i < cur.depth(); ++i) {
-		if (cur[i].inset().notifyCursorEnters(cur))
+		if (cur[i].inset().notifyCursorEnters(old, cur))
 			return true;
 	}
 
@@ -2466,6 +2486,26 @@ bool notifyCursorLeavesOrEnters(Cursor const & old, Cursor & cur)
 }
 
 
+void notifyMouseSelectionDone(Cursor & cur)
+{
+	// find inset in common
+	size_type i;
+	DocIterator const & anchor = cur.realAnchor();
+	for (i = 0; i < cur.depth() && i < anchor.depth(); ++i) {
+		if (&anchor[i].inset() != &cur[i].inset())
+			break;
+	}
+
+	// notify everything on top of the common part in old cursor
+	for (size_type j = i; j < anchor.depth(); ++j) {
+		Cursor tmpcur = cur;
+		tmpcur.realAnchor().resize(j + 1);
+		anchor[j].inset().notifyMouseSelectionDone(tmpcur);
+		cur.screenUpdateFlags(cur.result().screenUpdate() | tmpcur.result().screenUpdate());
+	}
+}
+
+
 void Cursor::setLanguageFromInput()
 {
 	if (!lyxrc.respect_os_kbd_language
@@ -2554,16 +2594,18 @@ void Cursor::checkBufferStructure()
 void Cursor::moveToClosestEdge(int const x, bool const edit)
 {
 	if (Inset const * inset = nextInset()) {
+		// This is only used in mathed.
+		LATTEST(inset->asInsetMath());
 		// stay in front of insets for which we want to open the dialog
 		// (e.g. InsetMathSpace).
 		if (edit && (inset->hasSettings() || !inset->contextMenuName().empty()))
 			return;
-		CoordCache::Insets const & insetCache = bv().coordCache().getInsets();
+		CoordCache::Insets const & insetCache = bv().coordCache().insets();
 		if (!insetCache.has(inset))
 			return;
 		int const wid = insetCache.dim(inset).wid;
 		Point p = insetCache.xy(inset);
-		if (x > p.x_ + (wid + 1) / 2)
+		if (x > p.x + (wid + 1) / 2)
 			posForward();
 	}
 }
