@@ -50,8 +50,8 @@ global.
   display-related information, in particular the BufferView that owns it.
 */
 
-#ifndef LCURSOR_H
-#define LCURSOR_H
+#ifndef CURSOR_H
+#define CURSOR_H
 
 #include "DispatchResult.h"
 #include "DocIterator.h"
@@ -205,7 +205,7 @@ public:
 	/// As the user is entering a word without leaving it
 	/// the result is not empty. When not in text mode
 	/// and after leaving the word the result is empty.
-	DocIterator newWord() const { return new_word_; }
+	DocIterator const & newWord() const { return new_word_; }
 
 	/// are we in math mode (2), text mode (1) or unsure (0)?
 	int currentMode() const;
@@ -506,9 +506,8 @@ public:
 	/// return true if fullscreen update is needed
 	bool down();
 	/// move up/down in a text inset, called for LFUN_UP/DOWN,
-	/// return true if successful, updateNeeded set to true if fullscreen
-	/// update is needed, otherwise it's not touched
-	bool upDownInText(bool up, bool & updateNeeded);
+	/// return true if fullscreen update is needed
+	bool upDownInText(bool up);
 	/// move up/down in math or any non text inset, call for LFUN_UP/DOWN
 	/// return true if successful
 	bool upDownInMath(bool up);
@@ -564,6 +563,9 @@ public:
 bool notifyCursorLeavesOrEnters(Cursor const & old, Cursor & cur);
 
 
+ // Notifies all insets which appear in \c cur anchor, but not in \c cur tip.
+void notifyMouseSelectionDone(Cursor & cur);
+
 } // namespace lyx
 
-#endif // LCURSOR_H
+#endif // CURSOR_H
