@@ -45,6 +45,7 @@
 #include "insets/InsetGraphicsParams.h"
 #include "insets/InsetInclude.h"
 #include "insets/InsetLabel.h"
+#include "insets/InsetRef.h"
 #include "insets/InsetTabular.h"
 
 #include "mathed/MathData.h"
@@ -60,7 +61,6 @@
 #include "support/limited_stack.h"
 #include "support/lstrings.h"
 #include "support/TempFile.h"
-#include "support/unique_ptr.h"
 
 #include "frontends/alert.h"
 #include "frontends/Clipboard.h"
@@ -126,7 +126,7 @@ static bool isFullyDeleted(ParagraphList const & pars)
 
 PasteReturnValue
 pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
-                     DocumentClassConstPtr oldDocClass, cap::BranchAction branchAction,
+                     DocumentClassConstPtr const & oldDocClass, cap::BranchAction branchAction,
                      ErrorList & errorlist)
 {
 	Buffer const & buffer = *cur.buffer();
@@ -229,7 +229,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 	}
 
 	// set the paragraphs to plain layout if necessary
-	DocumentClassConstPtr newDocClass = buffer.params().documentClassPtr();
+	DocumentClassConstPtr const & newDocClass = buffer.params().documentClassPtr();
 	Layout const & plainLayout = newDocClass->plainLayout();
 	Layout const & defaultLayout = newDocClass->defaultLayout();
 	if (cur.inset().usePlainLayout()) {
@@ -343,9 +343,9 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 				for (InsetIterator itt = begin(in);
 				      itt != i_end; ++itt) {
 					if (itt->lyxCode() == REF_CODE) {
-						InsetCommand * ref = itt->asInsetCommand();
-						if (ref->getParam("reference") == oldname)
-							ref->setParam("reference", newname);
+						InsetRef * ref = itt->asInsetRef();
+						if (ref->hasTarget(oldname))
+							ref->changeTarget(oldname, newname);
 					} else if (itt->lyxCode() == MATH_REF_CODE) {
 						InsetMathRef * mi = itt->asInsetMath()->asRefInset();
 						// this is necessary to prevent an uninitialized
@@ -373,9 +373,9 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 			// adapt the references
 			for (InsetIterator itt = begin(in); itt != i_end; ++itt) {
 				if (itt->lyxCode() == REF_CODE) {
-					InsetCommand & ref = static_cast<InsetCommand &>(*itt);
-					if (ref.getParam("reference") == oldname)
-						ref.setParam("reference", newname);
+					InsetRef & ref = static_cast<InsetRef &>(*itt);
+					if (ref.hasTarget(oldname))
+						ref.changeTarget(oldname, newname);
 				} else if (itt->lyxCode() == MATH_REF_CODE) {
 					InsetMathRef * mi = itt->asInsetMath()->asRefInset();
 					// this is necessary to prevent an uninitialized
@@ -600,7 +600,7 @@ PitPosPair eraseSelectionHelper(BufferParams const & params,
 }
 
 
-Buffer * copyToTempBuffer(ParagraphList const & paragraphs, DocumentClassConstPtr docclass)
+Buffer * copyToTempBuffer(ParagraphList const & paragraphs, DocumentClassConstPtr const & docclass)
 {
 	// This used to need to be static to avoid a memory leak. It no longer needs
 	// to be so, but the alternative is to construct a new one of these (with a
@@ -658,7 +658,7 @@ void putClipboard(ParagraphList const & paragraphs,
 	// We don't want to produce images that are not used. Therefore,
 	// output formulas as MathML. Even if this is not understood by all
 	// applications, the number that can parse it should go up in the future.
-	buffer->params().html_math_output = BufferParams::MathML;
+	buffer->params().html_math_output = BufferParams::MathMLCore;
 
 	// Copy authors to the params. We need those pointers.
 	for (Author const & a : bp.authors())
@@ -699,7 +699,7 @@ void putClipboard(ParagraphList const & paragraphs,
 
 void copySelectionHelper(Buffer const & buf, Text const & text,
 	pit_type startpit, pit_type endpit,
-	int start, int end, DocumentClassConstPtr dc, CutStack & cutstack)
+	int start, int end, DocumentClassConstPtr const & dc, CutStack & cutstack)
 {
 	ParagraphList const & pars = text.paragraphs();
 
@@ -828,15 +828,15 @@ bool multipleCellsSelected(CursorData const & cur)
 }
 
 
-void switchBetweenClasses(DocumentClassConstPtr oldone,
-		DocumentClassConstPtr newone, InsetText & in) {
+void switchBetweenClasses(DocumentClassConstPtr const & oldone,
+		DocumentClassConstPtr const & newone, InsetText & in) {
 	ErrorList el = {};
 	switchBetweenClasses(oldone, newone, in, el);
 }
 
 
-void switchBetweenClasses(DocumentClassConstPtr oldone,
-		DocumentClassConstPtr newone, InsetText & in, ErrorList & errorlist)
+void switchBetweenClasses(DocumentClassConstPtr const & oldone,
+		DocumentClassConstPtr const & newone, InsetText & in, ErrorList & errorlist)
 {
 	errorlist.clear();
 
@@ -883,10 +883,10 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 			docstring const layoutName = iit->layoutName();
 			docstring const & n = newone->insetLayout(layoutName).name();
 			bool const is_undefined = n.empty() ||
-				n == DocumentClass::plainInsetLayout().name();
+				n == InsetLayout::undefined().name();
 			docstring const & oldn = oldone->insetLayout(layoutName).name();
 			bool const was_undefined = oldn.empty() ||
-				oldn == DocumentClass::plainInsetLayout().name();
+				oldn == InsetLayout::undefined().name();
 			if (!is_undefined || was_undefined)
 				continue;
 
@@ -989,6 +989,8 @@ void cutSelectionHelper(Cursor & cur, CutStack & cuts, bool realcut, bool putcli
 
 		if (begpit != endpit)
 			cur.screenUpdateFlags(Update::Force | Update::FitCursor);
+		else
+			cur.screenUpdateFlags(Update::SinglePar | Update::FitCursor);
 
 		tie(endpit, endpos) =
 			eraseSelectionHelper(bp, text->paragraphs(), begpit, endpit,
@@ -1243,14 +1245,14 @@ docstring selection(size_t sel_index, DocInfoPair docinfo, bool for_math)
 
 
 void pasteParagraphList(Cursor & cur, ParagraphList const & parlist,
-			DocumentClassConstPtr docclass, AuthorList const & authors,
+			DocumentClassConstPtr const & docclass, AuthorList const & authors,
 			ErrorList & errorList,
 			cap::BranchAction branchAction)
 {
 	// Copy authors to the params. We need those pointers.
 	for (Author const & a : authors)
 		cur.buffer()->params().authors().record(a);
-	
+
 	if (cur.inTexted()) {
 		Text * text = cur.text();
 		LBUFERR(text);
