@@ -24,23 +24,10 @@ class DispatchResult
 {
 public:
 	///
-	DispatchResult() :
-			dispatched_(false),
-			error_(false),
-			update_(Update::None),
-			need_buf_update_(false),
-			need_msg_update_(true),
-			need_changes_update_(false)
-	{}
+	DispatchResult() = default;
 	///
 	DispatchResult(bool dispatched, Update::flags f) :
-			dispatched_(dispatched),
-			error_(false),
-			update_(f),
-			need_buf_update_(false),
-			need_msg_update_(true),
-			need_changes_update_(false)
-	{}
+			dispatched_(dispatched), update_(f) {}
 	///
 	bool dispatched() const { return dispatched_; }
 	///
@@ -50,7 +37,7 @@ public:
 	///
 	void setError(bool e) { error_ = e; }
 	///
-	docstring message() { return message_; }
+	docstring const & message() const { return message_; }
 	///
 	void setMessage(docstring const & m) { message_ = m; }
 	///
@@ -59,14 +46,12 @@ public:
 	Update::flags screenUpdate() const { return update_; }
 	///
 	void screenUpdate(Update::flags f) { update_ = f; }
-
 	/// Does the buffer need updating?
 	bool needBufferUpdate() const { return need_buf_update_; }
 	/// Force the buffer to be updated
 	void forceBufferUpdate() { need_buf_update_ = true; }
 	/// Clear the flag indicating we need an update
 	void clearBufferUpdate() { need_buf_update_ = false; }
-
 	/// Do we need to display a message in the status bar?
 	bool needMessageUpdate() const { return need_msg_update_; }
 	/// Force the message to be displayed
@@ -74,29 +59,19 @@ public:
 	/// Clear the flag indicating we need to display the message
 	void clearMessageUpdate() { need_msg_update_ = false; }
 
-	/// Do we need to update the change tracking presence flag?
-	bool needChangesUpdate() { return need_changes_update_; }
-	/// Force the change tracking presence flag to be updated
-	void forceChangesUpdate() { need_changes_update_ = true; }
-	/// Clear the flag indicating that we need to update the change tracking
-	/// presence flag
-	void clearChangesUpdate() { need_changes_update_ = false; }
-
 private:
 	/// was the event fully dispatched?
-	bool dispatched_;
+	bool dispatched_ = false;
 	/// was there an error?
-	bool error_;
+	bool error_ = false;
 	/// do we need to redraw the screen afterwards?
-	Update::flags update_;
+	Update::flags update_ = Update::None;
 	///
 	docstring message_;
 	///
-	bool need_buf_update_;
+	bool need_buf_update_ = false;
 	///
-	bool need_msg_update_;
-	///
-	bool need_changes_update_;
+	bool need_msg_update_ = true;
 };
 
 
