@@ -20,8 +20,8 @@
 #include "ColorSet.h"
 #include "Encoding.h"
 #include "Language.h"
+#include "LaTeXColors.h"
 #include "LaTeXFeatures.h"
-#include "Lexer.h"
 #include "LyXRC.h"
 #include "output_latex.h"
 #include "OutputParams.h"
@@ -31,6 +31,7 @@
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/gettext.h"
+#include "support/Lexer.h"
 #include "support/lstrings.h"
 
 #include <cstring>
@@ -158,9 +159,9 @@ string const Font::latexSize() const
 
 /// Writes the changes from this font to orgfont in .lyx format in file
 void Font::lyxWriteChanges(Font const & orgfont,
-			      ostream & os) const
+			      ostream & ret_os) const
 {
-	os << "\n";
+	ostringstream os;
 	if (orgfont.fontInfo().family() != bits_.family())
 		os << "\\family " << LyXFamilyNames[bits_.family()] << "\n";
 	if (orgfont.fontInfo().series() != bits_.series())
@@ -219,6 +220,9 @@ void Font::lyxWriteChanges(Font const & orgfont,
 		else
 			os << "\\lang unknown\n";
 	}
+
+    if (!os.str().empty())
+        ret_os << "\n" << os.str();
 }
 
 
@@ -242,8 +246,9 @@ int Font::latexWriteStartChanges(otexstream & os, BufferParams const & bparams,
 			if (needs_cprotection)
 				tmp += "\\cprotect";
 			tmp += "\\text" + language()->polyglossia();
-			if (!language()->polyglossiaOpts().empty()) {
-				tmp += "[" + language()->polyglossiaOpts() + "]";
+			string const opts = bparams.polyglossiaLangOptions(language()->lang());
+			if (!opts.empty()) {
+				tmp += "[" + opts + "]";
 				if (runparams.use_hyperref && runparams.moving_arg) {
 					// We need to strip the command for
 					// the pdf string, see #11813
@@ -812,31 +817,17 @@ void Font::validate(LaTeXFeatures & features) const
 		features.require("ulem");
 		LYXERR(Debug::OUTFILE, "Wavy underline enabled. Font: " << to_utf8(stateText()));
 	}
-	switch (bits_.color()) {
-		case Color_none:
-		case Color_inherit:
-		case Color_ignore:
-			// probably we should put here all interface colors used for
-			// font displaying! For now I just add this ones I know of (Jug)
-		case Color_latex:
-		case Color_notelabel:
-			break;
-		case Color_brown:
-		case Color_darkgray:
-		case Color_gray:
-		case Color_lightgray:
-		case Color_lime:
-		case Color_olive:
-		case Color_orange:
-		case Color_pink:
-		case Color_purple:
-		case Color_teal:
-		case Color_violet:
+	string const col = lcolor.getLyXName(bits_.color());
+	if (theLaTeXColors().isLaTeXColor(col)) {
+		LaTeXColor const lc = theLaTeXColors().getLaTeXColor(col);
+		for (auto const & r : lc.req())
+			features.require(r);
+		features.require("color");
+		if (!lc.model().empty()) {
 			features.require("xcolor");
-			break;
-		default:
-			features.require("color");
-			LYXERR(Debug::OUTFILE, "Color enabled. Font: " << to_utf8(stateText()));
+			features.require("xcolor:" + lc.model());
+		}
+		LYXERR(Debug::OUTFILE, "Color enabled. Font: " << to_utf8(stateText()));
 	}
 
 	// FIXME: Do something for background and soul package?
