@@ -27,10 +27,23 @@ class Buffer;
 class BufferParams;
 class InsetLayout;
 class Language;
+class Layout;
 class otexstream;
 class OutputParams;
 struct TexString;
 
+
+struct ThmInfo {
+	std::string name;
+	std::string latexname;
+	std::string counter;
+	std::string parent_counter;
+	std::string style;
+	std::string zrefname;
+	std::string refprefix;
+};
+
+
 /** The packages and commands that a buffer needs. This class
  *  contains a list<string>.  Each of the LaTeX packages that a buffer needs
  *  should be added with void require(string const & name).
@@ -68,10 +81,20 @@ public:
 	docstring const getBabelPresettings() const;
 	/// Extra preamble code after babel is called
 	docstring const getBabelPostsettings() const;
+	/// get theorem definitions
+	std::string const getThmDefinitions() const;
+	/// get theorem extra (package specific) definitions
+	std::string const getThmExtraDefinitions() const;
 	/// Load AMS packages when appropriate
 	std::string const loadAMSPackages() const;
 	/// The definitions needed by the document's textclass
 	docstring const getTClassPreamble() const;
+	/// Localizations for theorem reference defs
+	docstring const getThmI18nDefs(Layout const &) const;
+	/// Localizations for extra reference defs
+	docstring const getXRefI18nDefs(Layout const &) const;
+	/// Localizations for extra reference defs
+	docstring const getXRefI18nDefs(InsetLayout const &) const;
 	/// The language dependent definitions needed by the document's textclass
 	docstring const getTClassI18nPreamble(bool use_babel,
 				bool use_polyglossia, bool use_minted) const;
@@ -133,6 +156,8 @@ public:
 	/// A vector of all used languages supported only by babel
 	std::vector<std::string> getBabelExclusiveLanguages() const;
 	///
+	std::set<Language const *> getLanguages() const { return UsedLanguages_; }
+	///
 	std::string getBabelLanguages() const;
 	///
 	std::set<std::string> getPolyglossiaLanguages() const;
@@ -187,11 +212,17 @@ public:
 	bool hasRTLLanguage() const;
 	/// This is used to avoid recursion in InsetMathMacro::validate
 	std::set<docstring> & activeMacros() { return active_macros_; }
+	/// Return a series of \usepackage{} commands for unknown packages
+	/// mentioned in the Require tag of layouts/custom insets that
+	/// that have not already been accounted for
+	std::string getUnknownPackages() const;
 
 private:
 	///
 	void useLayout(docstring const &, int);
 	///
+	std::string const getColorValue(std::string const &) const;
+	///
 	std::list<docstring> usedLayouts_;
 	///
 	std::list<docstring> usedInsetLayouts_;
@@ -201,6 +232,8 @@ private:
 	Features features_;
 	/// Features that are provided
 	Features provides_;
+	/// Already accounted for features
+	mutable Features features_loaded_;
 	/// Static preamble bits, from external templates, or anywhere else
 	typedef std::list<TexString> SnippetList;
 	///
@@ -216,6 +249,8 @@ private:
 	///
 	UsedFloats usedFloats_;
 	///
+	std::list<ThmInfo> usedTheorems_;
+	///
 	typedef std::map<docstring, std::string> FileMap;
 	///
 	FileMap IncludedFiles_;
