@@ -25,12 +25,16 @@ namespace lyx {
 class LaTeXPackages {
 public:
 	/// Which of the required packages are installed?
-	static void getAvailable();
+	static void getAvailable(bool retry = false);
 	/// Is the (required) package available?
 	static bool isAvailable(std::string const & name);
+	/// Is the (required) package available at least as of
+	/// version y/m/d?
+	static bool isAvailableAtLeastFrom(std::string const & name,
+					   int const y, int const m, int const d = 1);
 private:
 	/// The available (required) packages
-	typedef std::set<std::string> Packages;
+	typedef std::set<std::pair<std::string,std::string>> Packages;
 	///
 	static Packages packages_;
 };
