@@ -13,16 +13,14 @@
 #include <config.h>
 
 #include "Layout.h"
-#include "FontInfo.h"
-#include "Language.h"
-#include "Lexer.h"
 #include "output_xhtml.h"
 #include "TextClass.h"
 
 #include "support/debug.h"
+#include "support/docstream.h"
 #include "support/lassert.h"
+#include "support/Lexer.h"
 #include "support/lstrings.h"
-#include "support/Messages.h"
 #include "support/textutils.h"
 
 
@@ -31,7 +29,7 @@ using namespace lyx::support;
 
 namespace lyx {
 
-/// Special value of toclevel for layouts that to not belong in a TOC
+/// Special value of toclevel for layouts that do not belong to a TOC
 const int Layout::NOT_IN_TOC = -1000;
 
 //  The order of the LayoutTags enum is no more important. [asierra300396]
@@ -50,10 +48,12 @@ enum LayoutTags {
 	LT_DEPENDSON,
 	LT_OBSOLETEDBY,
 	LT_END,
+	LT_ESCAPE_CHARS,
 	LT_FONT,
 	LT_FREE_SPACING,
 	LT_PASS_THRU,
 	LT_PASS_THRU_CHARS,
+	LT_NO_PASS_THRU_CHARS,
 	LT_PARBREAK_IS_NEWLINE,
 	LT_ITEMCOMMAND,
 	LT_ITEMSEP,
@@ -74,6 +74,9 @@ enum LayoutTags {
 	LT_LATEXTYPE,
 	LT_LEFTDELIM,
 	LT_LEFTMARGIN,
+	LT_MULTIPAR,
+	LT_NEED_CPROTECT,
+	LT_NEED_MBOXPROTECT,
 	LT_NEED_PROTECT,
 	LT_NEWLINE,
 	LT_NEXTNOINDENT,
@@ -92,8 +95,17 @@ enum LayoutTags {
 	LT_INNERTAG,
 	LT_LABELTAG,
 	LT_ITEMTAG,
+	LT_THM_NAME,
+	LT_THM_LATEXNAME,
+	LT_THM_COUNTER,
+	LT_THM_PARENTCOUNTER,
+	LT_THM_STYLE,
+	LT_THM_CROSSREFNAME,
+	LT_THM_CROSSREFNAMEPL,
+	LT_THM_ZREFNAME,
 	LT_HTMLTAG,
 	LT_HTMLATTR,
+	LT_HTMLCLASS,
 	LT_HTMLITEM,
 	LT_HTMLITEMATTR,
 	LT_HTMLLABEL,
@@ -102,34 +114,67 @@ enum LayoutTags {
 	LT_HTMLPREAMBLE,
 	LT_HTMLSTYLE,
 	LT_HTMLFORCECSS,
+	LT_DOCBOOKTAG,
+	LT_DOCBOOKATTR,
+	LT_DOCBOOKTAGTYPE,
+	LT_DOCBOOKINNERATTR,
+	LT_DOCBOOKINNERTAG,
+	LT_DOCBOOKINNERTAGTYPE,
+	LT_DOCBOOKININFO,
+	LT_DOCBOOKABSTRACT,
+	LT_DOCBOOKGENERATETITLE,
+	LT_DOCBOOKWRAPPERTAG,
+	LT_DOCBOOKWRAPPERATTR,
+	LT_DOCBOOKWRAPPERTAGTYPE,
+	LT_DOCBOOKWRAPPERMERGEWITHPREVIOUS,
+	LT_DOCBOOKSECTION,
+	LT_DOCBOOKSECTIONTAG,
+	LT_DOCBOOKITEMWRAPPERTAG,
+	LT_DOCBOOKITEMWRAPPERATTR,
+	LT_DOCBOOKITEMWRAPPERTAGTYPE,
+	LT_DOCBOOKITEMTAG,
+	LT_DOCBOOKITEMATTR,
+	LT_DOCBOOKITEMTAGTYPE,
+	LT_DOCBOOKITEMLABELTAG,
+	LT_DOCBOOKITEMLABELATTR,
+	LT_DOCBOOKITEMLABELTAGTYPE,
+	LT_DOCBOOKITEMINNERTAG,
+	LT_DOCBOOKITEMINNERATTR,
+	LT_DOCBOOKITEMINNERTAGTYPE,
+	LT_DOCBOOKFORCEABSTRACTTAG,
+	LT_DOCBOOKNOFONTINSIDE,
 	LT_INPREAMBLE,
 	LT_HTMLTITLE,
 	LT_SPELLCHECK,
 	LT_REFPREFIX,
 	LT_RESETARGS,
 	LT_RESUMECOUNTER,
-	LT_STEPMASTERCOUNTER,
+	LT_STEPPARENTCOUNTER,
 	LT_RIGHTDELIM,
 	LT_FORCELOCAL,
 	LT_TOGGLE_INDENT,
 	LT_ADDTOTOC,
 	LT_ISTOCCAPTION,
+	LT_HTMLINTOC,
 	LT_INTITLE // keep this last!
 };
 
 /////////////////////
 
 Layout::Layout()
-	: add_to_toc_(false), is_toc_caption_(true)
+	: add_to_toc_(false), multi_par_(false), is_toc_caption_(true)
 {
 	unknown_ = false;
 	margintype = MARGIN_STATIC;
 	latextype = LATEX_PARAGRAPH;
 	resumecounter = false;
-	stepmastercounter = false;
+	stepparentcounter = false;
 	intitle = false;
 	inpreamble = false;
 	needprotect = false;
+	needcprotect = false;
+	nocprotect = false;
+	needmboxprotect = false;
 	keepempty = false;
 	font = inherit_font;
 	labelfont = inherit_font;
@@ -157,23 +202,28 @@ Layout::Layout()
 	htmllabelfirst_ = false;
 	htmlforcecss_ = false;
 	htmltitle_ = false;
+	htmlintoc_ = true;
+	docbookabstract_ = false;
+	docbookwrappermergewithprevious_ = false;
+	docbooksection_ = false;
 	spellcheck = true;
 	forcelocal = 0;
 	itemcommand_ = "item";
 	toggle_indent = ITOGGLE_DOCUMENT_DEFAULT;
 	par_group_ = false;
+	thm_style_ = "plain";
 }
 
 
-bool Layout::read(Lexer & lex, TextClass const & tclass)
+bool Layout::read(Lexer & lex, TextClass const & tclass, bool validating)
 {
 	// If this is an empty layout, or if no force local version is set,
 	// we know that we will not discard the stuff to read
 	if (forcelocal == 0)
-		return readIgnoreForcelocal(lex, tclass);
+		return readIgnoreForcelocal(lex, tclass, validating);
 	Layout tmp(*this);
 	tmp.forcelocal = 0;
-	bool const ret = tmp.readIgnoreForcelocal(lex, tclass);
+	bool const ret = tmp.readIgnoreForcelocal(lex, tclass, validating);
 	// Keep the stuff if
 	// - the read version is higher
 	// - both versions are infinity (arbitrary decision)
@@ -185,7 +235,8 @@ bool Layout::read(Lexer & lex, TextClass const & tclass)
 }
 
 
-bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
+bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass,
+								  bool validating)
 {
 	// This table is sorted alphabetically [asierra 30March96]
 	LexerKeyword layoutTags[] = {
@@ -200,14 +251,46 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 		{ "commanddepth",   LT_COMMANDDEPTH },
 		{ "copystyle",      LT_COPYSTYLE },
 		{ "dependson",      LT_DEPENDSON },
+		{ "docbookabstract",           LT_DOCBOOKABSTRACT },
+		{ "docbookattr",               LT_DOCBOOKATTR },
+		{ "docbookforceabstracttag",   LT_DOCBOOKFORCEABSTRACTTAG },
+		{ "docbookgeneratetitle",      LT_DOCBOOKGENERATETITLE },
+		{ "docbookininfo",             LT_DOCBOOKININFO },
+		{ "docbookinnerattr",          LT_DOCBOOKINNERATTR },
+		{ "docbookinnertag",           LT_DOCBOOKINNERTAG },
+		{ "docbookinnertagtype",       LT_DOCBOOKINNERTAGTYPE },
+		{ "docbookitemattr",           LT_DOCBOOKITEMATTR },
+		{ "docbookiteminnerattr",      LT_DOCBOOKITEMINNERATTR },
+		{ "docbookiteminnertag",       LT_DOCBOOKITEMINNERTAG },
+		{ "docbookiteminnertagtype",   LT_DOCBOOKITEMINNERTAGTYPE },
+		{ "docbookitemlabelattr",      LT_DOCBOOKITEMLABELATTR },
+		{ "docbookitemlabeltag",       LT_DOCBOOKITEMLABELTAG },
+		{ "docbookitemlabeltagtype",   LT_DOCBOOKITEMLABELTAGTYPE },
+		{ "docbookitemtag",            LT_DOCBOOKITEMTAG },
+		{ "docbookitemtagtype",        LT_DOCBOOKITEMTAGTYPE },
+		{ "docbookitemwrapperattr",    LT_DOCBOOKITEMWRAPPERATTR },
+		{ "docbookitemwrappertag",     LT_DOCBOOKITEMWRAPPERTAG },
+		{ "docbookitemwrappertagtype", LT_DOCBOOKITEMWRAPPERTAGTYPE },
+		{ "docbooknofontinside",       LT_DOCBOOKNOFONTINSIDE, },
+		{ "docbooksection",            LT_DOCBOOKSECTION },
+		{ "docbooksectiontag",         LT_DOCBOOKSECTIONTAG },
+		{ "docbooktag",                LT_DOCBOOKTAG },
+		{ "docbooktagtype",            LT_DOCBOOKTAGTYPE },
+		{ "docbookwrapperattr",        LT_DOCBOOKWRAPPERATTR },
+		{ "docbookwrappermergewithprevious", LT_DOCBOOKWRAPPERMERGEWITHPREVIOUS },
+		{ "docbookwrappertag",         LT_DOCBOOKWRAPPERTAG },
+		{ "docbookwrappertagtype",     LT_DOCBOOKWRAPPERTAGTYPE },
 		{ "end",            LT_END },
 		{ "endlabelstring", LT_ENDLABELSTRING },
 		{ "endlabeltype",   LT_ENDLABELTYPE },
+		{ "escapechars",    LT_ESCAPE_CHARS },
 		{ "font",           LT_FONT },
 		{ "forcelocal",     LT_FORCELOCAL },
 		{ "freespacing",    LT_FREE_SPACING },
 		{ "htmlattr",       LT_HTMLATTR },
+		{ "htmlclass",      LT_HTMLCLASS },
 		{ "htmlforcecss",   LT_HTMLFORCECSS },
+		{ "htmlintoc",      LT_HTMLINTOC },
 		{ "htmlitem",       LT_HTMLITEM },
 		{ "htmlitemattr",   LT_HTMLITEMATTR },
 		{ "htmllabel",      LT_HTMLLABEL },
@@ -242,9 +325,13 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 		{ "leftdelim",      LT_LEFTDELIM },
 		{ "leftmargin",     LT_LEFTMARGIN },
 		{ "margin",         LT_MARGIN },
+		{ "multipar",       LT_MULTIPAR },
+		{ "needcprotect",    LT_NEED_CPROTECT },
+		{ "needmboxprotect", LT_NEED_MBOXPROTECT },
 		{ "needprotect",    LT_NEED_PROTECT },
 		{ "newline",        LT_NEWLINE },
 		{ "nextnoindent",   LT_NEXTNOINDENT },
+		{ "nopassthruchars", LT_NO_PASS_THRU_CHARS },
 		{ "obsoletedby",    LT_OBSOLETEDBY },
 		{ "paragraphgroup", LT_PAR_GROUP },
 		{ "parbreakisnewline", LT_PARBREAK_IS_NEWLINE },
@@ -262,8 +349,16 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 		{ "rightmargin",    LT_RIGHTMARGIN },
 		{ "spacing",        LT_SPACING },
 		{ "spellcheck",     LT_SPELLCHECK },
-		{ "stepmastercounter",  LT_STEPMASTERCOUNTER },
+		{ "stepparentcounter",  LT_STEPPARENTCOUNTER },
 		{ "textfont",       LT_TEXTFONT },
+		{ "theoremcounter",    LT_THM_COUNTER },
+		{ "theoremcrossrefname", LT_THM_CROSSREFNAME },
+		{ "theoremcrossrefpluralname", LT_THM_CROSSREFNAMEPL },
+		{ "theoremlatexname",  LT_THM_LATEXNAME },
+		{ "theoremname",       LT_THM_NAME },
+		{ "theoremparentcounter", LT_THM_PARENTCOUNTER },
+		{ "theoremstyle",      LT_THM_STYLE },
+		{ "theoremzrefname",   LT_THM_ZREFNAME },
 		{ "toclevel",       LT_TOCLEVEL },
 		{ "toggleindent",   LT_TOGGLE_INDENT },
 		{ "topsep",         LT_TOPSEP }
@@ -374,6 +469,7 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 				latexargs_.clear();
 				itemargs_.clear();
 				postcommandargs_.clear();
+				listpreamble_.clear();
 			}
 			break;
 
@@ -381,18 +477,34 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> resumecounter;
 			break;
 
-		case LT_STEPMASTERCOUNTER:
-			lex >> stepmastercounter;
+		case LT_STEPPARENTCOUNTER:
+			lex >> stepparentcounter;
 			break;
 
 		case LT_ARGUMENT:
-			readArgument(lex);
+			readArgument(lex, validating);
 			break;
 
 		case LT_NEED_PROTECT:
 			lex >> needprotect;
 			break;
 
+		case LT_NEED_CPROTECT: {
+			int i;
+			lex >> i;
+			nocprotect = false;
+			needcprotect = false;
+			if (i == -1)
+				nocprotect = true;
+			else if (i == 1)
+				needcprotect = true;
+			break;
+		}
+
+		case LT_NEED_MBOXPROTECT:
+			lex >> needmboxprotect;
+			break;
+
 		case LT_KEEPEMPTY:
 			lex >> keepempty;
 			break;
@@ -437,7 +549,7 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 
 		case LT_LATEXPARAM:
 			lex >> latexparam_;
-			latexparam_ = subst(latexparam_, "&quot;", "\"");
+			latexparam_ = subst(latexparam_, "&#34;", "\"");
 			break;
 
 		case LT_LEFTDELIM:
@@ -468,6 +580,38 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> itemcommand_;
 			break;
 
+		case LT_THM_NAME:
+			lex >> thm_name_;
+			break;
+
+		case LT_THM_LATEXNAME:
+			lex >> thm_latex_name_;
+			break;
+
+		case LT_THM_COUNTER:
+			lex >> thm_counter_;
+			break;
+
+		case LT_THM_PARENTCOUNTER:
+			lex >> thm_parent_counter_;
+			break;
+
+		case LT_THM_STYLE:
+			lex >> thm_style_;
+			break;
+
+		case LT_THM_CROSSREFNAME:
+			lex >> thm_xref_name_;
+			break;
+
+		case LT_THM_CROSSREFNAMEPL:
+			lex >> thm_xref_name_pl_;
+			break;
+
+		case LT_THM_ZREFNAME:
+			lex >> thm_zref_name_;
+			break;
+
 		case LT_PREAMBLE:
 			preamble_ = lex.getLongString(from_ascii("EndPreamble"));
 			break;
@@ -537,6 +681,10 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> newline_allowed;
 			break;
 
+		case LT_MULTIPAR:
+			lex >> multi_par_;
+			break;
+
 		case LT_ALIGN:
 			readAlign(lex);
 			break;
@@ -576,10 +724,20 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> pass_thru;
 			break;
 
+		case LT_ESCAPE_CHARS: {
+			lex.next(true);
+			escape_chars = lex.getDocString();
+			break;
+		}
+
 		case LT_PASS_THRU_CHARS:
 			lex >> pass_thru_chars;
 			break;
 
+		case LT_NO_PASS_THRU_CHARS:
+			lex >> no_pass_thru_chars;
+			break;
+
 		case LT_PARBREAK_IS_NEWLINE:
 			lex >> parbreak_is_newline;
 			break;
@@ -592,7 +750,7 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex.eatLine();
 			vector<string> const req =
 				getVectorFromString(lex.getString(true));
-			requires_.insert(req.begin(), req.end());
+			required_.insert(req.begin(), req.end());
 			break;
 		}
 
@@ -630,6 +788,10 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			break;
 		}
 
+		case LT_HTMLINTOC:
+			lex >> htmlintoc_;
+			break;
+
 		case LT_HTMLTAG:
 			lex >> htmltag_;
 			break;
@@ -638,6 +800,10 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> htmlattr_;
 			break;
 
+		case LT_HTMLCLASS:
+			lex >> htmlclass_;
+			break;
+
 		case LT_HTMLITEM:
 			lex >> htmlitemtag_;
 			break;
@@ -674,6 +840,122 @@ bool Layout::readIgnoreForcelocal(Lexer & lex, TextClass const & tclass)
 			lex >> htmltitle_;
 			break;
 
+		case LT_DOCBOOKTAG:
+			lex >> docbooktag_;
+			break;
+
+		case LT_DOCBOOKATTR:
+			lex >> docbookattr_;
+			break;
+
+		case LT_DOCBOOKTAGTYPE:
+			lex >> docbooktagtype_;
+			break;
+
+		case LT_DOCBOOKINNERTAG:
+			lex >> docbookinnertag_;
+			break;
+
+		case LT_DOCBOOKINNERATTR:
+			lex >> docbookinnerattr_;
+			break;
+
+		case LT_DOCBOOKINNERTAGTYPE:
+			lex >> docbookinnertagtype_;
+			break;
+
+		case LT_DOCBOOKFORCEABSTRACTTAG:
+			lex >> docbookforceabstracttag_;
+			break;
+
+		case LT_DOCBOOKININFO:
+			lex >> docbookininfo_;
+			break;
+
+		case LT_DOCBOOKABSTRACT:
+			lex >> docbookabstract_;
+			break;
+
+		case LT_DOCBOOKWRAPPERTAG:
+			lex >> docbookwrappertag_;
+			break;
+
+		case LT_DOCBOOKWRAPPERATTR:
+			lex >> docbookwrapperattr_;
+			break;
+
+		case LT_DOCBOOKWRAPPERTAGTYPE:
+			lex >> docbookwrappertagtype_;
+			break;
+
+		case LT_DOCBOOKWRAPPERMERGEWITHPREVIOUS:
+			lex >> docbookwrappermergewithprevious_;
+			break;
+
+		case LT_DOCBOOKSECTION:
+			lex >> docbooksection_;
+			break;
+
+		case LT_DOCBOOKSECTIONTAG:
+			lex >> docbooksectiontag_;
+			break;
+
+		case LT_DOCBOOKITEMWRAPPERTAG:
+			lex >> docbookitemwrappertag_;
+			break;
+	
+		case LT_DOCBOOKITEMWRAPPERATTR:
+			lex >> docbookitemwrapperattr_;
+			break;
+
+		case LT_DOCBOOKITEMWRAPPERTAGTYPE:
+			lex >> docbookitemwrappertagtype_;
+			break;
+
+		case LT_DOCBOOKITEMTAG:
+			lex >> docbookitemtag_;
+			break;
+
+		case LT_DOCBOOKITEMATTR:
+			lex >> docbookitemattr_;
+			break;
+
+		case LT_DOCBOOKITEMTAGTYPE:
+			lex >> docbookitemtagtype_;
+			break;
+
+		case LT_DOCBOOKITEMLABELTAG:
+			lex >> docbookitemlabeltag_;
+			break;
+
+		case LT_DOCBOOKITEMLABELATTR:
+			lex >> docbookitemlabelattr_;
+			break;
+
+		case LT_DOCBOOKITEMLABELTAGTYPE:
+			lex >> docbookitemlabeltagtype_;
+			break;
+
+		case LT_DOCBOOKITEMINNERTAG:
+			lex >> docbookiteminnertag_;
+			break;
+
+		case LT_DOCBOOKITEMINNERATTR:
+			lex >> docbookiteminnerattr_;
+			break;
+
+		case LT_DOCBOOKITEMINNERTAGTYPE:
+			lex >> docbookiteminnertagtype_;
+			break;
+
+		case LT_DOCBOOKNOFONTINSIDE:
+			lex >> docbooknofontinside_;
+			break;
+
+		case LT_DOCBOOKGENERATETITLE:
+			lex >> docbookgeneratetitle_;
+			break;
+
 		case LT_SPELLCHECK:
 			lex >> spellcheck;
 			break;
@@ -993,24 +1275,30 @@ void Layout::readSpacing(Lexer & lex)
 }
 
 
-void Layout::readArgument(Lexer & lex)
+void Layout::readArgument(Lexer & lex, bool validating)
 {
-	latexarg arg;
-	// writeArgument() makes use of these default values
-	arg.mandatory = false;
-	arg.autoinsert = false;
-	arg.insertcotext = false;
-	bool error = false;
-	bool finished = false;
-	arg.font = inherit_font;
-	arg.labelfont = inherit_font;
-	arg.is_toc_caption = false;
-	arg.passthru = PT_INHERITED;
-	string id;
-	lex >> id;
+	if (!lex.next()) {
+		LYXERR0("Unable to read argument ID!");
+		return;
+	}
+	string const id = lex.getString();
+
 	bool const itemarg = prefixIs(id, "item:");
 	bool const postcmd = prefixIs(id, "post:");
+	bool const listpreamble = prefixIs(id, "listpreamble:");
 
+	LaTeXArgMap & lam = itemarg ? itemargs_ :
+			(postcmd ? postcommandargs_ :
+			(listpreamble ? listpreamble_ :
+			latexargs_));
+	latexarg & arg = lam[id];
+
+	if (listpreamble)
+		// list preamble has no delimiters by default
+		arg.nodelims = true;
+
+	bool error = false;
+	bool finished = false;
 	while (!finished && lex.isOK() && !error) {
 		lex.next();
 		string const tok = ascii_lowercase(lex.getString());
@@ -1054,15 +1342,21 @@ void Layout::readArgument(Lexer & lex)
 			lex.next();
 			arg.tooltip = lex.getDocString();
 		} else if (tok == "requires") {
-			lex.next();
-			arg.requires = lex.getString();
+			lex.eatLine();
+			arg.required = lex.getString();
 		} else if (tok == "decoration") {
 			lex.next();
 			arg.decoration = lex.getString();
+		} else if (tok == "newlinecmd") {
+			lex.next();
+			arg.newlinecmd = lex.getString();
 		} else if (tok == "font") {
 			arg.font = lyxRead(lex, arg.font);
 		} else if (tok == "labelfont") {
 			arg.labelfont = lyxRead(lex, arg.labelfont);
+		} else if (tok == "escapechars") {
+			lex.next();
+			arg.escape_chars = lex.getDocString();
 		} else if (tok == "passthruchars") {
 			lex.next();
 			arg.pass_thru_chars = lex.getDocString();
@@ -1078,19 +1372,37 @@ void Layout::readArgument(Lexer & lex)
 		} else if (tok == "istoccaption") {
 			lex.next();
 			arg.is_toc_caption = lex.getBool();
+		} else if (tok == "freespacing") {
+			lex.next();
+			arg.free_spacing = lex.getBool();
+		} else if (tok == "inheritfont") {
+			lex.next();
+			arg.inh_font = lex.getBool();
+		} else if (tok == "docbooktag") {
+			lex.next();
+			arg.docbooktag = lex.getDocString();
+		} else if (tok == "docbookattr") {
+			lex.next();
+			arg.docbookattr = lex.getDocString();
+		} else if (tok == "docbooktagtype") {
+			lex.next();
+			arg.docbooktagtype = lex.getDocString();
+		} else if (tok == "docbookargumentaftermaintag") {
+			lex.next();
+			arg.docbookargumentaftermaintag = lex.getBool();
+		} else if (tok == "docbookargumentbeforemaintag") {
+			lex.next();
+			arg.docbookargumentbeforemaintag = lex.getBool();
 		} else {
 			lex.printError("Unknown tag");
 			error = true;
 		}
 	}
-	if (arg.labelstring.empty())
+	if (!validating && arg.labelstring.empty()) {
 		LYXERR0("Incomplete Argument definition!");
-	else if (itemarg)
-		itemargs_[id] = arg;
-	else if (postcmd)
-		postcommandargs_[id] = arg;
-	else
-		latexargs_[id] = arg;
+		// remove invalid definition
+		lam.erase(id);
+	}
 }
 
 
@@ -1121,14 +1433,18 @@ void writeArgument(ostream & os, string const & id, Layout::latexarg const & arg
 		os << "\t\tPresetArg \"" << to_utf8(arg.presetarg) << "\"\n";
 	if (!arg.tooltip.empty())
 		os << "\t\tToolTip \"" << to_utf8(arg.tooltip) << "\"\n";
-	if (!arg.requires.empty())
-		os << "\t\tRequires \"" << arg.requires << "\"\n";
+	if (!arg.required.empty())
+		os << "\t\tRequires " << arg.required << "\n";
 	if (!arg.decoration.empty())
 		os << "\t\tDecoration \"" << arg.decoration << "\"\n";
+	if (!arg.newlinecmd.empty())
+		os << "\t\tNewlineCmd \"" << arg.newlinecmd << "\"\n";
 	if (arg.font != inherit_font)
 		lyxWrite(os, arg.font, "Font", 2);
 	if (arg.labelfont != inherit_font)
 		lyxWrite(os, arg.labelfont, "LabelFont", 2);
+	if (!arg.inh_font)
+		os << "\t\tInheritFont " << arg.inh_font << "\n";
 	switch (arg.passthru) {
 		case PT_TRUE:
 			os << "\t\tPassThru true\n";
@@ -1140,8 +1456,12 @@ void writeArgument(ostream & os, string const & id, Layout::latexarg const & arg
 			os << "\t\tPassThru inherited\n";
 			break;
 	}
+	if (!arg.escape_chars.empty())
+		os << "\t\tEscapeChars \"" << to_utf8(arg.escape_chars) << "\"\n";
 	if (!arg.pass_thru_chars.empty())
 		os << "\t\tPassThruChars \"" << to_utf8(arg.pass_thru_chars) << "\"\n";
+	if (arg.free_spacing)
+		os << "\t\tFreeSpacing " << arg.free_spacing << "\n";
 	os << "\tEndArgument\n";
 }
 
@@ -1200,7 +1520,7 @@ void Layout::write(ostream & os) const
 	      "\tInPreamble " << inpreamble << "\n"
 	      "\tTocLevel " << toclevel << "\n"
 	      "\tResumeCounter " << resumecounter << "\n"
-	     "\tStepMasterCounter " << stepmastercounter << '\n';
+	      "\tStepParentCounter " << stepparentcounter << '\n';
 	// ResetArgs does not make sense here
 	for (LaTeXArgMap::const_iterator it = latexargs_.begin();
 	     it != latexargs_.end(); ++it)
@@ -1211,7 +1531,12 @@ void Layout::write(ostream & os) const
 	for (LaTeXArgMap::const_iterator it = postcommandargs_.begin();
 	     it != postcommandargs_.end(); ++it)
 		writeArgument(os, it->first, it->second);
+	for (LaTeXArgMap::const_iterator it = listpreamble_.begin();
+	     it != listpreamble_.end(); ++it)
+		writeArgument(os, it->first, it->second);
 	os << "\tNeedProtect " << needprotect << "\n"
+	      "\tNeedCProtect " << needcprotect << "\n"
+	      "\tNeedMBoxProtect " << needmboxprotect << "\n"
 	      "\tKeepEmpty " << keepempty << '\n';
 	if (labelfont == font)
 		lyxWrite(os, font, "Font", 1);
@@ -1224,7 +1549,7 @@ void Layout::write(ostream & os) const
 	if (!latexname_.empty())
 		os << "\tLatexName \"" << latexname_ << "\"\n";
 	if (!latexparam_.empty())
-		os << "\tLatexParam \"" << subst(latexparam_, "\"", "&quot;")
+		os << "\tLatexParam \"" << subst(latexparam_, "\"", "&#34;")
 		   << "\"\n";
 	if (!leftdelim_.empty())
 		os << "\tLeftDelim "
@@ -1386,6 +1711,8 @@ void Layout::write(ostream & os) const
 		os << "\tLabelCounter \"" << to_utf8(counter) << "\"\n";
 	os << "\tFreeSpacing " << free_spacing << '\n';
 	os << "\tPassThru " << pass_thru << '\n';
+	if (!escape_chars.empty())
+		os << "\tEscapeChars " << to_utf8(escape_chars) << '\n';
 	if (!pass_thru_chars.empty())
 		os << "\tPassThruChars " << to_utf8(pass_thru_chars) << '\n';
 	os << "\tParbreakIsNewline " << parbreak_is_newline << '\n';
@@ -1405,11 +1732,11 @@ void Layout::write(ostream & os) const
 	case Spacing::Default:
 		break;
 	}
-	if (!requires_.empty()) {
+	if (!required_.empty()) {
 		os << "\tRequires ";
-		for (set<string>::const_iterator it = requires_.begin();
-		     it != requires_.end(); ++it) {
-			if (it != requires_.begin())
+		for (set<string>::const_iterator it = required_.begin();
+		     it != required_.end(); ++it) {
+			if (it != required_.begin())
 				os << ',';
 			os << *it;
 		}
@@ -1426,15 +1753,31 @@ void Layout::write(ostream & os) const
 		os << "\n\tEndAutoNests\n";
 	}
 	if (!autonested_by_.empty()) {
-		os << "\tIsAutoNestedBy\n\t";
+		os << "\tIsAutoNestedBy\n\t\t";
 		for (set<docstring>::const_iterator it = autonested_by_.begin();
 		     it != autonested_by_.end(); ++it) {
 			if (it != autonested_by_.begin())
 				os << ',';
 			os << to_utf8(*it);
 		}
-		os << "\n\tIsAutoNestedBy\n";
+		os << "\n\tEndIsAutoNestedBy\n";
 	}
+	if (!thm_name_.empty())
+		os << "\tTheoremName " << thm_name_ << '\n';
+	if (!thm_latex_name_.empty())
+		os << "\tTheoremLaTeXName " << thm_latex_name_ << '\n';
+	if (!thm_counter_.empty())
+		os << "\tTheoremCounter " << thm_counter_ << '\n';
+	if (!thm_parent_counter_.empty())
+		os << "\tTheoremParentCounter " << thm_parent_counter_ << '\n';
+	if (!thm_style_.empty())
+		os << "\tTheoremStyle " << thm_style_ << '\n';
+	if (!thm_xref_name_.empty())
+		os << "\tTheoremCrossRefName " << thm_xref_name_ << '\n';
+	if (!thm_xref_name_pl_.empty())
+		os << "\tTheoremCrossRefPluralName " << thm_xref_name_pl_ << '\n';
+	if (!thm_zref_name_.empty())
+		os << "\tTheoremZRefName " << thm_zref_name_ << '\n';
 	if (refprefix.empty())
 		os << "\tRefPrefix OFF\n";
 	else
@@ -1443,6 +1786,10 @@ void Layout::write(ostream & os) const
 		os << "\tHTMLTag " << htmltag_ << '\n';
 	if (!htmlattr_.empty())
 		os << "\tHTMLAttr " << htmlattr_ << '\n';
+	if (!htmlclass_.empty())
+		os << "\tHTMLClass " << htmlclass_ << '\n';
+	if (!htmlintoc_)
+		os << "\tHTMLInToc " << htmlintoc_ << '\n';
 	if (!htmlitemtag_.empty())
 		os << "\tHTMLItem " << htmlitemtag_ << '\n';
 	if (!htmlitemattr_.empty())
@@ -1461,7 +1808,59 @@ void Layout::write(ostream & os) const
 		os << "\tHTMLPreamble\n"
 		   << to_utf8(rtrim(htmlpreamble_, "\n"))
 		   << "\n\tEndPreamble\n";
-	os << "\tHTMLTitle " << htmltitle_ << "\n"
+	os << "\tHTMLTitle " << htmltitle_ << "\n";
+	if (!docbooktag_.empty())
+		os << "\tDocBookTag " << docbooktag_ << '\n';
+	if (!docbookattr_.empty())
+		os << "\tDocBookAttr \"" << docbookattr_ << "\"\n";
+	if (!docbooktagtype_.empty())
+		os << "\tDocBookTagType " << docbooktagtype_ << '\n';
+	if (!docbookinnertag_.empty())
+		os << "\tDocBookInnerTag " << docbookinnertag_ << '\n';
+	if (!docbookinnerattr_.empty())
+		os << "\tDocBookInnerAttr \"" << docbookinnerattr_ << "\"\n";
+	if (!docbookinnertagtype_.empty())
+		os << "\tDocBookInnerTagType " << docbookinnertagtype_ << '\n';
+	if (!docbookininfo_.empty())
+		os << "\tDocBookInInfo " << docbookininfo_ << '\n';
+	os << "\tDocBookAbstract " << docbookabstract_ << '\n';
+	if (!docbookwrappertag_.empty())
+		os << "\tDocBookWrapperTag " << docbookwrappertag_ << '\n';
+	if (!docbookwrapperattr_.empty())
+		os << "\tDocBookWrapperAttr " << docbookwrapperattr_ << '\n';
+	if (!docbookwrappertagtype_.empty())
+		os << "\tDocBookWrapperTagType " << docbookwrappertagtype_ << '\n';
+	os << "\tDocBookSection " << docbooksection_ << '\n';
+	if (!docbooksectiontag_.empty())
+		os << "\tDocBookSectionTag " << docbooksectiontag_ << '\n';
+	if (!docbookitemtag_.empty())
+		os << "\tDocBookItemTag " << docbookitemtag_ << '\n';
+	if (!docbookitemattr_.empty())
+		os << "\tDocBookItemAttr " << docbookitemattr_ << '\n';
+	if (!docbookitemtagtype_.empty())
+		os << "\tDocBookItemTagType " << docbookitemtagtype_ << '\n';
+	if (!docbookitemwrappertag_.empty())
+		os << "\tDocBookItemWrapperTag " << docbookitemwrappertag_ << '\n';
+	if (!docbookitemwrapperattr_.empty())
+		os << "\tDocBookItemWrapperAttr " << docbookitemwrapperattr_ << '\n';
+	if (!docbookitemwrappertagtype_.empty())
+		os << "\tDocBookItemWrapperTagType " << docbookitemwrappertagtype_ << '\n';
+	os << "\tDocBookWrapperMergeWithPrevious " << docbookwrappermergewithprevious_ << '\n';
+	if (!docbookitemlabeltag_.empty())
+		os << "\tDocBookItemLabelTag " << docbookitemlabeltag_ << '\n';
+	if (!docbookitemlabelattr_.empty())
+		os << "\tDocBookItemLabelAttr " << docbookitemlabelattr_ << '\n';
+	if (!docbookitemlabeltagtype_.empty())
+		os << "\tDocBookItemLabelTagType " << docbookitemlabeltagtype_ << '\n';
+	if (!docbookiteminnertag_.empty())
+		os << "\tDocBookItemInnerTag " << docbookiteminnertag_ << '\n';
+	if (!docbookiteminnerattr_.empty())
+		os << "\tDocBookItemInnerAttr " << docbookiteminnerattr_ << '\n';
+	if (!docbookiteminnertagtype_.empty())
+		os << "\tDocBookItemInnerTagType " << docbookiteminnertagtype_ << '\n';
+	if (!docbookforceabstracttag_.empty())
+		os << "\tDocBookForceAbstractTag " << docbookforceabstracttag_ << '\n';
+    os << "\tDocBookNoFontInside " << docbooknofontinside_ << "\n"
 	      "\tSpellcheck " << spellcheck << "\n"
 	      "\tForceLocal " << forcelocal << "\n"
 	      "End\n";
@@ -1471,7 +1870,7 @@ void Layout::write(ostream & os) const
 bool Layout::hasArgs() const
 {
 	return !latexargs_.empty() || !postcommandargs_.empty() ||
-		!itemargs_.empty();
+		!itemargs_.empty() || !listpreamble_.empty();
 }
 
 
@@ -1480,6 +1879,8 @@ Layout::LaTeXArgMap Layout::args() const
 	LaTeXArgMap args = latexargs_;
 	if (!postcommandargs_.empty())
 		args.insert(postcommandargs_.begin(), postcommandargs_.end());
+	if (!listpreamble_.empty())
+		args.insert(listpreamble_.begin(), listpreamble_.end());
 	if (!itemargs_.empty())
 		args.insert(itemargs_.begin(), itemargs_.end());
 	return args;
@@ -1528,11 +1929,25 @@ string const & Layout::htmltag() const
 }
 
 
-string const & Layout::htmlattr() const
+string const & Layout::htmlclass() const
 {
-	if (htmlattr_.empty())
-		htmlattr_ = "class=\"" + defaultCSSClass() + "\"";
-	return htmlattr_;
+	// If it's an enumeration or itemize list, then we recalculate the class each
+	// time through (unless it has been given explicitly). So we do nothing here.
+	if (htmlclass_.empty() && labeltype != LABEL_ENUMERATE && labeltype != LABEL_ITEMIZE)
+		htmlclass_ = defaultCSSClass();
+	return htmlclass_;
+}
+
+
+string const & Layout::htmlGetAttrString() const {
+	if (!htmlfullattrs_.empty())
+		return htmlfullattrs_;
+	htmlfullattrs_ = htmlclass();
+	if (!htmlfullattrs_.empty())
+		htmlfullattrs_ = "class='" + htmlfullattrs_ + "'";
+	if (!htmlattr_.empty())
+		htmlfullattrs_ += " " + htmlattr_;
+	return htmlfullattrs_;
 }
 
 
@@ -1608,6 +2023,197 @@ string Layout::defaultCSSClass() const
 }
 
 
+string const & Layout::docbooktag() const
+{
+	if (docbooktag_.empty()) {
+		if (to_ascii(name_) == "Plain Layout")
+			docbooktag_ = "para";
+		else // No sensible default value, unhappily...
+			docbooktag_ = to_utf8(name_);
+	}
+	return docbooktag_;
+}
+
+
+string const & Layout::docbookattr() const
+{
+	// Perfectly OK to return no attributes, so docbookattr_ does not need to be filled.
+	return docbookattr_;
+}
+
+
+bool isValidTagType(std::string const & type)
+{
+	return !(type.empty() || (type != "block" && type != "paragraph" && type != "inline"));
+}
+
+
+string const & Layout::docbooktagtype() const
+{
+	if (!isValidTagType(docbooktagtype_))
+		docbooktagtype_ = "block";
+	return docbooktagtype_;
+}
+
+
+string const & Layout::docbookinnertag() const
+{
+	if (docbookinnertag_.empty())
+		docbookinnertag_ = "NONE";
+	return docbookinnertag_;
+}
+
+
+string const & Layout::docbookinnerattr() const
+{
+	return docbookinnerattr_;
+}
+
+
+string const & Layout::docbookinnertagtype() const
+{
+	if (!isValidTagType(docbookinnertagtype_))
+		docbookinnertagtype_ = "block";
+	return docbookinnertagtype_;
+}
+
+
+string const & Layout::docbookininfo() const
+{
+	// Indeed, a trilean. Only titles should be "maybe": otherwise, metadata is "always", content is "never". 
+	if (docbookininfo_.empty() || (docbookininfo_ != "never" && docbookininfo_ != "always" && docbookininfo_ != "maybe"))
+		docbookininfo_ = "never";
+	return docbookininfo_;
+}
+
+
+string const & Layout::docbookwrappertag() const
+{
+    if (docbookwrappertag_.empty())
+        docbookwrappertag_ = "NONE";
+    return docbookwrappertag_;
+}
+
+
+string const & Layout::docbookwrapperattr() const
+{
+    return docbookwrapperattr_;
+}
+
+
+string const & Layout::docbookwrappertagtype() const
+{
+	if (!isValidTagType(docbookwrappertagtype_))
+		docbookwrappertagtype_ = "block";
+	return docbookwrappertagtype_;
+}
+
+
+string const & Layout::docbooksectiontag() const
+{
+	if (docbooksectiontag_.empty())
+		docbooksectiontag_ = "section";
+	return docbooksectiontag_;
+}
+
+
+string const & Layout::docbookitemwrappertag() const
+{
+    if (docbookitemwrappertag_.empty())
+        docbookitemwrappertag_ = "NONE";
+    return docbookitemwrappertag_;
+}
+
+
+string const & Layout::docbookitemwrapperattr() const
+{
+    return docbookitemwrapperattr_;
+}
+
+
+string const & Layout::docbookitemwrappertagtype() const
+{
+	if (!isValidTagType(docbookitemwrappertagtype_))
+		docbookitemwrappertagtype_ = "block";
+	return docbookitemwrappertagtype_;
+}
+
+
+string const & Layout::docbookitemtag() const
+{
+	if (docbookitemtag_.empty())
+		docbookitemtag_ = "NONE";
+	return docbookitemtag_;
+}
+
+
+string const & Layout::docbookitemattr() const
+{
+    return docbookitemattr_;
+}
+
+
+string const & Layout::docbookitemtagtype() const
+{
+	if (!isValidTagType(docbookitemtagtype_))
+		docbookitemtagtype_ = "block";
+	return docbookitemtagtype_;
+}
+
+
+string const & Layout::docbookitemlabeltag() const
+{
+	if (docbookitemlabeltag_.empty())
+		docbookitemlabeltag_ = "NONE";
+	return docbookitemlabeltag_;
+}
+
+
+string const & Layout::docbookitemlabelattr() const
+{
+	return docbookitemlabelattr_;
+}
+
+
+string const & Layout::docbookitemlabeltagtype() const
+{
+	if (!isValidTagType(docbookitemlabeltagtype_))
+		docbookitemlabeltagtype_ = "block";
+	return docbookitemlabeltagtype_;
+}
+
+
+string const & Layout::docbookiteminnertag() const
+{
+	if (docbookiteminnertag_.empty())
+		docbookiteminnertag_ = "NONE";
+	return docbookiteminnertag_;
+}
+
+
+string const & Layout::docbookiteminnerattr() const
+{
+	return docbookiteminnerattr_;
+}
+
+
+string const & Layout::docbookiteminnertagtype() const
+{
+	if (!isValidTagType(docbookiteminnertagtype_))
+		docbookiteminnertagtype_ = "block";
+	return docbookiteminnertagtype_;
+}
+
+
+std::string const & Layout::docbookforceabstracttag() const
+{
+	if (docbookforceabstracttag_.empty())
+		docbookforceabstracttag_ = "NONE";
+	return docbookforceabstracttag_;
+}
+
+
+
 namespace {
 
 string makeMarginValue(char const * side, double d)
