@@ -14,8 +14,7 @@
 #ifndef LYX_H
 #define LYX_H
 
-#include "support/strfwd.h"
-
+#include <string>
 #include <vector>
 
 namespace lyx {
@@ -29,6 +28,7 @@ class Formats;
 class FuncRequest;
 class FuncStatus;
 class KeyMap;
+class LaTeXColors;
 class LaTeXFonts;
 class Messages;
 class Mover;
@@ -61,6 +61,21 @@ namespace frontend {
 class Application;
 }
 
+
+/// This chunk is above the LyX class so that
+/// we can use the [[noreturn]] property.
+/// (https://stackoverflow.com/a/79474684/1376404)
+///
+/// in the case of failure
+void emergencyCleanup();
+/// Try to exit LyX properly.
+/// \p exit_code is 0 by default, if a non zero value is passed,
+/// emergencyCleanup() will be called before exiting.
+[[noreturn]] void lyx_exit(int exit_code);
+/// Execute batch commands if available.
+void execBatchCommands();
+
+
 /// initial startup
 class LyX {
 	friend class LyXConsoleApp;
@@ -80,7 +95,7 @@ private:
 	void prepareExit();
 
 	/// Early exit during the initialisation process.
-	void earlyExit(int status);
+	[[noreturn]] void earlyExit(int status);
 
 	/// Initialise LyX and fills-in the vector of files to be loaded.
 	/**
@@ -150,6 +165,7 @@ private:
 	friend Movers & theSystemMovers();
 	friend frontend::Application * theApp();
 	friend Session & theSession();
+	friend LaTeXColors & theLaTeXColors();
 	friend LaTeXFonts & theLaTeXFonts();
 	friend CmdDef & theTopLevelCmdDef();
 	friend SpellChecker * theSpellChecker();
@@ -160,15 +176,6 @@ private:
 };
 
 
-/// in the case of failure
-void emergencyCleanup();
-/// Try to exit LyX properly.
-/// \p exit_code is 0 by default, if a non zero value is passed,
-/// emergencyCleanup() will be called before exiting.
-void lyx_exit(int exit_code);
-/// Execute batch commands if available.
-void execBatchCommands();
-
 ///
 FuncStatus getStatus(FuncRequest const & action);
 
