@@ -17,7 +17,6 @@
 #include "ColorCode.h"
 #include "FontInfo.h"
 
-#include "support/strfwd.h"
 #include "support/Changer.h"
 
 #include "insets/Inset.h"
@@ -29,6 +28,7 @@
 namespace lyx {
 
 namespace frontend { class Painter; }
+namespace frontend { class InputMethod; }
 class BufferView;
 class Length;
 class MacroContext;
@@ -40,7 +40,7 @@ class MacroContext;
 class MetricsBase {
 public:
 	///
-	MetricsBase(BufferView * bv = 0, FontInfo font = FontInfo(),
+	MetricsBase(BufferView * bv = 0, FontInfo const & font = FontInfo(),
 	            int textwidth = 0);
 
 	/// the current view
@@ -50,11 +50,11 @@ public:
 	/// font of the containing inset
 	FontInfo outer_font;
 	/// name of current font - mathed specific
-	std::string fontname;
+	std::string fontname = "mathnormal";
 	/// This is the width available in pixels
 	int textwidth;
 	/// count wether the current mathdata is nested in macro(s)
-	int macro_nesting;
+	int macro_nesting = 0;
 
 	/// Temporarily change a full font.
 	Changer changeFontSet(std::string const & name);
@@ -83,12 +83,11 @@ public:
 	int inPixels(Length const & len) const;
 
 private:
-	int solid_line_thickness_;
-	int solid_line_offset_;
-	int dotted_line_thickness_;
+	int solid_line_thickness_ = 1;
+	int solid_line_offset_ = 1;
+	int dotted_line_thickness_ = 1;
 };
 
-
 //
 // This contains a MetricsBase and information that's only relevant during
 // the first phase of the two-phase draw
@@ -106,11 +105,12 @@ public:
 	/// The context to resolve macros
 	MacroContext const & macrocontext;
 	/// Are we at the start of a paragraph (vertical mode)?
+	/// This is not used anymore, but could be useful
 	bool vmode;
 	/// if true, do not expand insets to max width artificially
 	bool tight_insets;
 	/// Extra width required by an inset, in addition to its dimension
-	int extrawidth;
+	int extrawidth = 0;
 };
 
 
@@ -126,6 +126,11 @@ public:
 	void draw(int x, int y, char_type c);
 	///
 	void draw(int x, int y, docstring const & str);
+	/// Draw preedit char using char_format_index for format
+	/// f is used to get the font size in the text mode of mathed
+	void draw(int x, int y, char_type c, frontend::InputMethod const * im,
+	          pos_type const char_format_index, FontInfo const * f = nullptr);
+
 	/// Determines the background color based on the
 	/// selection state, the background color inherited from the parent inset
 	/// and the inset's own background color (if one is specified).
@@ -142,22 +147,24 @@ public:
 	///
 	frontend::Painter & pain;
 	/// Whether the text at this point is right-to-left (for insets)
-	bool ltr_pos;
+	bool ltr_pos = false;
 	/// The change the parent is part of (change tracking)
 	Change change;
 	/// Whether the parent is selected as a whole
-	bool selected;
+	bool selected = false;
 	/// Whether the left/right margins are selected
-	bool selected_left, selected_right;
+	bool selected_left = false;
+	bool selected_right = false;
 	/// Whether the spell checker is enabled for the parent
-	bool do_spellcheck;
+	bool do_spellcheck = true;
 	/// True when it can be assumed that the screen has been cleared
-	bool full_repaint;
+	bool full_repaint = true;
 	/// Current background color
-	ColorCode background_color;
+	ColorCode background_color = Color_background;
 	/// The left and right position of current line (inside margins).
 	/// Useful for drawing display math numbering
-	int leftx, rightx;
+	int leftx = 0;
+	int rightx = 0;
 };
 
 class TextMetricsInfo {};
