@@ -14,8 +14,8 @@
 #include "PDFOptions.h"
 
 #include "Encoding.h"
+#include "LaTeXColors.h"
 #include "LaTeXFeatures.h"
-#include "Lexer.h"
 #include "OutputParams.h"
 #include "texstream.h"
 
@@ -23,10 +23,10 @@
 #include "support/debug.h"
 #include "support/docstream.h"
 #include "support/docstring.h"
+#include "support/Lexer.h"
 #include "support/lstrings.h"
 
 #include <sstream>
-#include <string>
 #include <algorithm>
 
 using namespace std;
@@ -304,4 +304,28 @@ void PDFOptions::clear()
 	pdfusetitle             = true;  //in contrast with hyperref
 }
 
+
+void PDFOptions::validate(LaTeXFeatures & features) const
+{
+	if (quoted_options.empty())
+		return;
+
+	vector<string> const opts = getVectorFromString(quoted_options);
+	for (auto const & opt : opts) {
+		if (!contains(opt, "color="))
+			continue;
+		string const color = split(opt, '=');
+		if (theLaTeXColors().isRealLaTeXColor(color)) {
+			string const lyxcolor = theLaTeXColors().getFromLaTeXColor(color);
+			LaTeXColor const lc = theLaTeXColors().getLaTeXColor(lyxcolor);
+			for (auto const & r : lc.req())
+				features.require(r);
+			features.require("xcolor");
+			if (!lc.model().empty())
+				features.require("xcolor:" + lc.model());
+		}
+	}
+	
+}
+
 } // namespace lyx
