@@ -125,14 +125,13 @@ void RowPainter::paintInset(Row::Element const & e) const
 	pi_.selected = pi_selected;
 
 #ifdef DEBUG_METRICS
-	Dimension const & dim = pi_.base.bv->coordCache().insets().dim(e.inset);
-	int const x2 = x1 + dim.wid;
-	int const y1 = yo_ + dim.des;
-	int const y2 = yo_ - dim.asc;
-	pi_.pain.line(x1, y1, x1, y2, Color_green);
-	pi_.pain.line(x1, y1, x2, y1, Color_green);
-	pi_.pain.line(x2, y1, x2, y2, Color_green);
-	pi_.pain.line(x1, y2, x2, y2, Color_green);
+	int const x2 = x1 + e.dim.wid;
+	int const y1 = yo_ + e.dim.des;
+	int const y2 = yo_ - e.dim.asc;
+	pi_.pain.line(x1, y1, x1, y2, Color_Green);
+	pi_.pain.line(x1, y1, x2, y1, Color_Green);
+	pi_.pain.line(x2, y1, x2, y2, Color_Green);
+	pi_.pain.line(x1, y2, x2, y2, Color_Green);
 #endif
 }
 
@@ -255,7 +254,10 @@ void RowPainter::paintStringAndSel(Row::Element const & e) const
 	bool const all_sel = (e.pos >= row_.sel_beg && e.endpos < row_.sel_end)
 		|| pi_.selected;
 
-	if (all_sel || e.change.changed()) {
+	if (e.type == Row::PREEDIT) {
+		// the case of the preedit element
+		pi_.pain.text(int(x_), yo_, e.str, e.im, e.char_format_index);
+	} else if (all_sel || e.change.changed()) {
 		Font copy = e.font;
 		Color const col = e.change.changed() ? e.change.color()
 		                                     : Color_selectiontext;
@@ -352,12 +354,12 @@ void RowPainter::paintDepthBar() const
 		int const starty = yo_ - row_.ascent();
 		int const h =  row_.height() - 1 - (i - next_depth - 1) * 3;
 
-		pi_.pain.line(x, starty, x, starty + h, Color_depthbar);
+		pi_.pain.line(x, starty + (i > prev_depth ? 1 : 0), x, starty + h, Color_depthbar);
 
 		if (i > prev_depth)
 			pi_.pain.fillRectangle(x, starty, w, 2, Color_depthbar);
 		if (i > next_depth)
-			pi_.pain.fillRectangle(x, starty + h, w, 2, Color_depthbar);
+			pi_.pain.fillRectangle(x, starty + h - 1, w, 2, Color_depthbar);
 	}
 }
 
@@ -567,6 +569,7 @@ void RowPainter::paintText()
 		switch (e.type) {
 		case Row::STRING:
 		case Row::VIRTUAL:
+		case Row::PREEDIT:
 			paintStringAndSel(e);
 
 			// Paint the spelling marks if enabled.
@@ -627,8 +630,8 @@ void RowPainter::paintSelection() const
 			|| pi_.selected;
 		// all the text selected?
 		bool const all_sel = (e.pos >= row_.sel_beg && e.endpos < row_.sel_end)
-		    || (e.isVirtual() && e.pos == row_.endpos() && row_.end_margin_sel)
-		    || pi_.selected;
+		        || (e.isVirtual() && e.pos == row_.endpos() && row_.end_margin_sel)
+		        || pi_.selected;
 
 		if (all_sel) {
 			// the 3rd argument is written like that to avoid rounding issues
