@@ -41,11 +41,12 @@
 #include "mathed/MacroTable.h"
 
 #include "frontends/FontMetrics.h"
+#include "frontends/InputMethod.h"
 #include "frontends/NullPainter.h"
 
+#include "support/Changer.h"
 #include "support/debug.h"
 #include "support/lassert.h"
-#include "support/Changer.h"
 
 #include <stdlib.h>
 #include <cmath>
@@ -106,9 +107,9 @@ int numberOfHfills(Row const & row, ParagraphMetrics const & pm,
 /////////////////////////////////////////////////////////////////////
 
 
-TextMetrics::TextMetrics(BufferView * bv, Text * text)
-	: bv_(bv), text_(text), dim_(bv_->workWidth(), 10, 10),
-	  max_width_(dim_.wid), tight_(false)
+TextMetrics::TextMetrics(BufferView * bv, Text const * text)
+    : bv_(bv), text_(text), im_(bv_->inputMethod()),
+      dim_(bv_->workWidth(), 10, 10), max_width_(dim_.wid)
 {}
 
 
@@ -163,14 +164,25 @@ void TextMetrics::setRowChanged(pit_type pit, pos_type pos)
 }
 
 
-ParagraphMetrics & TextMetrics::parMetrics(pit_type pit, bool redo)
+Dimension const & TextMetrics::dim(pit_type pit) const
 {
-	ParMetricsCache::iterator pmc_it = par_metrics_.find(pit);
+	auto pmc_it = par_metrics_.find(pit);
+	if (pmc_it == par_metrics_.end()) {
+		static Dimension empty_dim;
+		return empty_dim;
+	} else
+		return pmc_it->second.dim();
+}
+
+
+ParagraphMetrics & TextMetrics::parMetrics(pit_type pit)
+{
+	auto pmc_it = par_metrics_.find(pit);
 	if (pmc_it == par_metrics_.end()) {
 		pmc_it = par_metrics_.insert(
 			make_pair(pit, ParagraphMetrics(text_->getPar(pit)))).first;
 	}
-	if (pmc_it->second.rows().empty() && redo)
+	if (pmc_it->second.rows().empty())
 		redoParagraph(pit);
 	return pmc_it->second;
 }
@@ -178,13 +190,7 @@ ParagraphMetrics & TextMetrics::parMetrics(pit_type pit, bool redo)
 
 ParagraphMetrics const & TextMetrics::parMetrics(pit_type pit) const
 {
-	return const_cast<TextMetrics *>(this)->parMetrics(pit, true);
-}
-
-
-ParagraphMetrics & TextMetrics::parMetrics(pit_type pit)
-{
-	return parMetrics(pit, true);
+	return const_cast<TextMetrics *>(this)->parMetrics(pit);
 }
 
 
@@ -323,7 +329,7 @@ void TextMetrics::updatePosCache(pit_type pit) const
 {
 	frontend::NullPainter np;
 	PainterInfo pi(bv_, np);
-	drawParagraph(pi, pit, origin_.x_, par_metrics_[pit].position());
+	drawParagraph(pi, pit, origin_.x, par_metrics_[pit].position());
 }
 
 
@@ -483,141 +489,36 @@ bool TextMetrics::isRTLBoundary(pit_type pit, pos_type pos,
 
 bool TextMetrics::redoParagraph(pit_type const pit, bool const align_rows)
 {
-	Paragraph & par = text_->getPar(pit);
-	// IMPORTANT NOTE: We pass 'false' explicitly in order to not call
-	// redoParagraph() recursively inside parMetrics.
-	Dimension old_dim = parMetrics(pit, false).dim();
+	Paragraph const & par = text_->getPar(pit);
+	// This gets the dimension if it exists and an empty one otherwise.
+	Dimension old_dim = dim(pit);
 	ParagraphMetrics & pm = par_metrics_[pit];
 	pm.reset(par);
 
 	Buffer & buffer = bv_->buffer();
 	bool changed = false;
 
-	// Check whether there are InsetBibItems that need fixing
-	// FIXME: This check ought to be done somewhere else. It is the reason
-	// why text_ is not const. But then, where else to do it?
-	// Well, how can you end up with either (a) a biblio environment that
-	// has no InsetBibitem, (b) a biblio environment with more than one
-	// InsetBibitem or (c) a paragraph that has a bib item but is no biblio
-	// environment? I think the answer is: when paragraphs are merged;
-	// when layout is set; when material is pasted.
-	if (par.brokenBiblio()) {
-		Cursor & cur = bv_->cursor();
-		// In some cases, we do not know how to record undo
-		if (&cur.inset() == &text_->inset())
-			cur.recordUndo(pit, pit);
-
-		int const moveCursor = par.fixBiblio(buffer);
-
-		// Is it necessary to update the cursor?
-		if (&cur.inset() == &text_->inset() && cur.pit() == pit) {
-			if (moveCursor > 0)
-				cur.posForward();
-			else if (moveCursor < 0 && cur.pos() >= -moveCursor)
-				cur.posBackward();
-		}
-	}
-
 	// Optimisation: this is used in the next two loops
 	// so better to calculate that once here.
 	int const right_margin = rightMargin(pm);
 
-	// iterator pointing to paragraph to resolve macros
-	DocIterator parPos = text_->macrocontextPosition();
-	if (!parPos.empty())
-		parPos.pit() = pit;
-	else {
-		LYXERR(Debug::INFO, "MacroContext not initialised!"
-			<< " Going through the buffer again and hope"
-			<< " the context is better then.");
-		// FIXME audit updateBuffer calls
-		// This should not be here, but it is not clear yet where else it
-		// should be.
-		bv_->buffer().updateBuffer();
-		parPos = text_->macrocontextPosition();
-		LBUFERR(!parPos.empty());
-		parPos.pit() = pit;
-	}
-
-	// redo insets
-	par.setBeginOfBody();
-	Font const bufferfont = buffer.params().getFont();
-	CoordCache::Insets & insetCache = bv_->coordCache().insets();
-	map <Inset const *, int> extrawidths;
-	for (auto const & e : par.insetList()) {
-		// FIXME Doesn't this HAVE to be non-empty?
-		// position already initialized?
-		if (!parPos.empty()) {
-			parPos.pos() = e.pos;
-
-			// A macro template would normally not be visible
-			// by itself. But the tex macro semantics allow
-			// recursion, so we artifically take the context
-			// after the macro template to simulate this.
-			if (e.inset->lyxCode() == MATHMACRO_CODE)
-				parPos.pos()++;
-		}
-
-		// If there is an end of paragraph marker, its size should be
-		// substracted to the available width. The logic here is
-		// almost the same as in tokenizeParagraph, remember keep them in sync.
-		int eop = 0;
-		if (e.pos + 1 == par.size()
-		      && (lyxrc.paragraph_markers || par.lookupChange(par.size()).changed())
-		      && size_type(pit + 1) < text_->paragraphs().size()) {
-			Font f(text_->layoutFont(pit));
-			// ¶ U+00B6 PILCROW SIGN
-			eop = theFontMetrics(f).width(char_type(0x00B6));
-		}
-
-		// do the metric calculation
-		Dimension dim;
-		int const w = max_width_ - leftMargin(pit, e.pos)
-			- right_margin - eop;
-		Font const & font = e.inset->inheritFont() ?
-			displayFont(pit, e.pos) : bufferfont;
-		MacroContext mc(&buffer, parPos);
-		MetricsInfo mi(bv_, font.fontInfo(), w, mc, e.pos == 0, tight_);
-		mi.base.outer_font = displayFont(pit, e.pos).fontInfo();
-		e.inset->metrics(mi, dim);
-		/* FIXME: This is a hack. This allows InsetMathHull to state
-		 * that it needs some elbow room beyond its width, in order to
-		 * fit the numbering and/or the left margin (with left
-		 * alignment), which are outside of the inset itself.
-		 *
-		 * To this end, InsetMathHull::metrics() sets a value in
-		 * MetricsInfo::extrawidth and this value is added later to
-		 * the width of the row that contains the inset (when this row
-		 * is tight or shorter than the max allowed width).
-		 *
-		 * See ticket #12320 for details.
-		*/
-		extrawidths[e.inset] = mi.extrawidth;
-
-		if (!insetCache.has(e.inset) || insetCache.dim(e.inset) != dim) {
-			insetCache.add(e.inset, dim);
-			changed = true;
-		}
-	}
-
+	// FIXME: contents should not be modified, move elsewhere.
+	const_cast<Paragraph &>(par).setBeginOfBody();
 	// Transform the paragraph into a single row containing all the elements.
 	Row const bigrow = tokenizeParagraph(pit);
 	// Split the row in several rows fitting in available width
 	pm.rows() = breakParagraph(bigrow);
 
-	// Add the needed extra width to the rows that contain the insets that request it
-	for (Row & row : pm.rows())
-		for (Row::Element & e : row)
-			if (e.type == Row::INSET && (row.width() < max_width_ || tight_))
-				row.dim().wid += extrawidths[e.inset];
-
 	/* If there is more than one row, expand the text to the full
 	 * allowable width. This setting here is needed for the
 	 * setRowAlignment() below. We do nothing when tight insets are
-	 * requested.
+	 * requested or when the first row of two is empty due to
+	 * AlwaysBreakBefore flag.
 	 */
-	if (pm.rows().size() > 1 && !tight_ && dim_.wid < max_width_)
-			dim_.wid = max_width_;
+	if (!tight_ && dim_.wid < max_width_
+	    && (pm.rows().size() > 2
+	        || (pm.rows().size() == 2 && !pm.rows().front().empty())))
+		dim_.wid = max_width_;
 
 	// Compute height and alignment of the rows.
 	for (Row & row : pm.rows()) {
@@ -676,6 +577,9 @@ bool TextMetrics::redoParagraph(pit_type const pit, bool const align_rows)
 	pm.dim().asc += pm.rows().front().ascent();
 	pm.dim().des -= pm.rows().front().ascent();
 
+	// Tell the input method about pm
+	im_->setParagraphMetrics(pm);
+
 	changed |= old_dim.height() != pm.dim().height();
 
 	return changed;
@@ -709,7 +613,8 @@ LyXAlignment TextMetrics::getAlign(Paragraph const & par, Row const & row) const
 
 	// Display-style insets should always be on a centered row
 	if (Inset const * inset = par.getInset(row.pos())) {
-		if (inset->rowFlags() & Display) {
+		// If we are in empty row, alignment of inset does not apply (it is in next row)
+		if (!row.empty() && inset->rowFlags() & Display) {
 			if (inset->rowFlags() & AlignLeft)
 				align = LYX_ALIGN_LEFT;
 			else if (inset->rowFlags() & AlignRight)
@@ -726,7 +631,7 @@ LyXAlignment TextMetrics::getAlign(Paragraph const & par, Row const & row) const
 		// A forced block alignment can only be overridden the 'no
 		// justification on screen' setting.
 		if ((row.flushed() && !forced_block)
-		    || !bv_->buffer().params().justification)
+		    || !bv_->buffer().params().workareaJustified())
 			align = row.isRTL() ? LYX_ALIGN_RIGHT : LYX_ALIGN_LEFT;
 	}
 
@@ -854,7 +759,7 @@ void TextMetrics::setRowAlignment(Row & row, int width) const
 int TextMetrics::labelFill(Row const & row) const
 {
 	Paragraph const & par = text_->getPar(row.pit());
-	LBUFERR(par.beginOfBody() > 0 || par.isEnvSeparator(0));
+	LBUFERR(par.beginOfBody() > 0 || par.isEnvSeparator(0) || par.isNewline(0));
 
 	int w = 0;
 	// iterate over elements before main body (except the last one,
@@ -941,6 +846,7 @@ Row TextMetrics::tokenizeParagraph(pit_type const pit) const
 	row.pit(pit);
 	Paragraph const & par = text_->getPar(pit);
 	Buffer const & buf = text_->inset().buffer();
+	Cursor & cur = bv_->cursor();
 	BookmarksSection::BookmarkPosList bpl =
 		theSession().bookmarks().bookmarksInPar(buf.fileName(), par.id());
 
@@ -953,6 +859,8 @@ Row TextMetrics::tokenizeParagraph(pit_type const pit) const
 	if (ic_it.inTexted() && ic_it.text() == text_ && ic_it.pit() == pit)
 		ic_pos = ic_it.pos();
 
+	docstring & preedit = im_->preeditString();
+
 	// Now we iterate through until we reach the right margin
 	// or the end of the par, then build a representation of the row.
 	pos_type i = 0;
@@ -970,8 +878,25 @@ Row TextMetrics::tokenizeParagraph(pit_type const pit) const
 					row.addVirtual(i, docstring(1, ch), f, Change());
 				}
 
+		// the target is on the cursor and there are preedits to fill in
+		if (!preedit.empty() && i == cur.pos() && pit == cur.pit()) {
+
+			// the inset has a paragraph and the location matches
+			if (cur.inTexted() && &par.inInset() == &cur.paragraph().inInset()) {
+
+				// loop for differently presented segments of preedit string
+				// note that a sequence of preedit segments should have the
+				// same position since they have zero length
+				for (size_type j=0; j < im_->segmentSize(); j++) {
+					row.addPreedit(
+					    i, preedit.substr(im_->segmentStart(j), im_->segmentLength(j)),
+					    *fi, im_, (pos_type)j, Change());
+				}
+			}
+		}
+
 		// The stopping condition is here so that the display of a
-		// bookmark can take place at paragraph start too.
+		// bookmark or virtual preedits can take place at paragraph start too.
 		if (i >= end)
 			break;
 
@@ -979,8 +904,7 @@ Row TextMetrics::tokenizeParagraph(pit_type const pit) const
 		// The most special cases are handled first.
 		if (par.isInset(i)) {
 			Inset const * ins = par.getInset(i);
-			Dimension dim = bv_->coordCache().insets().dim(ins);
-			row.add(i, ins, dim, *fi, par.lookupChange(i));
+			row.addInset(i, ins, *fi, par.lookupChange(i));
 		} else if (c == ' ' && i + 1 == body_pos) {
 			// This space is an \item separator. Represent it with a
 			// special space element, which dimension will be computed
@@ -1006,10 +930,10 @@ Row TextMetrics::tokenizeParagraph(pit_type const pit) const
 			// ⤶ U+2936 ARROW POINTING DOWNWARDS THEN CURVING LEFTWARDS
 			// ¶ U+00B6 PILCROW SIGN
 			char_type const screen_char = (c == 0x2028) ? 0x2936 : 0x00B6;
-			row.add(i, screen_char, *fi, par.lookupChange(i));
+			row.addChar(i, screen_char, *fi, par.lookupChange(i));
 		} else
 			// row elements before body are unbreakable
-			row.add(i, c, *fi, par.lookupChange(i));
+			row.addChar(i, c, *fi, par.lookupChange(i));
 
 		// add inline completion width
 		// draw logically behind the previous character
@@ -1119,13 +1043,13 @@ bool operator==(flexible_const_iterator<T> const & t1,
 	return t1.cit_ == t2.cit_ && t1.pile_.empty() && t2.pile_.empty();
 }
 
-
-Row newRow(TextMetrics const & tm, pit_type pit, pos_type pos, bool is_rtl)
+Row newRow(TextMetrics const & tm, pit_type pit, pos_type pos, bool is_rtl,
+           size_type shift = 0)
 {
 	Row nrow;
 	nrow.pit(pit);
 	nrow.pos(pos);
-	nrow.left_margin = tm.leftMargin(pit, pos);
+	nrow.left_margin = tm.leftMargin(pit, pos + shift);
 	nrow.right_margin = tm.rightMargin(pit);
 	nrow.setRTL(is_rtl);
 	if (is_rtl)
@@ -1149,7 +1073,9 @@ void cleanupRow(Row & row, bool at_end)
 	if (!at_end && !row.flushed())
 		row.back().rtrim();
 	// boundary exists when there was no space at the end of row
-	row.end_boundary(!at_end && row.back().endpos == row.endpos());
+	row.end_boundary(!at_end
+	                 && row.back().endpos == row.endpos()
+	                 && !(row.back().row_flags & NoEndBoundary));
 	// make sure that the RTL elements are in reverse ordering
 	row.reverseRTL();
 }
@@ -1158,7 +1084,7 @@ void cleanupRow(Row & row, bool at_end)
 // Implement the priorities described in RowFlags.h.
 bool needsRowBreak(int f1, int f2)
 {
-	if (f1 & AlwaysBreakAfter /*|| f2 & AlwaysBreakBefore*/)
+	if (f1 & AlwaysBreakAfter || f2 & AlwaysBreakBefore)
 		return true;
 	if (f1 & NoBreakAfter || f2 & NoBreakBefore)
 		return false;
@@ -1171,13 +1097,70 @@ bool needsRowBreak(int f1, int f2)
 }
 
 
-RowList TextMetrics::breakParagraph(Row const & bigrow) const
+int TextMetrics::redoInset(Row::Element & elt, DocIterator & parPos, int w) const
 {
-	RowList rows;
+	Buffer const & buffer = bv_->buffer();
+	Font const bufferfont = buffer.params().getFont();
+	parPos.pos() = elt.pos;
+	// A macro template would normally not be visible by itself.
+	// But the tex macro semantics allow recursion, so we
+	// artifically take the context after the macro template to
+	// simulate this.
+	if (elt.inset->lyxCode() == MATH_MACROTEMPLATE_CODE)
+		parPos.pos()++;
+
+	// do the metric calculation
+	Font const & font = elt.inset->inheritFont() ?
+	                    displayFont(parPos.pit(), elt.pos) : bufferfont;
+	MacroContext const mc(&buffer, parPos);
+	MetricsInfo mi(bv_, font.fontInfo(), w, mc, elt.pos == 0, tight_);
+	mi.base.outer_font = displayFont(parPos.pit(), elt.pos).fontInfo();
+	elt.inset->metrics(mi, elt.dim);
+	bv_->coordCache().insets().add(elt.inset, elt.dim);
+	/* FIXME: This is a hack. This allows InsetMathHull to state that
+	 * it needs some elbow room beyond its width, in order to fit the
+	 * numbering and/or the left margin (with left alignment), which
+	 * are outside of the inset itself.
+	 *
+	 * To this end, InsetMathHull::metrics() sets a value in
+	 * MetricsInfo::extrawidth and this value is added later to the
+	 * width of the row that contains the inset (when this row is
+	 * tight or shorter than the max allowed width).
+	 *
+	 * See ticket #12320 for details.
+	 */
+	return mi.extrawidth;
+}
+
+
+Rows TextMetrics::breakParagraph(Row const & bigrow) const
+{
+	Rows rows;
 	bool const is_rtl = text_->isRTL(bigrow.pit());
 	bool const end_label = text_->getEndLabel(bigrow.pit()) != END_LABEL_NO_LABEL;
-	int const next_width = max_width_ - leftMargin(bigrow.pit(), bigrow.endpos())
+	pos_type const bigrow_endpos =
+	        !bigrow.empty() && bigrow.back().type == Row::PREEDIT ?
+	            bigrow.endpos() + im_->preeditString().length() :
+	            bigrow.endpos();
+	int const next_width = max_width_ - leftMargin(bigrow.pit(), bigrow_endpos)
 		- rightMargin(bigrow.pit());
+	pit_type const pit = bigrow.pit();
+	Paragraph const & par = text_->getPar(pit);
+
+	// iterator pointing to paragraph to resolve macros
+	DocIterator parPos = text_->macrocontextPosition();
+	if (parPos.empty()) {
+		LYXERR0("MacroContext not initialised!"
+			<< " Going through the buffer again and hope"
+			<< " the context is better then. Please report");
+		// FIXME audit updateBuffer calls
+		// This should not be here, but it is not clear yet where else it
+		// should be.
+		bv_->buffer().updateBuffer();
+		parPos = text_->macrocontextPosition();
+		LBUFERR(!parPos.empty());
+	}
+	parPos.pit() = pit;
 
 	int width = 0;
 	flexible_const_iterator<Row> fcit = flexible_begin(bigrow);
@@ -1186,13 +1169,21 @@ RowList TextMetrics::breakParagraph(Row const & bigrow) const
 		bool const row_empty = rows.empty() || rows.back().empty();
 		// The row flags of previous element, if there is one.
 		// Otherwise we use NoBreakAfter to avoid an empty row before
-		// e.g. a displayed equation.
+		// e.g. a displayed inset.
 		int const f1 = row_empty ? NoBreakAfter : rows.back().back().row_flags;
 		// The row flags of next element, if there is one.
 		// Otherwise we use NoBreakBefore (see above), unless the
 		// paragraph has an end label (for which an empty row is OK).
 		int const f2 = (fcit == end) ? (end_label ? Inline : NoBreakBefore)
 		                             : fcit->row_flags;
+		if (rows.empty() && needsRowBreak(f1, f2)) {
+			// Create an empty row before element
+			rows.push_back(newRow(*this, pit, 0, is_rtl));
+			Row & newrow = rows.back();
+			cleanupRow(newrow, false);
+			newrow.end_boundary(true);
+			newrow.left_margin = leftMargin(newrow.pit(), 0, true);
+		}
 		if (rows.empty() || needsRowBreak(f1, f2)) {
 			if (!rows.empty()) {
 				// Flush row as requested by row flags
@@ -1200,7 +1191,13 @@ RowList TextMetrics::breakParagraph(Row const & bigrow) const
 				cleanupRow(rows.back(), false);
 			}
 			pos_type pos = rows.empty() ? 0 : rows.back().endpos();
-			rows.push_back(newRow(*this, bigrow.pit(), pos, is_rtl));
+			if (!rows.empty() && !rows.back().empty()
+			      && rows.back().back().type == Row::PREEDIT) {
+				size_type const shift = rows.back().back().str.length();
+				rows.push_back(newRow(*this, pit, pos, is_rtl, shift));
+			} else
+				rows.push_back(newRow(*this, pit, pos, is_rtl));
+
 			// the width available for the row.
 			width = max_width_ - rows.back().right_margin;
 		}
@@ -1214,10 +1211,32 @@ RowList TextMetrics::breakParagraph(Row const & bigrow) const
 		// pile, or the place when we were in main row
 		Row::Element elt = *fcit;
 		Row::Elements tail;
-		elt.splitAt(width - rows.back().width(), next_width, Row::FIT, tail);
 		Row & rb = rows.back();
-		if (elt.type == Row::MARGINSPACE)
-			elt.dim.wid = max(elt.dim.wid, leftMargin(bigrow.pit()) - rb.width());
+
+		if (elt.type == Row::INSET) {
+			parPos.pos() = elt.pos;
+			// If there is an end of paragraph marker, its size should be
+			// substracted to the available width. The logic here is
+			// almost the same as in tokenizeParagraph, remember keep them in sync.
+			int eop = 0;
+			if (elt.pos + 1 == par.size()
+				&& (lyxrc.paragraph_markers || par.lookupChange(par.size()).changed())
+				&& size_type(pit + 1) < text_->paragraphs().size()) {
+				Font f(text_->layoutFont(pit));
+				// ¶ U+00B6 PILCROW SIGN
+				eop = theFontMetrics(f).width(char_type(0x00B6));
+			}
+			int const w = max_width_ - leftMargin(pit, elt.pos)
+				- rightMargin(pit) - eop;
+			int const extrawidth = redoInset(elt, parPos, w);
+			// For now only hull inset sets this, and it is alone on its row
+			if (elt.dim.wid < max_width_ || tight_)
+				rb.dim().wid += extrawidth;
+		} else if (elt.type == Row::MARGINSPACE)
+			elt.dim.wid = max(elt.dim.wid, leftMargin(pit) - rb.width());
+		else
+			elt.splitAt(width - rows.back().width(), next_width, Row::FIT, tail);
+
 		rb.push_back(elt);
 		rb.finalizeLast();
 		if (rb.width() > width) {
@@ -1244,6 +1263,13 @@ RowList TextMetrics::breakParagraph(Row const & bigrow) const
 			rows.back().needsChangeBar(true);
 	}
 
+	// Set start_boundary to be equal to the previous row's end boundary
+	bool sb = false;
+	for (auto & row : rows) {
+		row.start_boundary(sb);
+		sb = row.end_boundary();
+	}
+
 	return rows;
 }
 
@@ -1382,10 +1408,11 @@ void TextMetrics::setRowHeight(Row & row) const
 	if (row.pos() == 0 && layout.labelIsInline()) {
 		FontInfo const labelfont = text_->labelFont(par);
 		FontMetrics const & lfm = theFontMetrics(labelfont);
-		maxasc = max(maxasc, int(lfm.maxAscent()
+		Dimension const ldim = lfm.dimension(par.labelString());
+		maxasc = max(maxasc, int(ldim.ascent()
 			// add leading space
 			+ lfm.maxHeight() * (spacing_val - 1)));
-		maxdes = max(maxdes, int(lfm.maxDescent()));
+		maxdes = max(maxdes, int(ldim.descent()));
 	}
 
 	// Find the ascent/descent of the row contents
@@ -1395,10 +1422,11 @@ void TextMetrics::setRowHeight(Row & row) const
 			maxdes = max(maxdes, e.dim.descent());
 		} else {
 			FontMetrics const & fm2 = theFontMetrics(e.font);
-			maxasc = max(maxasc, int(fm2.maxAscent()
+			maxasc = max(maxasc, e.dim.ascent()
+				// FIXME: is this right (or shall we use a real height) ??
 				// add leading space
-				+ fm2.maxHeight() * (spacing_val - 1)));
-			maxdes = max(maxdes, int(fm2.maxDescent()));
+				+ int(fm2.maxHeight() * (spacing_val - 1)));
+			maxdes = max(maxdes, e.dim.descent());
 		}
 	}
 
@@ -1417,108 +1445,33 @@ void TextMetrics::setRowHeight(Row & row) const
 // x is an absolute screen coord
 // returns the column near the specified x-coordinate of the row
 // x is set to the real beginning of this column
-pos_type TextMetrics::getPosNearX(Row const & row, int & x,
-				  bool & boundary) const
+pair<pos_type, bool> TextMetrics::getPosNearX(Row const & row, int & x) const
 {
 	//LYXERR0("getPosNearX(" << x << ") row=" << row);
 	/// For the main Text, it is possible that this pit is not
 	/// yet in the CoordCache when moving cursor up.
 	/// x Paragraph coordinate is always 0 for main text anyway.
-	int const xo = origin_.x_;
-	x -= xo;
 
 	// Adapt to cursor row scroll offset if applicable.
 	int const offset = bv_->horizScrollOffset(text_, row.pit(), row.pos());
-	x += offset;
 
-	pos_type pos = row.pos();
-	boundary = false;
-	if (row.empty())
-		x = row.left_margin;
-	else if (x <= row.left_margin) {
-		pos = row.front().left_pos();
-		x = row.left_margin;
-	} else if (x >= row.width()) {
-		pos = row.back().right_pos();
-		x = row.width();
-	} else {
-		double w = row.left_margin;
-		Row::const_iterator cit = row.begin();
-		Row::const_iterator cend = row.end();
-		for ( ; cit != cend; ++cit) {
-			if (w <= x &&  w + cit->full_width() > x) {
-				int x_offset = int(x - w);
-				pos = cit->x2pos(x_offset);
-				x = int(x_offset + w);
-				break;
-			}
-			w += cit->full_width();
-		}
-		if (cit == row.end()) {
-			pos = row.back().right_pos();
-			x = row.width();
-		}
-		/** This tests for the case where the cursor is placed
-		 * just before a font direction change. See comment on
-		 * the boundary_ member in DocIterator.h to understand
-		 * how boundary helps here.
-		 */
-		else if (pos == cit->endpos
-		         && ((!cit->isRTL() && cit + 1 != row.end()
-		              && (cit + 1)->isRTL())
-		             || (cit->isRTL() && cit != row.begin()
-		                 && !(cit - 1)->isRTL())))
-			boundary = true;
-	}
+	x += offset - origin_.x;
+	auto [pos, boundary] = row.x2pos(x);
+	x -= offset - origin_.x;
 
-	/** This tests for the case where the cursor is set at the end
-	 * of a row which has been broken due something else than a
-	 * separator (a display inset or a forced breaking of the
-	 * row). We know that there is a separator when the end of the
-	 * row is larger than the end of its last element.
-	 */
-	if (!row.empty() && pos == row.back().endpos
-	    && row.back().endpos == row.endpos()) {
-		Inset const * inset = row.back().inset;
-		if (inset && (inset->lyxCode() == NEWLINE_CODE
-		              || inset->lyxCode() == SEPARATOR_CODE))
-			pos = row.back().pos;
-		else
-			boundary = row.end_boundary();
-	}
-
-	x += xo - offset;
 	//LYXERR0("getPosNearX ==> pos=" << pos << ", boundary=" << boundary);
-
-	return pos;
-}
-
-
-pos_type TextMetrics::x2pos(pit_type pit, int row, int x) const
-{
-	// We play safe and use parMetrics(pit) to make sure the
-	// ParagraphMetrics will be redone and OK to use if needed.
-	// Otherwise we would use an empty ParagraphMetrics in
-	// upDownInText() while in selection mode.
-	ParagraphMetrics const & pm = parMetrics(pit);
-
-	LBUFERR(row < int(pm.rows().size()));
-	bool bound = false;
-	Row const & r = pm.rows()[row];
-	return getPosNearX(r, x, bound);
+	return make_pair(pos, boundary);
 }
 
 
 // y is screen coordinate
-pit_type TextMetrics::getPitNearY(int y)
+pit_type TextMetrics::getPitNearY(int y, bool strict)
 {
 	LASSERT(!text_->paragraphs().empty(), return -1);
 	LASSERT(!par_metrics_.empty(), return -1);
-	LYXERR(Debug::PAINTING, "y: " << y << " cache size: " << par_metrics_.size());
 
 	// look for highest numbered paragraph with y coordinate less than given y
 	pit_type pit = -1;
-	int yy = -1;
 	ParMetricsCache::const_iterator it = par_metrics_.begin();
 	ParMetricsCache::const_iterator et = par_metrics_.end();
 	ParMetricsCache::const_iterator last = et;
@@ -1526,11 +1479,11 @@ pit_type TextMetrics::getPitNearY(int y)
 
 	if (y < it->second.top()) {
 		// We are looking for a position that is before the first paragraph in
-		// the cache (which is in priciple off-screen, that is before the
-		// visible part.
+		// the cache (which is in principle off-screen, that is before the
+		// visible part).
 		if (it->first == 0)
 			// We are already at the first paragraph in the inset.
-			return 0;
+			return strict ? -1 : 0;
 		// OK, this is the paragraph we are looking for.
 		pit = it->first - 1;
 		newParMetricsUp();
@@ -1539,105 +1492,68 @@ pit_type TextMetrics::getPitNearY(int y)
 
 	if (y >= par_metrics_[last->first].bottom()) {
 		// We are looking for a position that is after the last paragraph in
-		// the cache (which is in priciple off-screen), that is before the
-		// visible part.
+		// the cache (which is in principle off-screen, that is after the
+		// visible part).
 		pit = last->first + 1;
 		if (pit == int(text_->paragraphs().size()))
 			//  We are already at the last paragraph in the inset.
-			return last->first;
+			return strict ? -1 : last->first;
 		// OK, this is the paragraph we are looking for.
 		newParMetricsDown();
 		return pit;
 	}
 
 	for (; it != et; ++it) {
-		LYXERR(Debug::PAINTING, "examining: pit: " << it->first
-			<< " y: " << it->second.position());
-
-		if (it->first >= pit && it->second.top() <= y) {
+		if (it->first >= pit && it->second.top() <= y)
 			pit = it->first;
-			yy = it->second.position();
-		}
 	}
 
-	LYXERR(Debug::PAINTING, "found best y: " << yy << " for pit: " << pit);
-
 	return pit;
 }
 
 
-Row const & TextMetrics::getPitAndRowNearY(int & y, pit_type & pit,
-	bool assert_in_view, bool up)
+Row const * TextMetrics::getRowNearY(int y, bool strict)
 {
+	pit_type const pit = getPitNearY(y, strict);
+	if (pit == -1)
+		return nullptr;
 	ParagraphMetrics const & pm = par_metrics_[pit];
 
 	int yy = pm.top();
 	LBUFERR(!pm.rows().empty());
-	RowList::const_iterator rit = pm.rows().begin();
-	RowList::const_iterator rlast = pm.rows().end();
+	Rows::const_iterator rit = pm.rows().begin();
+	Rows::const_iterator rlast = pm.rows().end();
 	--rlast;
 	for (; rit != rlast; yy += rit->height(), ++rit)
 		if (yy + rit->height() > y)
 			break;
 
-	if (assert_in_view) {
-		if (!up && yy + rit->height() > y) {
-			if (rit != pm.rows().begin()) {
-				y = yy;
-				--rit;
-			} else if (pit != 0) {
-				--pit;
-				newParMetricsUp();
-				ParagraphMetrics const & pm2 = par_metrics_[pit];
-				rit = pm2.rows().end();
-				--rit;
-				y = yy;
-			}
-		} else if (up && yy != y) {
-			if (rit != rlast) {
-				y = yy + rit->height();
-				++rit;
-			} else if (pit < int(text_->paragraphs().size()) - 1) {
-				++pit;
-				newParMetricsDown();
-				ParagraphMetrics const & pm2 = par_metrics_[pit];
-				rit = pm2.rows().begin();
-				y = pm2.position();
-			}
-		}
-	}
-	return *rit;
+	return &(*rit);
 }
 
 
 // x,y are absolute screen coordinates
 // sets cursor recursively descending into nested editable insets
-Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
-	bool assert_in_view, bool up)
+Inset * TextMetrics::editXY(Cursor & cur, int x, int y)
 {
-	if (lyxerr.debugging(Debug::WORKAREA)) {
-		LYXERR0("TextMetrics::editXY(cur, " << x << ", " << y << ")");
-		cur.bv().coordCache().dump();
-	}
-	pit_type pit = getPitNearY(y);
-	LASSERT(pit != -1, return 0);
-	Row const & row = getPitAndRowNearY(y, pit, assert_in_view, up);
-	cur.pit() = pit;
+	Row const * row = getRowNearY(y);
+	LASSERT(row != nullptr, return nullptr);
+	cur.pit() = row->pit();
 
 	// Do we cover an inset?
-	InsetList::Element * e = checkInsetHit(pit, x, y);
+	Row::Element const * e = checkInsetHit(*row, x);
 
 	if (!e) {
 		// No inset, set position in the text
-		bool bound = false; // is modified by getPosNearX
-		cur.pos() = getPosNearX(row, x, bound);
+		auto [pos, bound] = getPosNearX(*row, x);
+		cur.pos() = pos;
 		cur.boundary(bound);
 		cur.setCurrentFont();
 		cur.setTargetX(x);
 		return 0;
 	}
 
-	Inset * inset = e->inset;
+	Inset * inset = const_cast<Inset *>(e->inset);
 	//lyxerr << "inset " << inset << " hit at x: " << x << " y: " << y << endl;
 
 	// Set position in front of inset
@@ -1652,8 +1568,9 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 	if (cur.text() == text_ && cur.pos() == e->pos) {
 		// non-editable inset, set cursor after the inset if x is
 		// nearer to that position (bug 9628)
-		bool bound = false; // is modified by getPosNearX
-		cur.pos() = getPosNearX(row, x, bound);
+		// No inset, set position in the text
+		auto [pos, bound] = getPosNearX(*row, x);
+		cur.pos() = pos;
 		cur.boundary(bound);
 		cur.setCurrentFont();
 		cur.setTargetX(x);
@@ -1665,61 +1582,35 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 }
 
 
-void TextMetrics::setCursorFromCoordinates(Cursor & cur, int const x, int const y)
+void TextMetrics::setCursorFromCoordinates(Cursor & cur, int x, int y)
 {
 	LASSERT(text_ == cur.text(), return);
-	pit_type const pit = getPitNearY(y);
-	LASSERT(pit != -1, return);
 
-	ParagraphMetrics const & pm = par_metrics_[pit];
-
-	int yy = pm.position() - pm.rows().front().ascent();
-	LYXERR(Debug::PAINTING, "x: " << x << " y: " << y <<
-		" pit: " << pit << " yy: " << yy);
-
-	int r = 0;
-	LBUFERR(pm.rows().size());
-	for (; r < int(pm.rows().size()) - 1; ++r) {
-		Row const & row = pm.rows()[r];
-		if (yy + row.height() > y)
-			break;
-		yy += row.height();
-	}
-
-	Row const & row = pm.rows()[r];
-
-	LYXERR(Debug::PAINTING, "row " << r << " from pos: " << row.pos());
-
-	bool bound = false;
-	int xx = x;
-	pos_type const pos = getPosNearX(row, xx, bound);
-
-	LYXERR(Debug::PAINTING, "setting cursor pit: " << pit << " pos: " << pos);
-
-	text_->setCursor(cur, pit, pos, true, bound);
+	Row const * row = getRowNearY(y);
+	LASSERT(row != nullptr, return);
+	auto [pos, bound] = getPosNearX(*row, x);
+	cur.text()->setCursor(cur, row->pit(), pos, true, bound);
 	// remember new position.
 	cur.setTargetX();
 }
 
 
-//takes screen x,y coordinates
-InsetList::Element * TextMetrics::checkInsetHit(pit_type pit, int x, int y)
+// takes a row and x screen coordinate
+Row::Element const * TextMetrics::checkInsetHit(Row const & row, int x) const
 {
-	Paragraph const & par = text_->paragraphs()[pit];
-	CoordCache::Insets const & insetCache = bv_->coordCache().getInsets();
+	// Adapt to cursor row scroll offset if applicable.
+	int const offset = bv_->horizScrollOffset(text_, row.pit(), row.pos());
+	x += offset - origin_.x;
 
-	LYXERR(Debug::PAINTING, "x: " << x << " y: " << y << "  pit: " << pit);
-
-	for (InsetList::Element const & e : par.insetList()) {
-		LYXERR(Debug::PAINTING, "examining inset " << e.inset);
-
-		if (insetCache.covers(e.inset, x, y)) {
-			LYXERR(Debug::PAINTING, "Hit inset: " << e.inset);
-			return const_cast<InsetList::Element *>(&e);
-		}
+	int xx = row.left_margin;
+	for (auto const & e : row) {
+		if (xx > x)
+			break;
+		if (xx + e.full_width() > x)
+			return (e.type == Row::INSET) ? &e : nullptr;
+		xx += e.full_width();
 	}
 
-	LYXERR(Debug::PAINTING, "No inset hit. ");
 	return nullptr;
 }
 
@@ -1727,14 +1618,12 @@ InsetList::Element * TextMetrics::checkInsetHit(pit_type pit, int x, int y)
 //takes screen x,y coordinates
 Inset * TextMetrics::checkInsetHit(int x, int y)
 {
-	pit_type const pit = getPitNearY(y);
-	LASSERT(pit != -1, return 0);
-	InsetList::Element * e = checkInsetHit(pit, x, y);
+	Row const * row = getRowNearY(y, true);
+	if (row == nullptr)
+		return nullptr;
+	Row::Element const * e = checkInsetHit(*row, x);
 
-	if (!e)
-		return 0;
-
-	return e->inset;
+	return e ? const_cast<Inset *>(e->inset) : nullptr;
 }
 
 
@@ -1747,12 +1636,7 @@ int TextMetrics::cursorX(CursorSlice const & sl,
 	if (pm.rows().empty())
 		return 0;
 	Row const & row = pm.getRow(sl.pos(), boundary);
-	pos_type const pos = sl.pos();
-
-	double x = 0;
-	row.findElement(pos, boundary, x);
-	return int(x);
-
+	return row.pos2x(sl.pos(), boundary);
 }
 
 
@@ -1765,13 +1649,10 @@ int TextMetrics::cursorY(CursorSlice const & sl, bool boundary) const
 
 	int h = 0;
 	h -= parMetrics(0).rows()[0].ascent();
-	for (pit_type pit = 0; pit < sl.pit(); ++pit) {
+	for (pit_type pit = 0; pit < sl.pit(); ++pit)
 		h += parMetrics(pit).height();
-	}
-	int pos = sl.pos();
-	if (pos && boundary)
-		--pos;
-	size_t const rend = pm.pos2row(pos);
+
+	size_t const rend = pm.getRowIndex(sl.pos(), boundary);
 	for (size_t rit = 0; rit != rend; ++rit)
 		h += pm.rows()[rit].height();
 	h += pm.rows()[rend].ascent();
@@ -1787,7 +1668,7 @@ bool TextMetrics::cursorHome(Cursor & cur)
 	LASSERT(text_ == cur.text(), return false);
 	ParagraphMetrics const & pm = par_metrics_[cur.pit()];
 	Row const & row = pm.getRow(cur.pos(),cur.boundary());
-	return text_->setCursor(cur, cur.pit(), row.pos());
+	return cur.text()->setCursor(cur, cur.pit(), row.pos());
 }
 
 
@@ -1811,7 +1692,7 @@ bool TextMetrics::cursorEnd(Cursor & cur)
 			--end;
 	} else if (cur.paragraph().isEnvSeparator(end-1))
 		--end;
-	return text_->setCursor(cur, cur.pit(), end, true, boundary);
+	return cur.text()->setCursor(cur, cur.pit(), end, true, boundary);
 }
 
 
@@ -1820,7 +1701,7 @@ void TextMetrics::deleteLineForward(Cursor & cur)
 	LASSERT(text_ == cur.text(), return);
 	if (cur.lastpos() == 0) {
 		// Paragraph is empty, so we just go forward
-		text_->cursorForward(cur);
+		cur.text()->cursorForward(cur);
 	} else {
 		cur.resetAnchor();
 		cur.selection(true); // to avoid deletion
@@ -1828,7 +1709,7 @@ void TextMetrics::deleteLineForward(Cursor & cur)
 		cur.setSelection();
 		// What is this test for ??? (JMarc)
 		if (!cur.selection())
-			text_->deleteWordForward(cur);
+			cur.text()->deleteWordForward(cur);
 		else
 			cap::cutSelection(cur, false);
 		cur.checkBufferStructure();
@@ -1844,7 +1725,7 @@ int TextMetrics::leftMargin(pit_type pit) const
 }
 
 
-int TextMetrics::leftMargin(pit_type const pit, pos_type const pos) const
+int TextMetrics::leftMargin(pit_type const pit, pos_type const pos, bool ignore_contents) const
 {
 	ParagraphList const & pars = text_->paragraphs();
 
@@ -2004,7 +1885,8 @@ int TextMetrics::leftMargin(pit_type const pit, pos_type const pos) const
 	    // in some insets, paragraphs are never indented
 	    && !text_->inset().neverIndent()
 	    // display style insets do not need indentation
-	    && !(!par.empty()
+	    && !(!ignore_contents
+	         && !par.empty()
 	         && par.isInset(0)
 	         && par.getInset(0)->rowFlags() & Display)
 	    && (!(tclass.isDefaultLayout(par.layout())
@@ -2030,8 +1912,8 @@ void TextMetrics::draw(PainterInfo & pi, int x, int y) const
 	if (par_metrics_.empty())
 		return;
 
-	origin_.x_ = x;
-	origin_.y_ = y;
+	origin_.x = x;
+	origin_.y = y;
 
 	y -= par_metrics_.begin()->second.ascent();
 	for (auto & pm_pair : par_metrics_) {
@@ -2179,9 +2061,9 @@ void TextMetrics::drawParagraph(PainterInfo & pi, pit_type const pit, int const
 		// 12 lines lower):
 		if (lyxerr.debugging(Debug::PAINTING)
 		    && (row.selection() || pi.full_repaint || row.changed())) {
-			string const foreword = text_->isMainText() ? "main text redraw "
-				: "inset text redraw: ";
-			LYXERR0(foreword << "pit=" << pit << " row=" << i
+			string const foreword = text_->isMainText() ? "main  text redraw "
+				: "inset text redraw ";
+			LYXERR0(foreword << "@(" << row_x << "," << y << ") " << "pit=" << pit << " row=" << i
 			        << (row.selection() ? " row_selection": "")
 			        << (pi.full_repaint ? " full_repaint" : "")
 			        << (row.changed() ? " row.changed" : ""));
@@ -2256,13 +2138,13 @@ void TextMetrics::completionPosAndDim(Cursor const & cur, int & x, int & y,
 	from.boundary(false);
 	Point lxy = cur.bv().getPos(from);
 	Point rxy = cur.bv().getPos(to);
-	dim.wid = abs(rxy.x_ - lxy.x_);
+	dim.wid = abs(rxy.x - lxy.x);
 
 	// calculate position of word
-	y = lxy.y_;
-	x = min(rxy.x_, lxy.x_);
+	y = lxy.y;
+	x = min(rxy.x, lxy.x);
 
-	//lyxerr << "wid=" << dim.width() << " x=" << x << " y=" << y << " lxy.x_=" << lxy.x_ << " rxy.x_=" << rxy.x_ << " word=" << word << std::endl;
+	//lyxerr << "wid=" << dim.width() << " x=" << x << " y=" << y << " lxy.x=" << lxy.x << " rxy.x=" << rxy.x << " word=" << word << std::endl;
 	//lyxerr << " wordstart=" << wordStart << " bvcur=" << bvcur << " cur=" << cur << std::endl;
 }
 
