@@ -13,13 +13,14 @@
 
 #include "Trans.h"
 
-#include "Lexer.h"
 #include "Text.h"
 
 #include "support/debug.h"
 #include "support/docstream.h"
+#include "support/lassert.h"
 #include "support/FileName.h"
 #include "support/filetools.h"
+#include "support/Lexer.h"
 #include "support/lstrings.h"
 
 using namespace std;
@@ -176,19 +177,13 @@ tex_accent getkeymod(string const &);
 
 void Trans::addDeadkey(tex_accent accent, docstring const & keys)
 {
-	KmodInfo tmp;
-	tmp.data = keys;
-	tmp.accent = accent;
-	kmod_list_[accent] = tmp;
+	kmod_list_[accent] = KmodInfo(keys, accent, KmodException());
 
 	for (char_type key : keys) {
 		// FIXME This is a hack.
 		// tmp is no valid UCS4 string, but misused to store the
 		// accent.
-		docstring tmpd;
-		tmpd += char_type(0);
-		tmpd += char_type(accent);
-		keymap_[key] = tmpd;
+		keymap_[key] = docstring() + char_type(0) + char_type(accent);
 	}
 }
 
@@ -368,6 +363,8 @@ docstring const Trans::process(char_type c, TransManager & k)
 	if (!t.empty() && t[0] != 0)
 		return t; //return k.normalkey(c);
 
+	LASSERT(t.size()>1, return docstring());
+
 	return k.deadkey(c, kmod_list_[static_cast<tex_accent>(t[1])]);
 }
 
@@ -451,7 +448,7 @@ docstring const TransInitState::normalkey(char_type c)
 }
 
 
-docstring const TransInitState::deadkey(char_type c, KmodInfo d)
+docstring const TransInitState::deadkey(char_type c, KmodInfo const & d)
 {
 	deadkey_ = c;
 	deadkey_info_ = d;
@@ -488,7 +485,7 @@ docstring const TransDeadkeyState::normalkey(char_type c)
 }
 
 
-docstring const TransDeadkeyState::deadkey(char_type c, KmodInfo d)
+docstring const TransDeadkeyState::deadkey(char_type c, KmodInfo const & d)
 {
 	docstring res;
 
@@ -548,7 +545,7 @@ docstring const TransCombinedState::normalkey(char_type c)
 }
 
 
-docstring const TransCombinedState::deadkey(char_type c, KmodInfo d)
+docstring const TransCombinedState::deadkey(char_type c, KmodInfo const & d)
 {
 	// Third key in a row. Output the first one and
 	// reenter with shifted deadkeys
