@@ -57,7 +57,7 @@ public:
 	/**
 	 * Return a string describing the KeySym with modifier mod.
 	 * Use the native UI format when \c forgui is true.
-	 * i.e. (translated and with special characters for Mac OS X)
+	 * i.e. (translated and with special characters for macOS)
 	 */
 	docstring const print(KeyModifier mod, bool forgui, bool untranslated = false) const;
 
@@ -66,7 +66,7 @@ public:
 	///
 	void setKey(int key) { key_ = key; }
 	///
-	docstring text() const { return text_; }
+	docstring const & text() const { return text_; }
 	///
 	void setText(docstring const & text) { text_ = text; }
 private:
