@@ -13,6 +13,7 @@
 #include "ColorCache.h"
 
 #include "LyXRC.h"
+#include "qt_helpers.h"
 
 #include "Color.h"
 #include "ColorSet.h"
@@ -31,6 +32,7 @@ QPalette::ColorRole role(ColorCode col)
 	case Color_graphicsbg:
 	case Color_mathmacrobg:
 	case Color_mathcorners:
+	case Color_preeditbg:
 		return QPalette::Base;
 		break;
 
@@ -39,14 +41,17 @@ QPalette::ColorRole role(ColorCode col)
 	case Color_preview:
 	case Color_tabularline:
 	case Color_previewframe:
+	case Color_preedittext:
 		return QPalette::Text;
 		break;
 
 	case Color_selection:
+	case Color_preeditfocus:
 		return QPalette::Highlight;
 		break;
 	case Color_selectionmath:
 	case Color_selectiontext:
+	case lyx::Color_preeditfocustext:
 		return QPalette::HighlightedText;
 		break;
 	case Color_urllabel:
@@ -63,7 +68,12 @@ QPalette::ColorRole role(ColorCode col)
 void ColorCache::init()
 {
 	for (int col = 0; col <= Color_ignore; ++col) {
-		lcolors_[col] = QColor(lcolor.getX11HexName(ColorCode(col), isDarkMode()).c_str());
+		// light-mode color
+		lcolors_[col].first =
+		    QColor(lcolor.getX11HexName(ColorCode(col), false).c_str());
+		// dark-mode color
+		lcolors_[col].second =
+		     QColor(lcolor.getX11HexName(ColorCode(col), true).c_str());
 	}
 
 	initialized_ = true;
@@ -79,6 +89,18 @@ QColor ColorCache::get(Color const & color) const
 
 /// get the given color
 QColor ColorCache::get(Color const & color, bool syscolors) const
+{
+	bool const dark_mode = isDarkMode();
+
+	if (dark_mode)
+		return getAll(color, syscolors).second;
+	else
+		return getAll(color, syscolors).first;
+}
+
+
+/// get the given color
+ColorPair ColorCache::getAll(Color const & color, bool syscolors) const
 {
 	if (!initialized_)
 		const_cast<ColorCache *>(this)->init();
@@ -92,22 +114,27 @@ QColor ColorCache::get(Color const & color, bool syscolors) const
 			if (cr == QPalette::Base && c == white)
 				return lcolors_[color.baseColor];
 			else
-				return c;
+				return {c, c};
 		} else
 			return lcolors_[color.baseColor];
 	}
 	if (color.mergeColor != Color_ignore) {
 		// FIXME: This would ideally be done in the Color class, but
 		// that means that we'd have to use the Qt code in the core.
-		QColor base_color = get(color.baseColor, syscolors).toRgb();
-		QColor merge_color = get(color.mergeColor, syscolors).toRgb();
-		return QColor(
-			(base_color.red() + merge_color.red()) / 2,
-			(base_color.green() + merge_color.green()) / 2,
-			(base_color.blue() + merge_color.blue()) / 2);
+		ColorPair base_colors = getAll(color.baseColor, syscolors);
+		ColorPair merge_colors = getAll(color.mergeColor, syscolors);
+		return {
+			QColor((base_colors.first.toRgb().red() + merge_colors.first.toRgb().red()) / 2,
+			       (base_colors.first.toRgb().green() + merge_colors.first.toRgb().green()) / 2,
+			       (base_colors.first.toRgb().blue() + merge_colors.first.toRgb().blue()) / 2),
+		    QColor((base_colors.second.toRgb().red() + merge_colors.second.toRgb().red()) / 2,
+		           (base_colors.second.toRgb().green() + merge_colors.second.toRgb().green()) / 2,
+		           (base_colors.second.toRgb().blue() + merge_colors.second.toRgb().blue()) / 2),
+		};
 	}
 	// used by branches
-	return QColor(lcolor.getX11HexName(color.baseColor, isDarkMode()).c_str());
+	return {QColor(lcolor.getX11HexName(color.baseColor, false).c_str()),
+		    QColor(lcolor.getX11HexName(color.baseColor, true).c_str())};
 }
 
 
@@ -124,10 +151,7 @@ bool ColorCache::isSystem(ColorCode const color) const
 
 bool ColorCache::isDarkMode() const
 {
-	QColor text_color = pal_.color(QPalette::Active, QPalette::WindowText);
-	QColor bg_color = pal_.color(QPalette::Active, QPalette::Window);
-
-	return (text_color.black() < bg_color.black());
+	return frontend::isPaletteDark(pal_);
 }
 
 
