@@ -32,6 +32,7 @@
 #include "ui_MasterChildUi.h"
 #include "ui_MathsUi.h"
 #include "ui_ModulesUi.h"
+#include "ui_NomenclUi.h"
 #include "ui_NumberingUi.h"
 #include "ui_OutputUi.h"
 #include "ui_PageLayoutUi.h"
@@ -39,9 +40,9 @@
 #include "ui_PreambleUi.h"
 #include "ui_TextLayoutUi.h"
 
-#include "support/unique_ptr.h"
 
 #include <QStandardItemModel>
+#include <QStyledItemDelegate>
 
 namespace lyx {
 
@@ -74,6 +75,17 @@ public:
 	UiWidget(QWidget * parent) : QWidget(parent) { UI::setupUi(this); }
 };
 
+// use to make single QTreeWidget column editable
+// courtesy of https://stackoverflow.com/a/4657065
+class NoEditDelegate : public QStyledItemDelegate {
+public:
+	NoEditDelegate(QObject* parent=0): QStyledItemDelegate(parent) {}
+	virtual QWidget* createEditor(QWidget *, const QStyleOptionViewItem &, const QModelIndex &) const
+	{
+		return 0;
+	}
+};
+
 
 class GuiDocument : public GuiDialog, public Ui::DocumentUi
 {
@@ -148,14 +160,6 @@ private Q_SLOTS:
 	void updateEngineDependends();
 	void updateModuleInfo();
 	void modulesChanged();
-	void changeBackgroundColor();
-	void deleteBackgroundColor();
-	void changeFontColor();
-	void deleteFontColor();
-	void changeNoteFontColor();
-	void deleteNoteFontColor();
-	void changeBoxBackgroundColor();
-	void deleteBoxBackgroundColor();
 	void languageChanged(int);
 	void osFontsChanged(bool);
 	void encodingSwitched(int);
@@ -173,7 +177,13 @@ private Q_SLOTS:
 	void setOutputSync(bool);
 	void bookmarksopenChanged(bool);
 	void changeTrackingChanged(bool);
-	void updateMarginLabels(int);
+	void updateLanguageOptions();
+	void addCustomColor();
+	void removeCustomColor();
+	void renameCustomColor();
+	void alterCustomColor();
+	void toggleCustomColor(QTreeWidgetItem *, int);
+	void updateMarginLabels();
 private:
 	/// validate listings parameters and return an error message, if any
 	QString validateListingsParameters();
@@ -194,6 +204,7 @@ private:
 	UiWidget<Ui::ModulesUi> *modulesModule;
 	UiWidget<Ui::OutputUi> *outputModule;
 	UiWidget<Ui::ListingsSettingsUi> *listingsModule;
+	UiWidget<Ui::NomenclUi> *nomenclModule;
 	PreambleModule * preambleModule;
 	LocalLayout * localLayout;
 
@@ -343,6 +354,10 @@ private:
 	///
 	void checkOnClosing();
 	///
+	void updateCustomColors();
+	///
+	bool checkColorUnique(QString const &);
+	///
 	BufferParams bp_;
 	/// Store buffer filename for the case we switch with
 	/// unapplied changes.
@@ -354,6 +369,8 @@ private:
 	///
 	std::list<std::string> includeonlys_;
 	///
+	std::map<QString, QString> custom_colors_;
+	///
 	bool biblioChanged_;
 	/// Track if a non-module document param changed
 	bool nonModuleChanged_;
@@ -407,12 +424,14 @@ protected:
 
 private:
 	void closeEvent(QCloseEvent *) override;
-	void on_preambleTE_textChanged() { changed(); }
+	void on_latexPreambleTE_textChanged() { changed(); }
+	void on_htmlPreambleTE_textChanged() { changed(); }
 
 	typedef std::map<BufferId, std::pair<int,int> > Coords;
-	Coords preamble_coords_;
+	Coords latex_preamble_coords_;
+	Coords html_preamble_coords_;
 	BufferId current_id_;
-	unique_ptr<support::TempFile> tempfile_;
+	std::unique_ptr<support::TempFile> tempfile_;
 	/// LaTeX syntax highlighter
 	LaTeXHighlighter * highlighter_;
 
@@ -451,7 +470,7 @@ private Q_SLOTS:
 private:
 	BufferId current_id_;
 	bool validated_;
-	unique_ptr<support::TempFile> tempfile_;
+	std::unique_ptr<support::TempFile> tempfile_;
 };
 
 
