@@ -21,7 +21,6 @@
 #include "LyX.h"
 #include "BufferView.h"
 #include "Buffer.h"
-#include "Cursor.h"
 #include "FuncRequest.h"
 #include "KeyMap.h"
 #include "GuiKeySymbol.h"
@@ -30,14 +29,11 @@
 #include "qt_helpers.h"
 #include "support/filetools.h"
 #include "support/debug.h"
-#include "support/gettext.h"
 #include "support/FileName.h"
-#include "frontends/alert.h"
 #include "frontends/Clipboard.h"
 
 #include <QClipboard>
 #include <QPainter>
-#include <QLineEdit>
 #include <QSettings>
 #include <QHideEvent>
 #include <QShowEvent>
@@ -598,7 +594,7 @@ void GuiSearchWidget::onDockWidgetVisibilityChanged()
 
 
 GuiSearch::GuiSearch(GuiView & parent, Qt::DockWidgetArea area, Qt::WindowFlags flags)
-	: DockView(parent, "findreplace", qt_("Search and Replace"), area, flags),
+	: DockView(parent, "findreplace", qt_("Find and Replace"), area, flags),
 	  widget_(new GuiSearchWidget(this, parent))
 {
 	setWidget(widget_);
@@ -613,7 +609,7 @@ GuiSearch::GuiSearch(GuiView & parent, Qt::DockWidgetArea area, Qt::WindowFlags
 void GuiSearch::mousePressEvent(QMouseEvent * event)
 {
 	if (isFloating() && event->button() == Qt::LeftButton) {
-#if QT_VERSION >= 0x060000
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 		dragPosition = event->globalPosition().toPoint() - frameGeometry().topLeft();
 #else
 		dragPosition = event->globalPos() - frameGeometry().topLeft();
@@ -627,7 +623,7 @@ void GuiSearch::mousePressEvent(QMouseEvent * event)
 void GuiSearch::mouseMoveEvent(QMouseEvent * event)
 {
 	if (isFloating() && event->buttons() & Qt::LeftButton) {
-#if QT_VERSION >= 0x060000
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 		move(event->globalPosition().toPoint() - dragPosition);
 #else
 		move(event->globalPos() - dragPosition);
