@@ -17,13 +17,12 @@
 
 #include "frontends/Delegates.h"
 
-#include "support/strfwd.h"
-
 #include <QMainWindow>
 #include <QLabel>
 #include <QMenu>
 #include <QSvgWidget>
 
+class QStandardItemModel;
 class QCloseEvent;
 class QDragEnterEvent;
 class QDropEvent;
@@ -119,6 +118,8 @@ public:
 	bool getStatus(FuncRequest const & cmd, FuncStatus & flag);
 	/// dispatch command.
 	/// \return true if the \c FuncRequest has been dispatched.
+	/// Do not use this version of dispatch() that uses a DispatchResult
+	/// if not using it after the call, see cd3d327366.
 	void dispatch(FuncRequest const & cmd, DispatchResult & dr);
 
 	void restartCaret();
@@ -174,14 +175,34 @@ public:
 	void resetAutosaveTimers() override;
 	// shows an error list
 	// if from_master is true, show master's error list
-	void errors(std::string const &, bool from_master = false) override;
+	void errors(std::string const &, bool from_master = false,
+		    int const item = -1) override;
 	void structureChanged() override;
 	void updateTocItem(std::string const &, DocIterator const &) override;
 	//@}
 
+	/** Find the next error from current cursor position.
+	 *  \return the error item position in the error list
+	 *  with \param navigateto the cursor moves to the error
+	 *  with \param atcursor the error at current cursor position
+	 *  is considered as well, otherwise only those who follow.
+	 */
+	int nextError(std::string const & error_type, bool from_master = false,
+		      bool navigateto = false, bool atcursor = false);
+
 	///
 	TocModels & tocModels();
 
+	///
+	void addColorItem(QString const & item, QString const & guiname,
+			  QString const & category = QString(),
+			  QString const color = QString(),
+			  bool custom = false, int row = -1) const;
+	///
+	QStandardItemModel * viewColorsModel();
+	///
+	void updateColorsModel() const;
+
 	/// called on timeout
 	void autoSave();
 
@@ -237,6 +258,8 @@ Q_SIGNALS:
 	void changeTrackingToggled(bool);
 	///
 	void dockWidgetVisibilityChanged();
+	///
+	void colorsModelChanged() const;
 
 public Q_SLOTS:
 	///
@@ -567,6 +590,9 @@ private:
 
 	// initial zoom for pinch gesture
 	int initialZoom_;
+
+	///
+	QStandardItemModel * colors_model_;
 };
 
 
@@ -586,7 +612,7 @@ class PressableSvgWidget : public QSvgWidget
 	Q_OBJECT
 public:
     explicit PressableSvgWidget(const QString &file, QWidget * parent = nullptr)
-	: QSvgWidget(file, parent) {};
+	: QSvgWidget(file, parent) {}
 protected:
     void mousePressEvent(QMouseEvent *event) override;
 Q_SIGNALS:
