@@ -12,8 +12,10 @@
 
 #include <config.h>
 
+#include <QString>
 #include "qt_helpers.h"
 
+#include "Buffer.h"
 #include "Format.h"
 #include "LengthCombo.h"
 #include "LyXRC.h"
@@ -33,6 +35,7 @@
 #include <QComboBox>
 #include <QDesktopServices>
 #include <QDir>
+#include <QHeaderView>
 #include <QInputDialog>
 #include <QLineEdit>
 #include <QMessageBox>
@@ -41,7 +44,6 @@
 #include <QPushButton>
 #include <QSet>
 #include <QSettings>
-#include <QTextLayout>
 #include <QTextDocument>
 #include <QToolTip>
 #include <QUrl>
@@ -266,7 +268,7 @@ void focusAndHighlight(QAbstractItemView * w)
 }
 
 
-void setMessageColour(list<QWidget *> highlighted, list<QWidget *> plain)
+void setMessageColour(list<QWidget *> const & highlighted, list<QWidget *> const & plain)
 {
 	QPalette pal = QApplication::palette();
 	QPalette newpal(pal.color(QPalette::Active, QPalette::HighlightedText),
@@ -395,6 +397,15 @@ void showTarget(string const & target_in, Buffer const & buf)
 			bformat(_("The target `%1$s' could not be resolved."),
 				from_utf8(target)));
 }
+
+bool isPaletteDark(QPalette const & pal)
+{
+	QColor text_color = pal.color(QPalette::Active, QPalette::WindowText);
+	QColor bg_color = pal.color(QPalette::Active, QPalette::Window);
+
+	return (text_color.black() < bg_color.black());
+}
+
 } // namespace frontend
 
 QString const qt_(char const * str, const char *)
