@@ -12,7 +12,7 @@
 #ifndef PREVIEWIMAGE_H
 #define PREVIEWIMAGE_H
 
-#include "support/strfwd.h"
+#include "support/docstring.h"
 
 namespace lyx {
 
@@ -22,7 +22,6 @@ class Dimension;
 
 namespace graphics {
 
-class Cache;
 class PreviewLoader;
 class Image;
 
@@ -32,14 +31,14 @@ public:
 	 *  descent = height * (1 - ascent_frac)
 	 */
 	PreviewImage(PreviewLoader & parent,
-		     std::string const & latex_snippet,
+		     docstring const & latex_snippet,
 		     support::FileName const & bitmap_file,
 		     double ascent_frac);
 	///
 	~PreviewImage();
 
 	///
-	std::string const & snippet() const;
+	docstring const & snippet() const;
 	///
 	Dimension dim() const;
 
