@@ -24,7 +24,6 @@
 #include "RowFlags.h"
 
 #include "support/docstring.h"
-#include "support/strfwd.h"
 #include "support/types.h"
 
 #include <climits>
@@ -46,29 +45,36 @@ class FuncStatus;
 class InsetArgument;
 class InsetCollapsible;
 class InsetCommand;
+class InsetFloat;
 class InsetGraphics;
 class InsetIndex;
 class InsetIterator;
 class InsetLayout;
 class InsetList;
 class InsetMath;
+class InsetNomencl;
+class InsetRef;
 class InsetTabular;
 class InsetText;
 class Language;
 class LaTeXFeatures;
-class Lexer;
 class MathAtom;
 class MetricsInfo;
 class PainterInfo;
 class ParIterator;
+class Statistics;
 class Text;
 class TocBackend;
 class XMLStream;
 class otexstream;
 
+namespace frontend { class InputMethod; }
 namespace graphics { class PreviewLoader; }
 
-namespace support {class FileNameList; }
+namespace support {
+class FileNameList;
+class Lexer;
+}
 
 /// returns the InsetCode corresponding to the \c name.
 /// Eg, insetCode("branch") == BRANCH_CODE
@@ -151,10 +157,16 @@ public:
 	virtual InsetArgument const * asInsetArgument() const { return nullptr; }
 	/// is this inset based on the InsetIndex class?
 	virtual InsetIndex const * asInsetIndex() const { return nullptr; }
+	/// is this inset based on the InsetNomencl class?
+	virtual InsetNomencl const * asInsetNomencl() const { return nullptr; }
+	/// is this inset based on the InsetIndex class?
+	virtual InsetRef * asInsetRef() { return nullptr; }
 	/// is this inset based on the InsetGraphics class?
 	virtual InsetGraphics * asInsetGraphics() { return nullptr; }
 	/// is this inset based on the InsetGraphics class?
 	virtual InsetGraphics const * asInsetGraphics() const { return nullptr; }
+	/// is this inset based on the InsetFloat class?
+	virtual InsetFloat const * asInsetFloat() const { return nullptr; }
 
 	/// the real dispatcher
 	void dispatch(Cursor & cur, FuncRequest & cmd);
@@ -227,6 +239,10 @@ public:
 	/// This can use \c drawMarkers() for example.
 	virtual void drawDecoration(PainterInfo &, int, int) const {}
 
+	/// do we have dimension information for this inset ?
+	bool hasDim(BufferView const & bv) const;
+	/// do we have full geometry information for this inset ?
+	bool hasGeometry(BufferView const & bv) const;
 	/// last metrics computed for the inset
 	Dimension const dimension(BufferView const &) const;
 	/// last drawn position for 'important' insets
@@ -258,7 +274,7 @@ public:
 	/// Label font
 	virtual FontInfo getLabelfont() const;
 
-	/// Where should we go when we press the up or down cursor key?
+	/// Move the cursor vertically into inset; returns false when not possible.
 	virtual bool idxUpDown(Cursor & cur, bool up) const;
 	/// Move one cell backwards
 	virtual bool idxBackward(Cursor &) const { return false; }
@@ -309,8 +325,15 @@ public:
 	/// anymore.
 	/// \c cur is the new cursor, some slice points to this. Use the update
 	/// flags to cause a redraw.
-	virtual bool notifyCursorEnters(Cursor & /*cur*/)
+	virtual bool notifyCursorEnters(Cursor const & /*old*/, Cursor & /*cur*/)
 		{ return false; }
+	// Is called when a mouse selection ends by a mouse release event.
+	// Previews of inset that have been left by the cursor are delayed
+	// until this point, so it is an occasion for insets to ask for
+	// metrics recomputation.
+	// \c cur is the cursor position that contains the final selection.
+	virtual void notifyMouseSelectionDone(Cursor & /*cur*/) {}
+
 	/// is called when the mouse enters or leaves this inset
 	/// return true if this inset needs a repaint
 	virtual bool setMouseHover(BufferView const *, bool) const
@@ -478,9 +501,8 @@ public:
 	/// is this equivalent to a space (which is BTW different from
 	/// a line separator)?
 	virtual bool isSpace() const { return false; }
-	/// returns chars, words if the inset is equivalent to such, otherwise
-	/// (0,0), which should be interpreted as 'false'
-	virtual std::pair<int, int> isWords() const { return std::pair<int,int>(0, 0); }
+	/// Count words, characters and spaces in inset
+	virtual void updateStatistics(Statistics & stats) const;
 	/// does this inset try to use all available space (like \\hfill does)?
 	virtual bool isHfill() const { return false; }
 
@@ -501,7 +523,7 @@ public:
 	/// write inset in .lyx format
 	virtual void write(std::ostream &) const {}
 	/// read inset in .lyx format
-	virtual void read(Lexer &) {}
+	virtual void read(support::Lexer &) {}
 	/** Export the inset to LaTeX.
 	 *  Don't use a temporary stringstream if the final output is
 	 *  supposed to go to a file.
@@ -626,7 +648,10 @@ public:
 protected:
 	/// Constructors
 	Inset(Buffer * buf) : buffer_(buf) {}
-	Inset(Inset const &) : buffer_(0) {}
+	Inset(Inset const &) : buffer_(nullptr) {}
+	Inset(Inset &&) : buffer_(nullptr) {}
+	Inset & operator=(Inset const &);
+	Inset & operator=(Inset &&);
 
 	/// replicate ourselves
 	friend class InsetList;
