@@ -34,7 +34,11 @@ public:
 	///
 	InsetBibitem(Buffer *, InsetCommandParams const &);
 	///
+	InsetBibitem(InsetBibitem const &) = default;
+	///
 	~InsetBibitem();
+	///
+	InsetBibitem & operator=(InsetBibitem const &) = default;
 
 	///
 	void updateCommand(docstring const & new_key, bool dummy = false);
@@ -51,7 +55,7 @@ public:
 	///
 	bool isLabeled() const override { return true; }
 	///
-	void read(Lexer & lex) override;
+	void read(support::Lexer & lex) override;
 	///
 	int plaintext(odocstringstream &, OutputParams const &,
 	              size_t max_length = INT_MAX) const override;
