@@ -11,25 +11,20 @@
 #include <config.h>
 
 #include "InsetCaption.h"
-#include "InsetFloat.h"
-#include "InsetWrap.h"
 
 #include "Buffer.h"
 #include "BufferParams.h"
 #include "BufferView.h"
-#include "Counters.h"
 #include "Cursor.h"
 #include "Dimension.h"
 #include "Floating.h"
 #include "FloatList.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
-#include "InsetList.h"
 #include "Language.h"
 #include "MetricsInfo.h"
+#include "xml.h"
 #include "output_latex.h"
-#include "output_xhtml.h"
-#include "OutputParams.h"
 #include "Paragraph.h"
 #include "ParIterator.h"
 #include "TexRow.h"
@@ -53,8 +48,7 @@ namespace lyx {
 
 
 InsetCaption::InsetCaption(Buffer * buf, string const & type)
-    : InsetText(buf, InsetText::PlainLayout),
-      labelwidth_(0), is_subfloat_(false), type_(type)
+    : InsetText(buf, InsetText::PlainLayout), type_(type)
 {
 	setDrawFrame(true);
 	setFrameColor(Color_collapsibleframe);
@@ -83,13 +77,8 @@ void InsetCaption::cursorPos(BufferView const & bv,
 		CursorSlice const & sl, bool boundary, int & x, int & y) const
 {
 	InsetText::cursorPos(bv, sl, boundary, x, y);
-	x += labelwidth_;
-}
-
-
-void InsetCaption::setCustomLabel(docstring const & label)
-{
-	custom_label_ = translateIfPossible(label);
+	if (!rtl_)
+		x += labelwidth_;
 }
 
 
@@ -117,17 +106,14 @@ void InsetCaption::addToToc(DocIterator const & cpit, bool output_active,
 
 void InsetCaption::metrics(MetricsInfo & mi, Dimension & dim) const
 {
-	FontInfo tmpfont = mi.base.font;
-	mi.base.font = mi.base.bv->buffer().params().getFont().fontInfo();
 	labelwidth_ = theFontMetrics(mi.base.font).width(full_label_);
 	// add some space to separate the label from the inset text
-	labelwidth_ += 2 * TEXT_TO_INSET_OFFSET;
+	labelwidth_ += leftOffset(mi.base.bv) + rightOffset(mi.base.bv);
 	dim.wid = labelwidth_;
 	Dimension textdim;
 	// Correct for button and label width
 	mi.base.textwidth -= dim.wid;
 	InsetText::metrics(mi, textdim);
-	mi.base.font = tmpfont;
 	mi.base.textwidth += dim.wid;
 	dim.des = max(dim.des - textdim.asc + dim.asc, textdim.des);
 	dim.asc = textdim.asc;
@@ -138,8 +124,10 @@ void InsetCaption::metrics(MetricsInfo & mi, Dimension & dim) const
 void InsetCaption::drawBackground(PainterInfo & pi, int x, int y) const
 {
 	TextMetrics & tm = pi.base.bv->textMetrics(&text());
-	int const h = tm.height() + 2 * TEXT_TO_INSET_OFFSET;
-	int const yy = y - TEXT_TO_INSET_OFFSET - tm.ascent();
+	int const h = tm.height() + topOffset(pi.base.bv) + bottomOffset(pi.base.bv);
+	int const yy = y - topOffset(pi.base.bv) - tm.ascent();
+	if (rtl_)
+		x+= + dimension(*pi.base.bv).wid - labelwidth_;
 	pi.pain.fillRectangle(x, yy, labelwidth_, h, pi.backgroundColor(this));
 }
 
@@ -154,12 +142,25 @@ void InsetCaption::draw(PainterInfo & pi, int x, int y) const
 
 	// Answer: the text inset (in buffer_funcs.cpp: setCaption).
 
+	rtl_ = !pi.ltr_pos;
 	FontInfo tmpfont = pi.base.font;
-	pi.base.font = pi.base.bv->buffer().params().getFont().fontInfo();
-	pi.base.font.setColor(pi.textColor(pi.base.font.color()).baseColor);
-	int const xx = x + TEXT_TO_INSET_OFFSET;
-	pi.pain.text(xx, y, full_label_, pi.base.font);
-	InsetText::draw(pi, x + labelwidth_, y);
+	if (non_float_)
+		pi.base.font.setColor(Color_error);
+	else
+		pi.base.font.setPaintColor(pi.textColor(pi.base.font.color()));
+	int const lo = leftOffset(pi.base.bv);
+	if (rtl_) {
+		InsetText::draw(pi, x, y);
+		pi.pain.text(x + dimension(*pi.base.bv).wid - labelwidth_ + lo,
+		             y, full_label_, pi.base.font);
+	} else {
+		pi.pain.text(x + lo, y, full_label_, pi.base.font);
+		InsetText::draw(pi, x + labelwidth_, y);
+	}
+	// Draw the change tracking cue on the label, unless RowPainter already
+	// takes care of it.
+	if (canPaintChange(*pi.base.bv))
+		pi.change.paintCue(pi, x, y, x + labelwidth_, pi.base.font);
 	pi.base.font = tmpfont;
 }
 
@@ -187,7 +188,7 @@ bool InsetCaption::insetAllowed(InsetCode code) const
 	case FOOT_CODE:
 	case NEWPAGE_CODE:
 	case MARGIN_CODE:
-	case MATHMACRO_CODE:
+	case MATH_MACROTEMPLATE_CODE:
 	case TABULAR_CODE:
 	case WRAP_CODE:
 		return false;
@@ -234,7 +235,7 @@ bool InsetCaption::getStatus(Cursor & cur, FuncRequest const & cmd,
 			if (cur.depth() > 1) {
 				varia = cur[cur.depth() - 2].inset().allowsCaptionVariation(type);
 			}
-			status.setEnabled(varia
+			status.setEnabled(!is_subfloat_ && varia
 					  && buffer().params().documentClass().hasInsetLayout(
 						from_ascii("Caption:" + type)));
 			return true;
@@ -265,7 +266,16 @@ void InsetCaption::latex(otexstream & os,
 	// \caption{...}, later we will make it take advantage
 	// of the one of the caption packages. (Lgb)
 	OutputParams runparams = runparams_in;
+	// Some fragile commands (labels, index entries)
+	// are output after the caption (#2154)
+	runparams.postpone_fragile_stuff = buffer().masterParams().postpone_fragile_content;
 	InsetText::latex(os, runparams);
+	if (!runparams.post_macro.empty()) {
+		// Output the stored fragile commands (labels, indices etc.)
+		// that need to be output after the caption.
+		os << runparams.post_macro;
+		runparams.post_macro.clear();
+	}
 	// Backwards compatibility: We always had a linebreak after
 	// the caption (see #8514)
 	os << breakln;
@@ -284,24 +294,19 @@ int InsetCaption::plaintext(odocstringstream & os,
 }
 
 
-int InsetCaption::docbook(odocstream & os,
-			  OutputParams const & runparams) const
+void InsetCaption::docbook(XMLStream &, OutputParams const &) const
 {
-	int ret;
-	os << "<title>";
-	ret = InsetText::docbook(os, runparams);
-	os << "</title>\n";
-	return ret;
+	// This function should never be called (rather InsetFloat::docbook, the titles should be skipped in floats).
 }
 
 
-docstring InsetCaption::xhtml(XHTMLStream & xs, OutputParams const & rp) const
+docstring InsetCaption::xhtml(XMLStream & xs, OutputParams const & rp) const
 {
 	if (rp.html_disable_captions)
 		return docstring();
 	InsetLayout const & il = getLayout();
 	string const & tag = il.htmltag();
-	string attr = il.htmlattr();
+	string attr = il.htmlGetAttrString();
 	if (!type_.empty()) {
 		string const our_class = "float-caption-" + type_;
 		size_t const loc = attr.find("class='");
@@ -310,10 +315,19 @@ docstring InsetCaption::xhtml(XHTMLStream & xs, OutputParams const & rp) const
 		else
 			attr = attr + " class='" + our_class + "'";
 	}
-	xs << html::StartTag(tag, attr);
-	docstring def = getCaptionAsHTML(xs, rp);
-	xs << html::EndTag(tag);
-	return def;
+	xs << xml::StartTag(tag, attr);
+	writeCaptionAsHTML(xs, rp);
+	xs << xml::EndTag(tag);
+	return docstring();
+}
+
+
+docstring InsetCaption::toolTip(BufferView const & bv, int x, int y) const
+{
+	if (non_float_)
+		return _("Standard captions are not allowed outside floats. You will get a LaTeX error.\n"
+			 "For captions outside floats, you can use the 'nonfloat' LaTeX package.");
+	return InsetText::toolTip(bv, x, y);
 }
 
 
@@ -330,6 +344,8 @@ void InsetCaption::getArgument(otexstream & os,
 		rp.pass_thru = true;
 	if (il.isNeedProtect())
 		rp.moving_arg = true;
+	if (il.isNeedMBoxProtect())
+		++rp.inulemcmd;
 	rp.par_begin = 0;
 	rp.par_end = paragraphs().size();
 
@@ -355,32 +371,45 @@ int InsetCaption::getCaptionAsPlaintext(odocstream & os,
 }
 
 
-docstring InsetCaption::getCaptionAsHTML(XHTMLStream & xs,
+void InsetCaption::writeCaptionAsDocBook(XMLStream & xs,
+				       OutputParams const & runparams) const
+{
+	if (runparams.docbook_in_float)
+		return;
+
+	// Ignore full_label_, as the DocBook processor will deal with the numbering.
+	InsetText::XHTMLOptions opts = InsetText::WriteInnerTag;
+	InsetText::docbook(xs, runparams, opts);
+}
+
+
+void InsetCaption::writeCaptionAsHTML(XMLStream & xs,
 			OutputParams const & runparams) const
 {
 	xs << full_label_ << ' ';
 	InsetText::XHTMLOptions const opts =
 		InsetText::WriteLabel | InsetText::WriteInnerTag;
-	return InsetText::insetAsXHTML(xs, runparams, opts);
+	InsetText::insetAsXHTML(xs, runparams, opts);
 }
 
 
-void InsetCaption::updateBuffer(ParIterator const & it, UpdateType utype)
+void InsetCaption::updateBuffer(ParIterator const & it, UpdateType utype, bool const deleted)
 {
 	Buffer const & master = *buffer().masterBuffer();
 	DocumentClass const & tclass = master.params().documentClass();
 	string const & lang = it.paragraph().getParLanguage(master.params())->code();
 	Counters & cnts = tclass.counters();
 	string const & type = cnts.current_float();
-	if (utype == OutputUpdate) {
-		// counters are local to the caption
-		cnts.saveLastCounter();
-	}
+	// counters are local to the caption
+	cnts.saveLastCounter();
+
+	is_deleted_ = deleted;
 	// Memorize type for addToToc().
 	floattype_ = type;
-	if (type.empty() || type == "senseless")
-		full_label_ = master.B_("Senseless!!! ");
-	else {
+	if (type.empty() || type == "senseless") {
+		full_label_ = master.B_("Orphaned caption:");
+		non_float_ = true;
+	} else {
 		// FIXME: life would be _much_ simpler if listings was
 		// listed in Floating.
 		docstring name;
@@ -389,38 +418,41 @@ void InsetCaption::updateBuffer(ParIterator const & it, UpdateType utype)
 		else
 			name = master.B_(tclass.floats().getType(type).name());
 		docstring counter = from_utf8(type);
-		if ((is_subfloat_ = cnts.isSubfloat())) {
+		is_subfloat_ = cnts.isSubfloat();
+		if (is_subfloat_) {
 			// only standard captions allowed in subfloats
 			type_ = "Standard";
 			counter = "sub-" + from_utf8(type);
-			name = bformat(_("Sub-%1$s"),
-				       master.B_(tclass.floats().getType(type).name()));
+			string const sf = "Sub-" + tclass.floats().getType(type).name();
+			name = master.B_(sf);
 		}
-		docstring sec;
 		docstring const lstring = getLayout().labelstring();
 		docstring const labelstring = isAscii(lstring) ?
 				master.B_(to_ascii(lstring)) : lstring;
 		if (cnts.hasCounter(counter)) {
+			int const val = cnts.value(counter);
 			// for longtables, we step the counter upstream
 			if (!cnts.isLongtable())
 				cnts.step(counter, utype);
-			sec = cnts.theCounter(counter, lang);
+			val_ = cnts.theCounter(counter, lang);
+			if (deleted && !cnts.isLongtable())
+				// un-step after deleted counter
+				cnts.set(counter, val);
 		}
 		if (labelstring != master.B_("standard")) {
-			if (!sec.empty())
-				sec += from_ascii(" ");
-			sec += bformat(from_ascii("(%1$s)"), labelstring);
+			if (!val_.empty())
+				val_ += from_ascii(" ");
+			val_ += bformat(from_ascii("(%1$s)"), labelstring);
 		}
-		if (!sec.empty())
-			full_label_ = bformat(from_ascii("%1$s %2$s: "), name, sec);
-		else
-			full_label_ = bformat(from_ascii("%1$s #: "), name);
+		if (val_.empty())
+			val_ = from_ascii("#");
+		full_label_ = bformat(master.B_("%1$s %2$s: [[Caption label (ex. Figure 1: )]]"), name, val_);
+		non_float_ = false;
 	}
 
 	// Do the real work now.
-	InsetText::updateBuffer(it, utype);
-	if (utype == OutputUpdate)
-		cnts.restoreLastCounter();
+	InsetText::updateBuffer(it, utype, deleted);
+	cnts.restoreLastCounter();
 }
 
 
