@@ -30,17 +30,15 @@
 
 #include "Buffer.h"
 #include "Encoding.h"
-#include "Lexer.h"
 
 #include "frontends/alert.h"
 
 #include "support/debug.h"
-#include "support/docstream.h"
 #include "support/ExceptionMessage.h"
 #include "support/gettext.h"
-#include "support/lstrings.h"
-
 #include "support/lassert.h"
+#include "support/Lexer.h"
+#include "support/lstrings.h"
 
 #include <algorithm>
 #include <functional>
@@ -77,8 +75,6 @@ static ParamInfo const & findInfo(InsetCode code, string const & cmdName)
 		return InsetLabel::findInfo(cmdName);
 	case LINE_CODE:
 		return InsetLine::findInfo(cmdName);
-	case NOMENCL_CODE:
-		return InsetNomencl::findInfo(cmdName);
 	case NOMENCL_PRINT_CODE:
 		return InsetPrintNomencl::findInfo(cmdName);
 	case REF_CODE:
@@ -102,7 +98,7 @@ static ParamInfo const & findInfo(InsetCode code, string const & cmdName)
 
 ParamInfo::ParamData::ParamData(std::string const & s, ParamType t,
                                 ParamHandling h, bool ignore,
-                                docstring default_value)
+                                docstring const & default_value)
 	: name_(s), type_(t), handling_(h), ignore_(ignore),
 	  default_value_(default_value)
 {}
@@ -217,8 +213,6 @@ string InsetCommandParams::getDefaultCmd(InsetCode code)
 			return InsetLabel::defaultCommand();
 		case LINE_CODE:
 			return InsetLine::defaultCommand();
-		case NOMENCL_CODE:
-			return InsetNomencl::defaultCommand();
 		case NOMENCL_PRINT_CODE:
 			return InsetPrintNomencl::defaultCommand();
 		case REF_CODE:
@@ -256,8 +250,6 @@ bool InsetCommandParams::isCompatibleCommand(InsetCode code, string const & s)
 			return InsetLabel::isCompatibleCommand(s);
 		case LINE_CODE:
 			return InsetLine::isCompatibleCommand(s);
-		case NOMENCL_CODE:
-			return InsetNomencl::isCompatibleCommand(s);
 		case NOMENCL_PRINT_CODE:
 			return InsetPrintNomencl::isCompatibleCommand(s);
 		case REF_CODE:
@@ -447,8 +439,7 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 		result = command;
 		ltrimmed = true;
 	}
-	if (handling & ParamInfo::HANDLING_LATEXIFY
-	    || handling & ParamInfo::HANDLING_INDEX_ESCAPE)
+	if (handling & ParamInfo::HANDLING_LATEXIFY)
 		if ((*this)["literal"] == "true")
 			handling = ParamInfo::HANDLING_NONE;
 
@@ -547,25 +538,6 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 				uncodable));
 		}
 	}
-	// INDEX_ESCAPE is independent of the others
-	if (handling & ParamInfo::HANDLING_INDEX_ESCAPE) {
-		// Now escape special commands
-		// We only use this for nomencl, which has the
-		// escape char '%'
-		static docstring const esc_char = from_ascii("%");
-		int const nchars_escape = 4;
-		static char_type const chars_escape[nchars_escape] = { '@', '|', '!' };
-
-		if (!result.empty()) {
-			// The characters in chars_name[] need to be changed to a command when
-			// they are LaTeXified.
-			for (int k = 0; k < nchars_escape; k++)
-				for (size_t i = 0, pos;
-					(pos = result.find(chars_escape[k], i)) != string::npos;
-					i = pos + 2)
-						result.replace(pos, 1, esc_char + chars_escape[k]);
-		}
-	}
 
 	return ltrimmed ? ltrim(result) : result;
 }
