@@ -17,7 +17,6 @@
 #include "ExternalTemplate.h"
 
 #include "support/FileName.h"
-#include "support/unique_ptr.h"
 
 
 namespace lyx {
@@ -51,7 +50,7 @@ public:
 	InsetExternalParams();
 
 	void write(Buffer const &, std::ostream &) const;
-	bool read(Buffer const &, Lexer &);
+	bool read(Buffer const &, support::Lexer &);
 
 	/// The name of the tempfile used for manipulations.
 	support::FileName tempname() const { return tempname_(); }
@@ -141,7 +140,7 @@ public:
 	///
 	void write(std::ostream &) const override;
 	///
-	void read(Lexer & lex) override;
+	void read(support::Lexer & lex) override;
 	///
 	int plaintext(odocstringstream & ods, OutputParams const & op,
 	              size_t max_length = INT_MAX) const override;
@@ -182,7 +181,7 @@ private:
 	/// The current params
 	InsetExternalParams params_;
 	/// The thing that actually draws the image on LyX's screen.
-	mutable unique_ptr<RenderBase> renderer_;
+	mutable std::unique_ptr<RenderBase> renderer_;
 	/// changes color of the button when mouse enters/leaves this inset
 	mutable std::map<BufferView const *, bool> mouse_hover_;
 };
