@@ -12,12 +12,6 @@
 
 #include "InsetFootlike.h"
 
-#include "Buffer.h"
-#include "BufferView.h"
-#include "BufferParams.h"
-#include "Font.h"
-#include "MetricsInfo.h"
-
 #include "support/lstrings.h"
 
 #include <iostream>
@@ -26,6 +20,8 @@ using namespace std;
 
 namespace lyx {
 
+class Buffer;
+
 using support::token;
 
 InsetFootlike::InsetFootlike(Buffer * buf)
