@@ -18,10 +18,9 @@
 #include "FuncStatus.h"
 #include "LaTeXFeatures.h"
 #include "LyX.h"
-#include "output_docbook.h"
-#include "output_xhtml.h"
-#include "xml.h"
+#include "Statistics.h"
 #include "texstream.h"
+#include "xml.h"
 
 #include "support/debug.h"
 #include "support/docstream.h"
@@ -31,8 +30,6 @@
 #include "support/lstrings.h"
 #include "support/qstring_helpers.h"
 
-#include "frontends/alert.h"
-
 #include <QtGui/QDesktopServices>
 #include <QUrl>
 
@@ -299,10 +296,9 @@ void InsetHyperlink::validate(LaTeXFeatures & features) const
 }
 
 
-pair<int, int> InsetHyperlink::isWords() const
+void InsetHyperlink::updateStatistics(Statistics & stats) const
 {
-	docstring const label = getParam("name");
-	return pair<int, int>(label.size(), wordCount(label));
+	stats.update(getParam("name"));
 }
 
 
