@@ -30,10 +30,7 @@
 #include "Paragraph.h"
 #include "LaTeX.h"
 #include "LaTeXFeatures.h"
-#include "Lexer.h"
 #include "LyX.h"
-#include "output_latex.h"
-#include "output_xhtml.h"
 #include "xml.h"
 #include "texstream.h"
 #include "TextClass.h"
@@ -43,11 +40,10 @@
 #include "support/docstream.h"
 #include "support/FileName.h"
 #include "support/gettext.h"
+#include "support/Lexer.h"
 #include "support/lstrings.h"
 #include "support/Translator.h"
 
-#include "frontends/alert.h"
-
 #include <algorithm>
 #include <set>
 #include <iostream>
@@ -142,7 +138,10 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 
 	if (buffer().masterBuffer()->params().use_indices && !params_.index.empty()
 		&& params_.index != "idx") {
-		os << "\\sindex[";
+		if (runparams.use_memindex)
+			os << "\\index[";
+		else
+			os << "\\sindex[";
 		os << escape(params_.index);
 		os << "]{";
 	} else {
@@ -187,6 +186,11 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 		// These are the LaTeX and plaintext representations
 		docstring latexstr = ourlatex.str();
 		docstring plainstr = ourplain.str();
+
+		// get the escape char from the layout
+		docstring const escape_char = (runparams.escape_chars.empty())
+				? from_ascii("\"")
+				: runparams.escape_chars.substr(0,1);
 	
 		// This will get what follows | if anything does,
 		// the command (e.g., see, textbf) for pagination
@@ -202,10 +206,10 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 			cmd = from_utf8(params_.pagefmt);
 		} else {
 			// Check for the | separator to strip the cmd.
-			// This goes wrong on an escaped "|", but as the escape
-			// character can be changed in style files, we cannot
-			// prevent that.
+			// Consider escaped "|"
 			size_t pos = latexstr.find(from_ascii("|"));
+			while (pos > 0 && pos < docstring::npos && prefixIs(latexstr.substr(pos - 1), escape_char))
+				pos = latexstr.find(from_ascii("|"), pos + 1);
 			if (pos != docstring::npos) {
 				// Put the bit after "|" into cmd...
 				cmd = latexstr.substr(pos + 1);
@@ -225,23 +229,22 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 		getSubentries(otsub, runparams, ourlatex.str());
 		if (subentries.str().empty()) {
 			// Separate the entries and subentries, i.e., split on "!".
-			// This goes wrong on an escaped "!", but as the escape
-			// character can be changed in style files, we cannot
-			// prevent that.
+			// Consider escaped "!"
+			// temporarily replace "! with SUBST character
+			docstring const sub_latexstring =
+					subst(latexstr, escape_char + from_ascii("!"), docstring(1, 0x001A));
 			std::vector<docstring> const levels =
-					getVectorFromString(latexstr, from_ascii("!"), true);
-			std::vector<docstring> const levels_plain =
-					getVectorFromString(plainstr, from_ascii("!"), true);
+					getVectorFromString(sub_latexstring, from_ascii("!"), true);
 		
 			vector<docstring>::const_iterator it = levels.begin();
 			vector<docstring>::const_iterator end = levels.end();
-			vector<docstring>::const_iterator it2 = levels_plain.begin();
 			bool first = true;
 			for (; it != end; ++it) {
-				if ((*it).empty()) {
+				// replace back "!
+				docstring const thislevel =
+					subst(*it, docstring(1, 0x001A), escape_char + from_ascii("!"));
+				if (thislevel.empty()) {
 					emptySubentriesWarning(ourlatex.str());
-					if (it2 < levels_plain.end())
-						++it2;
 					continue;
 				}
 				// The separator needs to be put back when
@@ -258,15 +261,16 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 				// e.g. \index{LyX@\LyX}, \index{text@\textbf{text}}.
 				// We do this on all levels.
 				// We don't do it if the level already contains a '@', though.
-				// Plaintext might return nothing (e.g. for ERTs).
-				// In that case, we use LaTeX.
-				docstring const spart = (levels_plain.empty() || (*it2).empty()) ? *it : *it2;
-				processLatexSorting(os, runparams, *it, spart);
-				if (it2 < levels_plain.end())
-					++it2;
+				// We use a somewhat "plain" representation for this
+				docstring spart = Encodings::convertLaTeXCommands(thislevel);
+				// if convertLaTeXCommands() returns nothing, we fall back
+				// to the command name without backslash
+				if (trim(spart).empty())
+					spart = ltrim(thislevel, "\\");
+				processLatexSorting(os, runparams, thislevel, spart, escape_char);
 			}
 		} else {
-			processLatexSorting(os, runparams, latexstr, plainstr);
+			processLatexSorting(os, runparams, latexstr, plainstr, escape_char);
 			os << subentries.str();
 		}
 
@@ -292,9 +296,13 @@ void InsetIndex::latex(otexstream & ios, OutputParams const & runparams_in) cons
 
 
 void InsetIndex::processLatexSorting(otexstream & os, OutputParams const & runparams,
-				docstring const & latex, docstring const & spart) const
+				     docstring const & latex, docstring const & spart,
+				     docstring const & esc) const
 {
-	if (contains(latex, '\\') && !contains(latex, '@')) {
+	size_t at_pos = latex.find(from_ascii("@"));
+	while (at_pos > 0 && at_pos < docstring::npos && latex.substr(at_pos - 1, at_pos) == esc)
+		at_pos = latex.find(from_ascii("|"), at_pos + 1);
+	if (contains(latex, '\\') && at_pos == docstring::npos) {
 		// Now we need to validate that all characters in
 		// the sorting part are representable in the current
 		// encoding. If not try the LaTeX macro which might
@@ -320,6 +328,8 @@ void InsetIndex::processLatexSorting(otexstream & os, OutputParams const & runpa
 		// Plain quotes need to be escaped, however (#10649), as this
 		// is the default escape character
 		ppart = subst(ppart, from_ascii("\""), from_ascii("\\\""));
+		// Also escape hashes, as this is a special character in LaTeX
+		ppart = subst(ppart, from_ascii("#"), from_ascii("\\#"));
 
 		// Now insert the sortkey, separated by '@'.
 		os << ppart;
@@ -964,7 +974,7 @@ bool InsetIndex::hasSortKey() const
 }
 
 
-bool InsetIndex::macrosPossible(string const type) const
+bool InsetIndex::macrosPossible(string const & type) const
 {
 	if (type != "see" && type != "seealso"
 	    && type != "sortkey" && type != "subentry")
@@ -1406,7 +1416,17 @@ void InsetPrintIndex::latex(otexstream & os, OutputParams const & runparams_in)
 		return;
 	}
 	OutputParams runparams = runparams_in;
-	os << getCommand(runparams);
+	if (runparams.use_memindex) {
+		if (getParam("type") == from_ascii("idx"))
+			os << "\\printindex" << termcmd;
+		else {
+			os << "\\begingroup" << breakln;
+			os << "\\renewcommand{\\indexname}{" << getParam("name") << "}" << breakln;
+			os << "\\printindex[" << getParam("type") << "]" << breakln;
+			os << "\\endgroup" << breakln;
+		}
+	} else
+		os << getCommand(runparams);
 }
 
 
@@ -1463,10 +1483,6 @@ public:
 private:
 	bool isModern()
 	{
-#ifdef LYX_INSET_INDEX_DEBUG
-		std::cout << to_utf8(entry_) << std::endl;
-#endif // LYX_INSET_INDEX_DEBUG
-
 		// If a modern parameter is present, this is definitely a modern index inset. Similarly, if it contains the
 		// usual LaTeX symbols (!|@), then it is definitely a legacy index inset. Otherwise, if it has features of
 		// neither, it is both: consider this is a modern inset, to trigger the least complex code. Mixing both types
@@ -1887,7 +1903,7 @@ docstring InsetPrintIndex::xhtml(XMLStream &, OutputParams const & op) const
 	for (const TocItem& item : *toc) {
 		const auto* inset = static_cast<const InsetIndex*>(&(item.dit().inset()));
 		if (item.isOutput() && inset && inset->params().index == indexType)
-			entries.emplace_back(IndexEntry{inset, &op});
+			entries.emplace_back(inset, &op);
 	}
 
 	// If all the index entries are in notes or not displayed, get out sooner.
