@@ -34,7 +34,7 @@ public:
 	///
 	void write(std::ostream & os) const;
 	///
-	void read(Lexer & lex);
+	void read(support::Lexer & lex);
 	///
 	docstring index;
 	///
@@ -72,7 +72,7 @@ private:
 	///
 	void write(std::ostream & os) const override;
 	///
-	void read(Lexer & lex) override;
+	void read(support::Lexer & lex) override;
 	///
 	void docbook(XMLStream &, OutputParams const &) const override;
 	///
@@ -81,7 +81,8 @@ private:
 	void latex(otexstream &, OutputParams const &) const override;
 	///
 	void processLatexSorting(otexstream &, OutputParams const &,
-				 docstring const &, docstring const &) const;
+				 docstring const &, docstring const &,
+				 docstring const &) const;
 	///
 	bool showInsetDialog(BufferView *) const override;
 	///
@@ -127,7 +128,7 @@ private:
 	///
 	bool hasSortKey() const;
 	///
-	bool macrosPossible(std::string const type) const;
+	bool macrosPossible(std::string const & type) const;
 	///
 	std::string contextMenuName() const override;
 	///
