@@ -28,22 +28,26 @@ public:
 	///
 	docstring const & counterValue() const { return counter_value_; }
 	///
-	docstring const & prettyCounter() const { return pretty_counter_; }
+	docstring const & textRef() const { return textref_; }
 	///
-	docstring const & formattedCounter() const { return formatted_counter_; }
+	docstring const & prettyCounter(bool lc = false) const { return lc ? pretty_counter_lc_ : pretty_counter_; }
 	///
-	void setCounterValue(docstring cv) { counter_value_ = cv; }
+	docstring formattedCounter(bool lc = false, bool pl = false) const;
 	///
-	void setPrettyCounter(docstring pc) { pretty_counter_ = pc; }
+	void setCounterValue(docstring const & cv) { counter_value_ = cv; }
 	///
-	void setFormattedCounter(docstring fc) { formatted_counter_ = fc; }
+	void setPrettyCounter(docstring const & pc) { pretty_counter_ = pc; }
+	/** Formatted counter definition: lowercase singular (\p ls), lowercase plural (\p lp),
+	    uppercase singular (\p us), and uppercase plural (\p up) */
+	void setFormattedCounter(docstring const & ls, docstring const & lp,
+				 docstring const & us, docstring const & up);
 	///
 	int rowFlags() const override { return CanBreakBefore | CanBreakAfter; }
 	/// Updates only the label string, doesn't handle undo nor references.
-	void updateLabel(docstring const & new_label, bool const active = true);
+	void updateLabel(docstring const & new_label, bool active = true);
 	/// Updates the label and the references to it.
 	/// Will also handle undo/redo if \p cursor is passed.
-	void updateLabelAndRefs(docstring const & new_label, Cursor * cursor = 0);
+	void updateLabelAndRefs(docstring const & new_label, Cursor * cursor = nullptr);
 
 	/// \name Public functions inherited from Inset class
 	//@{
@@ -65,7 +69,7 @@ public:
 	///
 	docstring xhtml(XMLStream &, OutputParams const &) const override;
 	///
-	void updateBuffer(ParIterator const & it, UpdateType, bool const deleted = false) override;
+	void updateBuffer(ParIterator const & it, UpdateType, bool deleted = false) override;
 	///
 	void addToToc(DocIterator const & di, bool output_active,
 				  UpdateType utype, TocBackend & backend) const override;
@@ -108,7 +112,7 @@ private:
 	void uniqueLabel(docstring & label) const;
 	///
 	void updateReferences(docstring const & old_label,
-		docstring const & new_label, bool const changes);
+		docstring const & new_label, bool changes);
 	///
 	docstring screen_label_;
 	///
@@ -119,6 +123,16 @@ private:
 	docstring pretty_counter_;
 	///
 	docstring formatted_counter_;
+	///
+	docstring pretty_counter_lc_;
+	///
+	docstring formatted_counter_lc_;
+	///
+	docstring formatted_counter_pl_;
+	///
+	docstring formatted_counter_lc_pl_;
+	///
+	docstring textref_;
 };
 
 
