@@ -20,8 +20,6 @@
 #include "RenderPreview.h"
 #include "texstream.h"
 
-#include "frontends/Painter.h"
-
 #include "graphics/PreviewImage.h"
 
 #include "mathed/InsetMathHull.h"
@@ -72,13 +70,6 @@ void InsetPreview::write(ostream & os) const
 }
 
 
-void InsetPreview::addPreview(DocIterator const & inset_pos,
-	graphics::PreviewLoader &) const
-{
-	preparePreview(inset_pos);
-}
-
-
 MacroNameSet gatherMacroDefinitions(const Buffer* buffer, const Inset * inset)
 {
 	// Collect macros for this inset.
@@ -123,6 +114,13 @@ docstring insetToLaTeXSnippet(const Buffer* buffer, const Inset * inset)
 }
 
 
+void InsetPreview::addPreview(DocIterator const & inset_pos,
+	graphics::PreviewLoader &) const
+{
+	preparePreview(inset_pos);
+}
+
+
 void InsetPreview::preparePreview(DocIterator const & pos) const
 {
 	docstring const snippet = insetToLaTeXSnippet(pos.buffer(), this);
@@ -148,34 +146,6 @@ void InsetPreview::reloadPreview(DocIterator const & pos) const
 }
 
 
-void InsetPreview::draw(PainterInfo & pi, int x, int y) const
-{
-	if (previewState(pi.base.bv)) {
-		// one pixel gap in front
-		preview_->draw(pi, x + 1, y);
-	} else
-		InsetText::draw(pi, x, y);
-}
-
-
-void InsetPreview::edit(Cursor & cur, bool front, EntryDirection entry_from)
-{
-	cur.push(*this);
-	InsetText::edit(cur, front, entry_from);
-}
-
-
-Inset * InsetPreview::editXY(Cursor & cur, int x, int y)
-{
-	if (previewState(&cur.bv())) {
-		edit(cur, true, ENTRY_DIRECTION_IGNORE);
-		return this;
-	}
-	cur.push(*this);
-	return InsetText::editXY(cur, x, y);
-}
-
-
 void InsetPreview::metrics(MetricsInfo & mi, Dimension & dim) const
 {
 	if (previewState(mi.base.bv)) {
@@ -197,6 +167,61 @@ void InsetPreview::metrics(MetricsInfo & mi, Dimension & dim) const
 }
 
 
+void InsetPreview::draw(PainterInfo & pi, int x, int y) const
+{
+	if (previewState(pi.base.bv)) {
+		// one pixel gap in front
+		preview_->draw(pi, x + 1, y);
+	} else
+		InsetText::draw(pi, x, y);
+}
+
+
+void InsetPreview::doDispatch(Cursor & cur, FuncRequest & cmd)
+{
+	bool const has_preview = previewState(&cur.bv());
+	InsetText::doDispatch(cur, cmd);
+	if (&cur.inset() == this && has_preview != previewState(&cur.bv()))
+		// FIXME : it should be possible to trigger a SinglePar update
+		//   on the parent paragraph.
+		cur.screenUpdateFlags(Update::Force);
+}
+
+
+void InsetPreview::edit(Cursor & cur, bool front, EntryDirection entry_from)
+{
+	bool const has_preview = previewState(&cur.bv());
+	cur.push(*this);
+	InsetText::edit(cur, front, entry_from);
+	if (has_preview) {
+		// The inset contents dimension is in general different from the
+		// one of the instant preview image, so we have to indicate to the
+		// BufferView that a metrics update is needed.
+		// FIXME : it should be possible to trigger a SinglePar update
+		//   on the parent paragraph.
+		cur.screenUpdateFlags(Update::Force);
+	}
+}
+
+
+Inset * InsetPreview::editXY(Cursor & cur, int x, int y)
+{
+	if (previewState(&cur.bv())) {
+		edit(cur, true, ENTRY_DIRECTION_IGNORE);
+		return this;
+	}
+	cur.push(*this);
+	return InsetText::editXY(cur, x, y);
+}
+
+
+bool InsetPreview::notifyCursorEnters(Cursor const & old, Cursor & cur)
+{
+	cur.screenUpdateFlags(Update::Force);
+	return InsetText::notifyCursorEnters(old, cur);
+}
+
+
 bool InsetPreview::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 {
 	reloadPreview(old);
@@ -205,4 +230,12 @@ bool InsetPreview::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 }
 
 
+void InsetPreview::notifyMouseSelectionDone(Cursor & cur)
+{
+	reloadPreview(cur.realAnchor());
+	cur.screenUpdateFlags(Update::Force);
+	return InsetText::notifyMouseSelectionDone(cur);
+}
+
+
 } // namespace lyx
