@@ -12,8 +12,6 @@
 
 #include "InsetText.h"
 
-#include "mathed/MacroTable.h"
-
 #include "insets/InsetArgument.h"
 #include "insets/InsetLayout.h"
 #include "insets/InsetPreview.h"
@@ -21,26 +19,22 @@
 #include "graphics/PreviewImage.h"
 #include "graphics/PreviewLoader.h"
 
-#include "buffer_funcs.h"
 #include "Buffer.h"
 #include "BufferParams.h"
 #include "BufferView.h"
 #include "CompletionList.h"
 #include "CoordCache.h"
 #include "Cursor.h"
-#include "CutAndPaste.h"
 #include "DispatchResult.h"
 #include "ErrorList.h"
 #include "Exporter.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
 #include "InsetList.h"
-#include "Intl.h"
 #include "Language.h"
 #include "Layout.h"
 #include "LaTeXFeatures.h"
-#include "Lexer.h"
-#include "lyxfind.h"
+#include "support/Lexer.h"
 #include "LyXRC.h"
 #include "MetricsInfo.h"
 #include "output_docbook.h"
@@ -50,6 +44,7 @@
 #include "Paragraph.h"
 #include "ParagraphParameters.h"
 #include "ParIterator.h"
+#include "Statistics.h"
 #include "TexRow.h"
 #include "texstream.h"
 #include "TextClass.h"
@@ -58,10 +53,8 @@
 #include "TocBackend.h"
 #include "TocBuilder.h"
 
-#include "frontends/alert.h"
 #include "frontends/Painter.h"
 
-#include "support/convert.h"
 #include "support/debug.h"
 #include "support/filetools.h"
 #include "support/gettext.h"
@@ -98,6 +91,18 @@ InsetText::InsetText(InsetText const & in)
 }
 
 
+InsetText & InsetText::operator=(InsetText const & in)
+{
+	Inset::operator=(in);
+	drawFrame_ = in.drawFrame_;
+	is_changed_ = in.is_changed_;
+	intitle_context_ = false;
+	frame_color_ = in.frame_color_;
+	text_ = Text(this, in.text_);
+	return *this;
+}
+
+
 void InsetText::setBuffer(Buffer & buf)
 {
 	ParagraphList::iterator end = paragraphs().end();
@@ -412,6 +417,16 @@ bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 		return true;
 	}
 
+	// The below can be called when the math panel is on in the text mode
+	case LFUN_MATH_INSERT:
+		if (!cmd.argument().empty()) {
+			status.setEnabled(
+			            insetAllowed(
+			                insetCode(
+			                    to_utf8("math" + ltrim(cmd.argument(), "\\")))));
+			return true;
+		}
+	// fall through
 	default:
 		// Dispatch only to text_ if the cursor is inside
 		// the text_. It is not for context menus (bug 5797).
@@ -547,9 +562,14 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 		rp.moving_arg = true;
 	if (il.isNeedMBoxProtect())
 		++rp.inulemcmd;
+	if (!il.escapeChars().empty())
+		rp.escape_chars = il.escapeChars();
 	if (!il.passThruChars().empty())
 		rp.pass_thru_chars += il.passThruChars();
-	if (il.noCProtect())
+	if (!il.noPassThruChars().empty())
+		rp.no_pass_thru_chars += il.noPassThruChars();
+	// cprotect generally does not work in tables
+	if (il.noCProtect() || lyxCode() == CELL_CODE)
 		rp.no_cprotect = true;
 	if (!il.newlineCmd().empty())
 		rp.newlinecmd = il.newlineCmd();
@@ -841,7 +861,8 @@ void InsetText::docbookText(XMLStream & xs, OutputParams const & rp, XHTMLOption
 
 docstring InsetText::xhtml(XMLStream & xs, OutputParams const & runparams) const
 {
-	return insetAsXHTML(xs, runparams, WriteEverything);
+	insetAsXHTML(xs, runparams, WriteEverything);
+	return docstring();
 }
 
 
@@ -858,7 +879,7 @@ docstring InsetText::xhtml(XMLStream & xs, OutputParams const & runparams) const
 // if so, try to close fonts, etc.
 // There are probably limits to how well we can do here, though, and we will
 // have to rely upon users not putting footnotes inside noun-type insets.
-docstring InsetText::insetAsXHTML(XMLStream & xs, OutputParams const & rp,
+void InsetText::insetAsXHTML(XMLStream & xs, OutputParams const & rp,
                                   XHTMLOptions opts) const
 {
 	// we will always want to output all our paragraphs when we are
@@ -871,7 +892,7 @@ docstring InsetText::insetAsXHTML(XMLStream & xs, OutputParams const & rp,
 		xs.startDivision(false);
 		xhtmlParagraphs(text_, buffer(), xs, runparams);
 		xs.endDivision();
-		return docstring();
+		return;
 	}
 
 	InsetLayout const & il = getLayout();
@@ -895,8 +916,11 @@ docstring InsetText::insetAsXHTML(XMLStream & xs, OutputParams const & rp,
 		}
 	}
 
-	if (opts & WriteInnerTag)
-		xs << xml::StartTag(il.htmlinnertag(), il.htmlinnerattr());
+	if (opts & WriteInnerTag) {
+		string inner_attr = il.htmlinnerattr();
+		inner_attr = subst(inner_attr, "$$content", to_utf8(text_.asString(AS_STR_INSETS)));
+		xs << xml::StartTag(il.htmlinnertag(), inner_attr);
+	}
 
 	// we will eventually lose information about the containing inset
 	if (!allowMultiPar() || opts == JustText)
@@ -913,8 +937,6 @@ docstring InsetText::insetAsXHTML(XMLStream & xs, OutputParams const & rp,
 
 	if (opts & WriteOuterTag)
 		xs << xml::EndTag(il.htmltag());
-
-	return docstring();
 }
 
 
@@ -1051,6 +1073,9 @@ bool InsetText::insetAllowed(InsetCode code) const
 	// These are only allowed in index insets
 	case INDEXMACRO_CODE:
 	case INDEXMACRO_SORTKEY_CODE:
+	// The below cannot be called from the text mode directly
+	case MATH_INTERTEXT_CODE:
+	case MATH_SHORTINTERTEXT_CODE:
 		return false;
 	default:
 		return !isPassThru();
@@ -1139,6 +1164,12 @@ void InsetText::updateBuffer(ParIterator const & it, UpdateType utype, bool cons
 }
 
 
+void InsetText::updateStatistics(Statistics & stats) const
+{
+		stats.update(text());
+}
+
+
 void InsetText::toString(odocstream & os) const
 {
 	os << text().asString(0, 1, AS_STR_LABEL | AS_STR_INSETS);
@@ -1282,11 +1313,11 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 
 	// find text inset in old cursor
 	Cursor insetCur = old;
-	int scriptSlice	= insetCur.find(this);
+	size_type scriptSlice = insetCur.find(this);
 	// we can try to continue here. returning true means
 	// the cursor is "now" invalid. which it was.
-	LASSERT(scriptSlice != -1, return true);
-	insetCur.cutOff(scriptSlice);
+	LASSERT(scriptSlice != lyx::npos, return true);
+	insetCur.resize(scriptSlice + 1);
 	LASSERT(&insetCur.inset() == this, return true);
 
 	// update the old paragraph's words
