@@ -23,7 +23,7 @@ public:
 	///
 	explicit InsetMathComment(Buffer * buf);
 	///
-	explicit InsetMathComment(MathData const & ar);
+	explicit InsetMathComment(MathData const & md);
 	///
 	InsetMathComment(Buffer * buf, docstring const &);
 	///
@@ -36,7 +36,7 @@ public:
 	void drawT(TextPainter & pi, int x, int y) const override;
 
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	void maple(MapleStream &) const override;
 	///
