@@ -34,7 +34,7 @@ public:
 	///
 	void draw(PainterInfo &, int x, int y) const override;
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	void metrics(MetricsInfo & mi, Dimension & dim) const override;
 	///
