@@ -28,8 +28,6 @@
 #include "CoordCache.h"
 #include "Cursor.h"
 #include "Encoding.h"
-#include "FuncStatus.h"
-#include "FuncRequest.h"
 #include "LaTeXFeatures.h"
 #include "LyX.h"
 #include "LyXRC.h"
@@ -66,7 +64,7 @@ public:
 		: InsetMath(&mathMacro->buffer()), mathMacro_(mathMacro), idx_(idx),
 		  def_(&mathMacro->buffer())
 	{
-			asArray(def, def_);
+			asMathData(def, def_);
 	}
 	///
 	void setBuffer(Buffer & buffer) override
@@ -101,7 +99,7 @@ public:
 
 		MathRow::Element e_beg(mi, MathRow::BEGIN);
 		e_beg.inset = this;
-		e_beg.ar = &mathMacro_->cell(idx_);
+		e_beg.md = &mathMacro_->cell(idx_);
 		mrow.push_back(e_beg);
 
 		mathMacro_->macro()->unlock();
@@ -120,7 +118,7 @@ public:
 
 		MathRow::Element e_end(mi, MathRow::END);
 		e_end.inset = this;
-		e_end.ar = &mathMacro_->cell(idx_);
+		e_end.md = &mathMacro_->cell(idx_);
 		mrow.push_back(e_end);
 
 		return has_contents;
@@ -286,17 +284,17 @@ void InsetMathMacro::Private::updateChildren(InsetMathMacro * owner)
 void InsetMathMacro::Private::updateNestedChildren(InsetMathMacro * owner, InsetMathNest * ni)
 {
 	for (size_t i = 0; i < ni->nargs(); ++i) {
-		MathData & ar = ni->cell(i);
-		for (size_t j = 0; j < ar.size(); ++j) {
+		MathData & md = ni->cell(i);
+		for (size_t j = 0; j < md.size(); ++j) {
 			InsetArgumentProxy * ap = dynamic_cast
-				<InsetArgumentProxy *>(ar[j].nucleus());
+				<InsetArgumentProxy *>(md[j].nucleus());
 			if (ap) {
 				InsetMathMacro::Private * md = ap->owner()->d;
 				if (md->macro_)
 					md->macro_ = &md->macroBackup_;
 				ap->setOwner(owner);
 			}
-			InsetMathNest * imn = ar[j].nucleus()->asNestInset();
+			InsetMathNest * imn = md[j].nucleus()->asNestInset();
 			if (imn)
 				updateNestedChildren(owner, imn);
 		}
@@ -364,7 +362,7 @@ bool InsetMathMacro::addToMathRow(MathRow & mrow, MetricsInfo & mi) const
 	bool has_contents = d->expanded_.addToMathRow(mrow, mi);
 	d->macro_->unlock();
 
-	// if there was no contents and the array is editable, then we
+	// if there was no contents and the cell is editable, then we
 	// insert a grey box instead.
 	if (!has_contents && mi.base.macro_nesting == 1) {
 		// mathclass is unknown because it is irrelevant for spacing
@@ -492,7 +490,7 @@ docstring InsetMathMacro::name() const
 }
 
 
-docstring InsetMathMacro::macroName() const
+docstring const & InsetMathMacro::macroName() const
 {
 	return d->name_;
 }
@@ -526,7 +524,7 @@ bool InsetMathMacro::editMode(BufferView const * bv) const {
 			for (; i != cur.depth(); ++i) {
 				InsetMath * im = cur[i].asInsetMath();
 				if (im) {
-					InsetMathMacro const * macro = im->asMacro();
+					InsetMathMacro const * macro = im->asMacroInset();
 					if (macro && macro->displayMode() == DISPLAY_NORMAL)
 						return false;
 				}
@@ -554,7 +552,7 @@ bool InsetMathMacro::editMetrics(BufferView const * bv) const
 
 marker_type InsetMathMacro::marker(BufferView const * bv) const
 {
-	if (nargs() == 0)
+	if (!isActive())
 		return marker_type::NO_MARKER;
 
 	switch (d->displayMode_) {
@@ -573,7 +571,6 @@ marker_type InsetMathMacro::marker(BufferView const * bv) const
 			return marker_type::MARKER2;
 		}
 	}
-	// please gcc 4.6
 	return marker_type::NO_MARKER;
 }
 
@@ -684,6 +681,9 @@ private:
 */
 #define UpdateLocker(x) unnamed_UpdateLocker;
 // Tip gotten from Bobby Schmidt's column in C/C++ Users Journal
+// Now we no-op use it to avoid -Wunused-macros
+#if defined UpdateLocker
+#endif
 
 
 void InsetMathMacro::updateRepresentation(Cursor * cur, MacroContext const & mc,
@@ -741,7 +741,7 @@ void InsetMathMacro::updateRepresentation(Cursor * cur, MacroContext const & mc,
 			d->expanded_.updateMacros(cur, mc, utype, nesting);
 	}
 	// get definition for list edit mode
-	asArray(display.empty() ? d->macro_->definition() : display,
+	asMathData(display.empty() ? d->macro_->definition() : display,
 		d->definition_, Parse::QUIET | Parse::MACRODEF);
 }
 
@@ -782,7 +782,7 @@ void InsetMathMacro::draw(PainterInfo & pi, int x, int y) const
 
 		// draw definition
 		d->definition_.draw(pi, x, y);
-		Dimension const & defDim = coords.getArrays().dim(&d->definition_);
+		Dimension const & defDim = coords.cells().dim(&d->definition_);
 		y += max(fontDim.des, defDim.des);
 
 		// draw parameters
@@ -792,7 +792,7 @@ void InsetMathMacro::draw(PainterInfo & pi, int x, int y) const
 
 		for (idx_type i = 0; i < nargs(); ++i) {
 			// position of label
-			Dimension const & cdim = coords.getArrays().dim(&cell(i));
+			Dimension const & cdim = coords.cells().dim(&cell(i));
 			x = expx + 1;
 			y += max(fontDim.asc, cdim.asc) + 1;
 
@@ -829,7 +829,7 @@ void InsetMathMacro::setDisplayMode(InsetMathMacro::DisplayMode mode, int appeti
 		// transfer name if changing from or to DISPLAY_UNFOLDED
 		if (mode == DISPLAY_UNFOLDED) {
 			cells_.resize(1, MathData(buffer_));
-			asArray(d->name_, cell(0));
+			asMathData(d->name_, cell(0));
 		} else if (d->displayMode_ == DISPLAY_UNFOLDED) {
 			d->name_ = asString(cell(0));
 			cells_.clear();
@@ -865,16 +865,14 @@ bool InsetMathMacro::validName() const
 
 	// converting back and force doesn't swallow anything?
 	/*MathData ma;
-	asArray(n, ma);
+	asMathData(n, ma);
 	if (asString(ma) != n)
 		return false;*/
 
 	// valid characters?
 	if (n.size() > 1) {
 		for (char_type c : n) {
-			if (!(c >= 'a' && c <= 'z')
-			    && !(c >= 'A' && c <= 'Z')
-			    && c != '*')
+			if (!isAlphaASCII(c) && c != '*')
 				return false;
 		}
 	}
@@ -993,9 +991,9 @@ void InsetMathMacro::validate(LaTeXFeatures & features) const
 				d->definition_.validate(features);
 		} else if (displayMode() == DISPLAY_INIT) {
 			if (MacroData const * data = buffer().getMacro(name())) {
-				MathData ar(const_cast<Buffer *>(&buffer()));
-				asArray(data->definition(), ar);
-				ar.validate(features);
+				MathData md(const_cast<Buffer *>(&buffer()));
+				asMathData(data->definition(), md);
+				md.validate(features);
 			}
 		}
 	}
@@ -1014,7 +1012,8 @@ void InsetMathMacro::edit(Cursor & cur, bool front, EntryDirection entry_from)
 Inset * InsetMathMacro::editXY(Cursor & cur, int x, int y)
 {
 	// We may have 0 arguments, but InsetMathNest requires at least one.
-	if (nargs() > 0) {
+	if (isActive()) {
+		// FIXME: do we need this? Otherwise the method could go
 		cur.screenUpdateFlags(Update::SinglePar);
 		return InsetMathNest::editXY(cur, x, y);
 	} else
@@ -1108,11 +1107,11 @@ bool InsetMathMacro::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 		if (unfolded_name != d->name_) {
 			// The macro name was changed
 			Cursor inset_cursor = old;
-			int macroSlice = inset_cursor.find(this);
+			size_type macroSlice = inset_cursor.find(this);
 			// returning true means the cursor is "now" invalid,
 			// which it was.
-			LASSERT(macroSlice != -1, return true);
-			inset_cursor.cutOff(macroSlice);
+			LASSERT(macroSlice != lyx::npos, return true);
+			inset_cursor.resize(macroSlice + 1);
 			inset_cursor.recordUndoInset();
 			inset_cursor.pop();
 			inset_cursor.cell().erase(inset_cursor.pos());
@@ -1152,7 +1151,7 @@ bool InsetMathMacro::folded() const
 }
 
 
-void InsetMathMacro::write(TeXMathStream & os) const
+void InsetMathMacro::writeMath(TeXMathStream & os) const
 {
 	mode_type mode = currentMode();
 	MathEnsurer ensurer(os, mode == MATH_MODE, true, mode == TEXT_MODE);
@@ -1248,7 +1247,7 @@ void InsetMathMacro::write(TeXMathStream & os) const
 			braced = cell(i)[0]->asScriptInset()->nuc().empty();
 		} else {
 			for (size_type j = 0; j < cell(i).size(); ++j) {
-				InsetMathMacro const * ma = cell(i)[j]->asMacro();
+				InsetMathMacro const * ma = cell(i)[j]->asMacroInset();
 				if (ma && ma->optionals()) {
 					braced = true;
 					break;
@@ -1314,7 +1313,7 @@ void InsetMathMacro::mathmlize(MathMLStream & ms) const
 	if (d->macro_) {
 		docstring const xmlname = d->macro_->xmlname();
 		if (!xmlname.empty()) {
-			char const * type = d->macro_->MathMLtype();
+			const std::string type = d->macro_->mathml_type();
 			ms << MTagInline(type) << xmlname << ETagInline(type);
 			return;
 		}
@@ -1443,7 +1442,7 @@ bool InsetMathMacro::insertCompletion(Cursor & cur, docstring const & s, bool fi
 
 	// append completion
 	docstring newName = name() + s;
-	asArray(newName, cell(0));
+	asMathData(newName, cell(0));
 	cur.bv().cursor().pos() = name().size();
 	cur.screenUpdateFlags(Update::SinglePar);
 
@@ -1472,10 +1471,9 @@ void InsetMathMacro::completionPosAndDim(Cursor const & cur, int & x, int & y,
 	dim.asc += 3;
 
 	// and position
-	Point xy
-	= cur.bv().coordCache().insets().xy(this);
-	x = xy.x_;
-	y = xy.y_;
+	Point xy = cur.bv().coordCache().insets().xy(this);
+	x = xy.x;
+	y = xy.y;
 }
 
 
