@@ -16,6 +16,7 @@
 #include "MathStream.h"
 
 #include "Cursor.h"
+#include "Dimension.h"
 #include "LaTeXFeatures.h"
 #include "MetricsInfo.h"
 
@@ -88,7 +89,7 @@ bool InsetMathOverset::idxLast(Cursor & cur) const
 }
 
 
-void InsetMathOverset::write(TeXMathStream & os) const
+void InsetMathOverset::writeMath(TeXMathStream & os) const
 {
 	MathEnsurer ensurer(os);
 	if (os.fragile())
