@@ -34,7 +34,7 @@ public:
 	///
 	bool idxLast(Cursor &) const override;
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	void normalize(NormalStream &) const override;
 	///
