@@ -13,6 +13,7 @@
 #include "InsetMathScript.h"
 
 #include "InsetMathBrace.h"
+#include "InsetMathChar.h"
 #include "MathData.h"
 #include "MathStream.h"
 #include "MathSupport.h"
@@ -23,7 +24,6 @@
 #include "MetricsInfo.h"
 
 #include "support/debug.h"
-#include "support/gettext.h"
 #include "support/lassert.h"
 
 
@@ -505,23 +505,17 @@ bool InsetMathScript::idxUpDown(Cursor & cur, bool up) const
 }
 
 
-void InsetMathScript::write(TeXMathStream & os) const
+void InsetMathScript::writeMath(TeXMathStream & os) const
 {
 	MathEnsurer ensurer(os);
 
-	if (!nuc().empty()) {
+	if (!nuc().empty())
 		os << nuc();
-		// Avoid double superscript errors (bug 1633)
-		if (os.latex() && hasUp() && nuc().back()->getChar() == '\'')
-			os << "{}";
-	} else if (os.firstitem())
+	else if (os.firstitem())
 		LYXERR(Debug::MATHED, "suppressing {} when writing");
 	else
 		os << "{}";
 
-	if (hasDown() /*&& !down().empty()*/)
-		os << "_{" << down() << '}';
-
 	if (hasUp() /*&& !up().empty()*/) {
 		// insert space if up() is empty or an empty brace inset
 		// (see bug 8305)
@@ -529,15 +523,20 @@ void InsetMathScript::write(TeXMathStream & os) const
 		    (up().size() == 1 && up().back()->asBraceInset() &&
 		     up().back()->asBraceInset()->cell(0).empty())))
 			os << "^ {}";
-		else {
+		else
 			os << "^{" << up() << '}';
-			// Avoid double superscript errors by writing an
-			// empty group {} when a prime immediately follows
-			if (os.latex())
-				os.useBraces(true);
-		}
 	}
 
+	if (hasDown() /*&& !down().empty()*/)
+		os << "_{" << down() << '}';
+
+	// Avoid double superscript errors by writing an
+	// empty group {} when a prime immediately follows
+	if (os.latex() && (hasUp() ||
+	    (hasDown() && !nuc().empty() && nuc().back()->asCharInset() &&
+	     nuc().back()->asCharInset()->getChar() == '\'')))
+		os.useBraces(true);
+
 	if (lock_ && !os.latex())
 		os << "\\lyxlock ";
 }
@@ -716,9 +715,9 @@ bool InsetMathScript::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 		// it does not necessarily point to us anymore. But we
 		// should be on top of the cursor old.
 		Cursor insetCur = old;
-		int scriptSlice	= insetCur.find(this);
-		LASSERT(scriptSlice != -1, /**/);
-		insetCur.cutOff(scriptSlice);
+		size_type scriptSlice	= insetCur.find(this);
+		LASSERT(scriptSlice != lyx::npos, /**/);
+		insetCur.resize(scriptSlice + 1);
 		insetCur.recordUndoInset();
 
 		// Let the script inset commit suicide. This is
@@ -726,10 +725,10 @@ bool InsetMathScript::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 		// invoke notifyCursorLeaves again and does not touch
 		// cur (since the top slice will be deleted
 		// afterwards)
-		MathData ar = cell(0);
+		MathData md = cell(0);
 		insetCur.pop();
 		insetCur.cell().erase(insetCur.pos());
-		insetCur.cell().insert(insetCur.pos(), ar);
+		insetCur.cell().insert(insetCur.pos(), md);
 
 		// redraw
 		cur.screenUpdateFlags(cur.result().screenUpdate() | Update::SinglePar);
