@@ -35,7 +35,7 @@ public:
 	void draw(PainterInfo &, int x, int y) const override;
 
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	void normalize(NormalStream &) const override;
 	///
