@@ -32,7 +32,7 @@ public:
 	///
 	void draw(PainterInfo & pi, int x, int y) const override;
 	///
-	docstring str() const { return str_; }
+	docstring const & str() const { return str_; }
 	///
 	InsetMathString * asStringInset() override { return this; }
 	///
@@ -49,7 +49,7 @@ public:
 	///
 	void mathmlize(MathMLStream &) const override;
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	InsetCode lyxCode() const override { return MATH_STRING_CODE; }
 
