@@ -29,6 +29,8 @@ public:
 	///
 	void draw(PainterInfo & pi, int x, int y) const override;
 	///
+	InsetMathSubstack * asSubstackInset() override { return this; }
+	///
 	InsetMathSubstack const * asSubstackInset() const override { return this; }
 
 	///
@@ -39,7 +41,7 @@ public:
 	///
 	void infoize(odocstream & os) const override;
 	///
-	void write(TeXMathStream & os) const override;
+	void writeMath(TeXMathStream & os) const override;
 	///
 	void mathmlize(MathMLStream &) const override;
 	///
