@@ -18,11 +18,8 @@
 #include "MathAtom.h"
 #include "MathClass.h"
 
-#include "Dimension.h"
 #include "OutputEnums.h"
 
-#include "support/strfwd.h"
-
 #include <cstddef>
 #include <vector>
 
@@ -78,14 +75,16 @@ public:
 	///
 	Buffer const * buffer() const { return buffer_; }
 	///
-	void append(MathData const & ar);
+	void append(MathData const & md);
 
 	/// inserts single atom at position pos
 	void insert(size_type pos, MathAtom const & at);
 	/// inserts multiple atoms at position pos
-	void insert(size_type pos, MathData const & ar);
+	void insert(size_type pos, MathData const & md);
 	/// inserts single atom at end
 	void push_back(MathAtom const & at);
+	/// construct single atom at end
+	void emplace_back(InsetMath * ins);
 
 	/// erase range from pos1 to pos2
 	void erase(iterator pos1, iterator pos2);
@@ -106,15 +105,15 @@ public:
 	void substitute(MathData const & m);
 
 	/// looks for exact match
-	bool match(MathData const & ar) const;
+	bool match(MathData const & md) const;
 	/// looks for inclusion match starting at pos
-	bool matchpart(MathData const & ar, pos_type pos) const;
+	bool matchpart(MathData const & md, pos_type pos) const;
 	/// looks for containment, return == size mean not found
-	size_type find(MathData const & ar) const;
+	size_type find(MathData const & md) const;
 	/// looks for containment, return == size mean not found
-	size_type find_last(MathData const & ar) const;
+	size_type find_last(MathData const & md) const;
 	///
-	bool contains(MathData const & ar) const;
+	bool contains(MathData const & md) const;
 	///
 	void validate(LaTeXFeatures &) const;
 
@@ -123,7 +122,7 @@ public:
 	/// checked read access
 	MathAtom const & operator[](pos_type) const;
 
-	/// Add this array to a math row. Return true if contents got added
+	/// Add this data to a math row. Return true if contents got added
 	bool addToMathRow(MathRow &, MetricsInfo & mi) const;
 
 	/// rebuild cached metrics information
@@ -156,13 +155,9 @@ public:
 	int xo(BufferView const & bv) const;
 	/// access to cached y coordinate of last drawing
 	int yo(BufferView const & bv) const;
-	/// access to cached x coordinate of mid point of last drawing
-	int xm(BufferView const & bv) const;
-	/// access to cached y coordinate of mid point of last drawing
-	int ym(BufferView const & bv) const;
 	/// write access to coordinate;
 	void setXY(BufferView & bv, int x, int y) const;
-	/// returns x coordinate of given position in the array
+	/// returns x coordinate of given position in the data
 	int pos2x(BufferView const * bv, size_type pos) const;
 	/// returns position of given x coordinate
 	size_type x2pos(BufferView const * bv, int targetx) const;
@@ -181,7 +176,7 @@ public:
 	/// Italic correction as described in InsetMathScript.h
 	int kerning(BufferView const *) const;
 	///
-	void swap(MathData & ar) { base_type::swap(ar); }
+	void swap(MathData & md) { base_type::swap(md); }
 
 	/// attach/detach arguments to macros, updating the cur to
 	/// stay visually at the same position (cur==0 is allowed)
@@ -205,32 +200,32 @@ protected:
 
 private:
 	/// is this an exact match at this position?
-	bool find1(MathData const & ar, size_type pos) const;
+	bool find1(MathData const & md, size_type pos) const;
 
 	///
-	void detachMacroParameters(DocIterator * dit, const size_type macroPos);
+	void detachMacroParameters(DocIterator * dit, size_type macroPos);
 	///
-	void attachMacroParameters(Cursor * cur, const size_type macroPos,
-		const size_type macroNumArgs, const int macroOptionals,
-		const bool fromInitToNormalMode, const bool interactiveInit,
-		const size_t appetite);
+	void attachMacroParameters(Cursor * cur, size_type macroPos,
+		size_type macroNumArgs, int macroOptionals,
+		bool fromInitToNormalMode, bool interactiveInit,
+		size_t appetite);
 	///
 	void collectOptionalParameters(Cursor * cur,
-		const size_type numOptionalParams, std::vector<MathData> & params,
+		size_type numOptionalParams, std::vector<MathData> & params,
 		size_t & pos, MathAtom & scriptToPutAround,
-		const pos_type macroPos, const int thisPos, const int thisSlice);
+		pos_type macroPos, pos_type thisPos, size_type thisSlice);
 	///
 	void collectParameters(Cursor * cur,
-		const size_type numParams, std::vector<MathData> & params,
+		size_type numParams, std::vector<MathData> & params,
 		size_t & pos, MathAtom & scriptToPutAround,
-		const pos_type macroPos, const int thisPos, const int thisSlice,
-		const size_t appetite);
+		pos_type macroPos, pos_type thisPos, size_type thisSlice,
+		size_t appetite);
 };
 
 ///
-std::ostream & operator<<(std::ostream & os, MathData const & ar);
+std::ostream & operator<<(std::ostream & os, MathData const & md);
 ///
-odocstream & operator<<(odocstream & os, MathData const & ar);
+odocstream & operator<<(odocstream & os, MathData const & md);
 
 
 } // namespace lyx
