@@ -72,11 +72,11 @@ following hack as starting point to write some macros:
 #include "Buffer.h"
 #include "BufferParams.h"
 #include "Encoding.h"
-#include "Lexer.h"
 
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/docstream.h"
+#include "support/Lexer.h"
 
 #include <cstdint>
 #include <sstream>
@@ -87,6 +87,8 @@ using namespace std;
 
 namespace lyx {
 
+using support::Lexer;
+
 namespace {
 
 InsetMath::mode_type asMode(InsetMath::mode_type oldmode, docstring const & str)
@@ -396,7 +398,7 @@ public:
 	///
 	bool parse(MathAtom & at);
 	///
-	bool parse(MathData & array, unsigned flags, mode_type mode);
+	bool parse(MathData & md, unsigned flags, mode_type mode);
 	///
 	bool parse1(InsetMathGrid & grid, unsigned flags, mode_type mode,
 		bool numbered);
@@ -722,19 +724,19 @@ void Parser::error(string const & msg)
 bool Parser::parse(MathAtom & at)
 {
 	skipSpaces();
-	MathData ar(buffer_);
-	parse(ar, false, InsetMath::UNDECIDED_MODE);
-	if (ar.size() != 1 || ar.front()->getType() == hullNone) {
+	MathData md(buffer_);
+	parse(md, false, InsetMath::UNDECIDED_MODE);
+	if (md.size() != 1 || md.front()->getType() == hullNone) {
 		if (!(mode_ & Parse::QUIET))
-			lyxerr << "unusual contents found: " << ar << endl;
-		at = MathAtom(new InsetMathPar(buffer_, ar));
+			lyxerr << "unusual contents found: " << md << endl;
+		at = MathAtom(new InsetMathPar(buffer_, md));
 		//if (at->nargs() > 0)
-		//	at.nucleus()->cell(0) = ar(buffer_);
+		//	at.nucleus()->cell(0) = md(buffer_);
 		//else
-		//	lyxerr << "unusual contents found: " << ar << endl;
+		//	lyxerr << "unusual contents found: " << md << endl;
 		success_ = false;
 	} else
-		at = ar[0];
+		at = md[0];
 	return success_;
 }
 
@@ -778,11 +780,11 @@ docstring Parser::parse_verbatim_item()
 }
 
 
-bool Parser::parse(MathData & array, unsigned flags, mode_type mode)
+bool Parser::parse(MathData & md, unsigned flags, mode_type mode)
 {
 	InsetMathGrid grid(buffer_, 1, 1);
 	parse1(grid, flags, mode, false);
-	array = grid.cell(0);
+	md = grid.cell(0);
 	return success_;
 }
 
@@ -851,9 +853,9 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 		if (flags & FLAG_OPTION) {
 			if (t.cat() == catOther && t.character() == '[') {
-				MathData ar(buf);
-				parse(ar, FLAG_BRACK_LAST, mode);
-				cell->append(ar);
+				MathData md(buf);
+				parse(md, FLAG_BRACK_LAST, mode);
+				cell->append(md);
 			} else {
 				// no option found, put back token and we are done
 				putback();
@@ -871,7 +873,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				if (n.cat() == catMath) {
 					// TeX's $$...$$ syntax for displayed math
 					if (mode == InsetMath::UNDECIDED_MODE) {
-						cell->push_back(MathAtom(new InsetMathHull(buf, hullEquation)));
+						cell->emplace_back(new InsetMathHull(buf, hullEquation));
 						parse2(cell->back(), FLAG_SIMPLE, InsetMath::MATH_MODE, false);
 						getToken(); // skip the second '$' token
 					} else {
@@ -884,11 +886,11 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					// simple $...$  stuff
 					putback();
 					if (mode == InsetMath::UNDECIDED_MODE) {
-						cell->push_back(MathAtom(new InsetMathHull(buf, hullSimple)));
+						cell->emplace_back(new InsetMathHull(buf, hullSimple));
 						parse2(cell->back(), FLAG_SIMPLE, InsetMath::MATH_MODE, false);
 					} else {
 						// Don't create nested math hulls (bug #5392)
-						cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+						cell->emplace_back(new InsetMathEnsureMath(buf));
 						parse(cell->back().nucleus()->cell(0), FLAG_SIMPLE, InsetMath::MATH_MODE);
 					}
 				}
@@ -910,47 +912,68 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					// are inside a text mode user macro. We have no way to tell, so
 					// let's play safe by using \ensuremath, as it will work in any case.
 					putback();
-					cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+					cell->emplace_back(new InsetMathEnsureMath(buf));
 					parse(cell->back().nucleus()->cell(0), FLAG_SIMPLE, InsetMath::MATH_MODE);
 				}
 			}
 		}
 
 		else if (t.cat() == catLetter)
-			cell->push_back(MathAtom(new InsetMathChar(buf, t.character())));
+			cell->emplace_back(new InsetMathChar(buf, t.character()));
 
 		else if (t.cat() == catSpace && mode != InsetMath::MATH_MODE) {
 			if (cell->empty() || cell->back()->getChar() != ' ')
-				cell->push_back(MathAtom(new InsetMathChar(buf, t.character())));
+				cell->emplace_back(new InsetMathChar(buf, t.character()));
 		}
 
 		else if (t.cat() == catNewline && mode != InsetMath::MATH_MODE) {
 			if (cell->empty() || cell->back()->getChar() != ' ')
-				cell->push_back(MathAtom(new InsetMathChar(buf, ' ')));
+				cell->emplace_back(new InsetMathChar(buf, ' '));
 		}
 
 		else if (t.cat() == catParameter) {
 			Token const & n	= nextToken();
 			char_type c = n.character();
 			if (c && '0' < c && c <= '9') {
-				cell->push_back(MathAtom(new InsetMathMacroArgument(buf, c - '0')));
+				cell->emplace_back(new InsetMathMacroArgument(buf, c - '0'));
 				getToken();
 			} else
-				cell->push_back(MathAtom(new InsetMathHash(buf)));
+				cell->emplace_back(new InsetMathHash(buf));
 		}
 
 		else if (t.cat() == catActive)
-			cell->push_back(MathAtom(new InsetMathSpace(buf, string(1, t.character()), "")));
+			cell->emplace_back(new InsetMathSpace(buf, string(1, t.character()), ""));
 
 		else if (t.cat() == catBegin) {
-			MathData ar(buf);
-			parse(ar, FLAG_BRACE_LAST, mode);
+			MathData md(buf);
+			parse(md, FLAG_BRACE_LAST, mode);
+			// Correctly set the label if this is a display inset.
+			// A label inside braces is parsed as a normal macro,
+			// so recover its argument stored as a verbatim string.
+			// This issue may occur when importing from latex
+			// (see bug 13195).
+			docstring label;
+			if (md.size() > 1 && grid.asHullInset()) {
+				for (size_t i = 0; i < md.size() - 1; ++i) {
+					InsetMathMacro * mm = md[i]->asMacroInset();
+					InsetMathBrace * mb = md[i + 1]->asBraceInset();
+					if (mm && mb && mm->name() == "label") {
+						MathData & ar = mb->cell(0);
+						if (ar.size() == 1 && ar[0]->asStringInset()) {
+							label = ar[0]->asStringInset()->str();
+							grid.asHullInset()->label(cellrow, label);
+							md.erase(i, i + 2);
+							break;
+						}
+					}
+				}
+			}
 			// do not create a BraceInset if they were written by LyX
 			// this helps to keep the annoyance of  "a choose b"  to a minimum
-			if (ar.size() == 1 && ar[0]->extraBraces())
-				cell->append(ar);
-			else
-				cell->push_back(MathAtom(new InsetMathBrace(buf, ar)));
+			if (md.size() == 1 && md[0]->extraBraces())
+				cell->append(md);
+			else if (md.size() > 0 || label.empty())
+				cell->emplace_back(new InsetMathBrace(buf, md));
 		}
 
 		else if (t.cat() == catEnd) {
@@ -975,12 +998,12 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			// we need no new script inset if the last thing was a scriptinset,
 			// which has that script already not the same script already
 			if (cell->empty())
-				cell->push_back(MathAtom(new InsetMathScript(buf, up)));
+				cell->emplace_back(new InsetMathScript(buf, up));
 			else if (cell->back()->asScriptInset() &&
 					!cell->back()->asScriptInset()->has(up))
 				cell->back().nucleus()->asScriptInset()->ensure(up);
 			else if (cell->back()->asScriptInset())
-				cell->push_back(MathAtom(new InsetMathScript(buf, up)));
+				cell->emplace_back(new InsetMathScript(buf, up));
 			else
 				cell->back() = MathAtom(new InsetMathScript(buf, cell->back(), up));
 			InsetMathScript * p = cell->back().nucleus()->asScriptInset();
@@ -1018,14 +1041,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			    || mode_ & Parse::VERBATIM
 			    || !(mode_ & Parse::USETEXT)
 			    || mode == InsetMath::TEXT_MODE) {
-				cell->push_back(MathAtom(new InsetMathChar(buf, c)));
+				cell->emplace_back(new InsetMathChar(buf, c));
 			} else {
 				MathAtom at = createInsetMath("text", buf);
-				at.nucleus()->cell(0).push_back(MathAtom(new InsetMathChar(buf, c)));
+				at.nucleus()->cell(0).emplace_back(new InsetMathChar(buf, c));
 				while (nextToken().cat() == catOther
 				       && Encodings::isUnicodeTextOnly(nextToken().character())) {
 					c = getToken().character();
-					at.nucleus()->cell(0).push_back(MathAtom(new InsetMathChar(buf, c)));
+					at.nucleus()->cell(0).emplace_back(new InsetMathChar(buf, c));
 				}
 				cell->push_back(at);
 			}
@@ -1039,7 +1062,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					break;
 				s += tt.asInput();
 			}
-			cell->push_back(MathAtom(new InsetMathComment(buf, s)));
+			cell->emplace_back(new InsetMathComment(buf, s));
 			skipSpaces();
 		}
 
@@ -1079,12 +1102,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
-				name, nargs, 0, MacroTypeDef,
-				vector<MathData>(), def, display)));
+			size_t const optionals = 0;
+			cell->emplace_back(new InsetMathMacroTemplate(buf,
+				name, nargs, optionals, MacroTypeDef,
+				vector<MathData>(), def, display));
 
 			if (buf && (mode_ & Parse::TRACKMACRO))
-				buf->usermacros.insert(name);
+				buf->usermacros[name] = optionals;
 		}
 
 		else if (t.cs() == "newcommand" ||
@@ -1127,12 +1151,12 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
+			cell->emplace_back(new InsetMathMacroTemplate(buf,
 				name, nargs, optionals, MacroTypeNewcommand,
-				optionalValues, def, display)));
+				optionalValues, def, display));
 
 			if (buf && (mode_ & Parse::TRACKMACRO))
-				buf->usermacros.insert(name);
+				buf->usermacros[name] = optionals;
 		}
 
 		else if (t.cs() == "newcommandx" ||
@@ -1248,21 +1272,21 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
+			cell->emplace_back(new InsetMathMacroTemplate(buf,
 				name, nargs, optionals, MacroTypeNewcommandx,
-				optionalValues, def, display)));
+				optionalValues, def, display));
 
 			if (buf && (mode_ & Parse::TRACKMACRO))
-				buf->usermacros.insert(name);
+				buf->usermacros[name] = optionals;
 		}
 
 		else if (t.cs() == "(") {
 			if (mode == InsetMath::UNDECIDED_MODE) {
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullSimple)));
+				cell->emplace_back(new InsetMathHull(buf, hullSimple));
 				parse2(cell->back(), FLAG_SIMPLE2, InsetMath::MATH_MODE, false);
 			} else {
 				// Don't create nested math hulls (bug #5392)
-				cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+				cell->emplace_back(new InsetMathEnsureMath(buf));
 				parse(cell->back().nucleus()->cell(0), FLAG_SIMPLE2, InsetMath::MATH_MODE);
 			}
 		}
@@ -1272,7 +1296,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				error("bad math environment [");
 				break;
 			}
-			cell->push_back(MathAtom(new InsetMathHull(buf, hullEquation)));
+			cell->emplace_back(new InsetMathHull(buf, hullEquation));
 			parse2(cell->back(), FLAG_EQUATION, InsetMath::MATH_MODE, false);
 		}
 
@@ -1398,7 +1422,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			} else {
 				MathAtom at = MathAtom(new InsetMathMacro(buf, t.cs()));
 				cell->push_back(at);
-				cell->push_back(MathAtom(new InsetMathBrace(buf, count)));
+				cell->emplace_back(new InsetMathBrace(buf, count));
 			}
 		}
 
@@ -1424,47 +1448,47 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		}
 
 		else if (t.cs() == "sqrt") {
-			MathData ar(buf);
-			parse(ar, FLAG_OPTION, mode);
-			if (!ar.empty()) {
-				cell->push_back(MathAtom(new InsetMathRoot(buf)));
-				cell->back().nucleus()->cell(1) = ar;
+			MathData md(buf);
+			parse(md, FLAG_OPTION, mode);
+			if (!md.empty()) {
+				cell->emplace_back(new InsetMathRoot(buf));
+				cell->back().nucleus()->cell(1) = md;
 			} else
-				cell->push_back(MathAtom(new InsetMathSqrt(buf)));
+				cell->emplace_back(new InsetMathSqrt(buf));
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 		}
 
 		else if (t.cs() == "cancelto") {
-			MathData ar(buf);
-			parse(ar, FLAG_ITEM, mode);
-				cell->push_back(MathAtom(new InsetMathCancelto(buf)));
-				cell->back().nucleus()->cell(1) = ar;
+			MathData md(buf);
+			parse(md, FLAG_ITEM, mode);
+				cell->emplace_back(new InsetMathCancelto(buf));
+				cell->back().nucleus()->cell(1) = md;
 				parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 		}
 
 		else if (t.cs() == "unit") {
 			// Allowed formats \unit[val]{unit}
-			MathData ar(buf);
-			parse(ar, FLAG_OPTION, mode);
-			if (!ar.empty()) {
-				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNIT)));
-				cell->back().nucleus()->cell(0) = ar;
+			MathData md(buf);
+			parse(md, FLAG_OPTION, mode);
+			if (!md.empty()) {
+				cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::UNIT));
+				cell->back().nucleus()->cell(0) = md;
 				parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
 			} else {
-				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNIT, 1)));
+				cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::UNIT, 1));
 				parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 			}
 		}
 
 		else if (t.cs() == "unitfrac") {
 			// Here allowed formats are \unitfrac[val]{num}{denom}
-			MathData ar(buf);
-			parse(ar, FLAG_OPTION, mode);
-			if (!ar.empty()) {
-				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNITFRAC, 3)));
-				cell->back().nucleus()->cell(2) = ar;
+			MathData md(buf);
+			parse(md, FLAG_OPTION, mode);
+			if (!md.empty()) {
+				cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::UNITFRAC, 3));
+				cell->back().nucleus()->cell(2) = md;
 			} else {
-				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNITFRAC)));
+				cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::UNITFRAC));
 			}
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 			parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
@@ -1475,11 +1499,11 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			docstring const arg = getArg('[', ']');
 			//lyxerr << "got so far: '" << arg << "'" << endl;
 				if (arg == "l")
-					cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::CFRACLEFT)));
+					cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::CFRACLEFT));
 				else if (arg == "r")
-					cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::CFRACRIGHT)));
+					cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::CFRACRIGHT));
 				else if (arg.empty() || arg == "c")
-					cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::CFRAC)));
+					cell->emplace_back(new InsetMathFrac(buf, InsetMathFrac::CFRAC));
 				else {
 					error("found invalid optional argument");
 					break;
@@ -1490,39 +1514,39 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 		else if (t.cs() == "sideset") {
 			// Here allowed formats are \sideset{_{bl}^{tl}}{_{br}^{tr}}{operator}
-			MathData ar[2]= { MathData(buf), MathData(buf) };
+			MathData md[2]= { MathData(buf), MathData(buf) };
 			InsetMathScript * script[2] = {0, 0};
 			for (int i = 0; i < 2; ++i) {
-				parse(ar[i], FLAG_ITEM, mode);
-				if (ar[i].size() == 1)
-					script[i] = ar[i][0].nucleus()->asScriptInset();
+				parse(md[i], FLAG_ITEM, mode);
+				if (md[i].size() == 1)
+					script[i] = md[i][0].nucleus()->asScriptInset();
 			}
 			bool const hasscript[2] = {script[0] != nullptr, script[1] != nullptr};
-			cell->push_back(MathAtom(new InsetMathSideset(buf, hasscript[0], hasscript[1])));
+			cell->emplace_back(new InsetMathSideset(buf, hasscript[0], hasscript[1]));
 			if (hasscript[0]) {
 				if (script[0]->hasDown())
 					cell->back().nucleus()->cell(1) = script[0]->down();
 				if (script[0]->hasUp())
 					cell->back().nucleus()->cell(2) = script[0]->up();
 			} else
-				cell->back().nucleus()->cell(1) = ar[0];
+				cell->back().nucleus()->cell(1) = md[0];
 			if (hasscript[1]) {
 				if (script[1]->hasDown())
 					cell->back().nucleus()->cell(2 + hasscript[0]) = script[1]->down();
 				if (script[1]->hasUp())
 					cell->back().nucleus()->cell(3 + hasscript[0]) = script[1]->up();
 			} else
-				cell->back().nucleus()->cell(2 + hasscript[0]) = ar[1];
+				cell->back().nucleus()->cell(2 + hasscript[0]) = md[1];
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 		}
 
 		else if (t.cs() == "stackrel") {
 			// Here allowed formats are \stackrel[subscript]{superscript}{operator}
-			MathData ar(buf);
-			parse(ar, FLAG_OPTION, mode);
-			cell->push_back(MathAtom(new InsetMathStackrel(buf, !ar.empty())));
-			if (!ar.empty())
-				cell->back().nucleus()->cell(2) = ar;
+			MathData md(buf);
+			parse(md, FLAG_OPTION, mode);
+			cell->emplace_back(new InsetMathStackrel(buf, !md.empty()));
+			if (!md.empty())
+				cell->back().nucleus()->cell(2) = md;
 			parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 		}
@@ -1546,10 +1570,10 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		}
 
 		else if (t.cs() == "ref" || t.cs() == "eqref" || t.cs() == "prettyref"
-			  || t.cs() == "nameref" || t.cs() == "pageref"
+			  || t.cs() == "nameref" || t.cs() == "pageref" || t.cs() == "cpageref"
 			  || t.cs() == "vpageref" || t.cs() == "vref" 
 			  || t.cs() == "formatted" || t.cs() == "labelonly") {
-			cell->push_back(MathAtom(new InsetMathRef(buf, t.cs())));
+			cell->emplace_back(new InsetMathRef(buf, t.cs()));
 			docstring const opt = parse_verbatim_option();
 			docstring const ref = parse_verbatim_item();
 			if (!opt.empty()) {
@@ -1567,14 +1591,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			// can't handle \|
 			// FIXME: fix this in InsetMathDelim itself!
 			docstring const l = tl.cs() == "|" ? from_ascii("Vert") : tl.asString();
-			MathData ar(buf);
-			parse(ar, FLAG_RIGHT, mode);
+			MathData md(buf);
+			parse(md, FLAG_RIGHT, mode);
 			if (!good())
 				break;
 			skipSpaces();
 			Token const & tr = getToken();
 			docstring const r = tr.cs() == "|" ? from_ascii("Vert") : tr.asString();
-			cell->push_back(MathAtom(new InsetMathDelim(buf, l, r, ar)));
+			cell->emplace_back(new InsetMathDelim(buf, l, r, md));
 		}
 
 		else if (t.cs() == "right") {
@@ -1599,16 +1623,16 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (name == "array" || name == "subarray") {
 				docstring const valign = parse_verbatim_option() + 'c';
 				docstring const halign = parse_verbatim_item();
-				cell->push_back(MathAtom(new InsetMathArray(buf, name,
-					InsetMathGrid::guessColumns(halign), 1, (char)valign[0], halign)));
+				cell->emplace_back(new InsetMathArray(buf, name,
+					InsetMathGrid::guessColumns(halign), 1, (char)valign[0], halign));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, false);
 			}
 
 			else if (name == "tabular") {
 				docstring const valign = parse_verbatim_option() + 'c';
 				docstring const halign = parse_verbatim_item();
-				cell->push_back(MathAtom(new InsetMathTabular(buf, name,
-					InsetMathGrid::guessColumns(halign), 1, (char)valign[0], halign)));
+				cell->emplace_back(new InsetMathTabular(buf, name,
+					InsetMathGrid::guessColumns(halign), 1, (char)valign[0], halign));
 				parse2(cell->back(), FLAG_END, InsetMath::TEXT_MODE, false);
 			}
 
@@ -1621,17 +1645,17 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				docstring const valign = parse_verbatim_option() + 'c';
 				// ignore this for a while
 				getArg('{', '}');
-				cell->push_back(MathAtom(new InsetMathSplit(buf, name, (char)valign[0])));
+				cell->emplace_back(new InsetMathSplit(buf, name, (char)valign[0]));
 				parse2(cell->back(), FLAG_END, mode, false);
 			}
 
 			else if (name == "math") {
 				if (mode == InsetMath::UNDECIDED_MODE) {
-					cell->push_back(MathAtom(new InsetMathHull(buf, hullSimple)));
+					cell->emplace_back(new InsetMathHull(buf, hullSimple));
 					parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, false);
 				} else {
 					// Don't create nested math hulls (bug #5392)
-					cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+					cell->emplace_back(new InsetMathEnsureMath(buf));
 					parse(cell->back().nucleus()->cell(0), FLAG_END, InsetMath::MATH_MODE);
 				}
 			}
@@ -1642,7 +1666,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					error("bad math environment " + name);
 					break;
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullEquation)));
+				cell->emplace_back(new InsetMathHull(buf, hullEquation));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, (name == "equation"));
 			}
 
@@ -1651,17 +1675,17 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					error("bad math environment " + name);
 					break;
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullEqnArray)));
+				cell->emplace_back(new InsetMathHull(buf, hullEqnArray));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
 			else if (name == "align" || name == "align*") {
 				if (mode == InsetMath::UNDECIDED_MODE) {
-					cell->push_back(MathAtom(new InsetMathHull(buf, hullAlign)));
+					cell->emplace_back(new InsetMathHull(buf, hullAlign));
 					parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 				} else {
-					cell->push_back(MathAtom(new InsetMathSplit(buf, name,
-							'c', !stared(name))));
+					cell->emplace_back(new InsetMathSplit(buf, name,
+							'c', !stared(name)));
 					parse2(cell->back(), FLAG_END, mode, !stared(name));
 				}
 			}
@@ -1671,7 +1695,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					error("bad math environment " + name);
 					break;
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullFlAlign)));
+				cell->emplace_back(new InsetMathHull(buf, hullFlAlign));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1682,7 +1706,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				}
 				// ignore this for a while
 				getArg('{', '}');
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullAlignAt)));
+				cell->emplace_back(new InsetMathHull(buf, hullAlignAt));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1693,7 +1717,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				}
 				// ignore this for a while
 				getArg('{', '}');
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullXAlignAt)));
+				cell->emplace_back(new InsetMathHull(buf, hullXAlignAt));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1704,7 +1728,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				}
 				// ignore this for a while
 				getArg('{', '}');
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullXXAlignAt)));
+				cell->emplace_back(new InsetMathHull(buf, hullXXAlignAt));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1713,7 +1737,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					error("bad math environment " + name);
 					break;
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullMultline)));
+				cell->emplace_back(new InsetMathHull(buf, hullMultline));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1722,7 +1746,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					error("bad math environment " + name);
 					break;
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullGather)));
+				cell->emplace_back(new InsetMathHull(buf, hullGather));
 				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
@@ -1732,8 +1756,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					parse2(cell->back(), FLAG_END, mode, false);
 				} else if (l->inset == "split") {
 					docstring const valign = parse_verbatim_option() + 'c';
-					cell->push_back(MathAtom(
-						new InsetMathSplit(buf, name, (char)valign[0])));
+					cell->emplace_back(new InsetMathSplit(buf, name, (char)valign[0]));
 					parse2(cell->back(), FLAG_END, mode, false);
 				} else {
 					success_ = false;
@@ -1746,7 +1769,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						       << "'." << endl;
 					}
 					// create generic environment inset
-					cell->push_back(MathAtom(new InsetMathEnv(buf, name)));
+					cell->emplace_back(new InsetMathEnv(buf, name));
 					parse(cell->back().nucleus()->cell(0), FLAG_END, mode);
 				}
 			}
@@ -1760,7 +1783,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					       << to_utf8(name) << "'" << endl;
 				}
 				// create generic environment inset
-				cell->push_back(MathAtom(new InsetMathEnv(buf, name)));
+				cell->emplace_back(new InsetMathEnv(buf, name));
 				parse(cell->back().nucleus()->cell(0), FLAG_END, mode);
 			}
 		}
@@ -1783,21 +1806,21 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					break;
 			}
 			if (s.empty())
-				cell->push_back(MathAtom(new InsetMathMacro(buf, t.cs())));
+				cell->emplace_back(new InsetMathMacro(buf, t.cs()));
 			else
-				cell->push_back(MathAtom(new InsetMathKern(buf, s)));
+				cell->emplace_back(new InsetMathKern(buf, s));
 		}
 
 		else if (t.cs() == "label") {
 			// FIXME: This is swallowed in inline formulas
 			docstring label = parse_verbatim_item();
-			MathData ar(buf);
-			asArray(label, ar);
+			MathData md(buf);
+			md.emplace_back(new InsetMathString(buf, label));
 			if (grid.asHullInset()) {
 				grid.asHullInset()->label(cellrow, label);
 			} else {
 				cell->push_back(createInsetMath(t.cs(), buf));
-				cell->push_back(MathAtom(new InsetMathBrace(buf, ar)));
+				cell->emplace_back(new InsetMathBrace(buf, md));
 			}
 		}
 
@@ -1814,14 +1837,14 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 		else if (t.cs() == "color") {
 			docstring const color = parse_verbatim_item();
-			cell->push_back(MathAtom(new InsetMathColor(buf, true, color)));
+			cell->emplace_back(new InsetMathColor(buf, true, color));
 			parse(cell->back().nucleus()->cell(0), flags, mode);
 			return success_;
 		}
 
 		else if (t.cs() == "textcolor") {
 			docstring const color = parse_verbatim_item();
-			cell->push_back(MathAtom(new InsetMathColor(buf, false, color)));
+			cell->emplace_back(new InsetMathColor(buf, false, color));
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, InsetMath::TEXT_MODE);
 		}
 
@@ -1886,16 +1909,16 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			docstring const arg = parse_verbatim_item();
 			Length length;
 			if (prot && arg == "\\fill")
-				cell->push_back(MathAtom(new InsetMathSpace(buf, "hspace*{\\fill}", "")));
+				cell->emplace_back(new InsetMathSpace(buf, "hspace*{\\fill}", ""));
 			else if (isValidLength(to_utf8(arg), &length))
-				cell->push_back(MathAtom(new InsetMathSpace(buf, length, prot)));
+				cell->emplace_back(new InsetMathSpace(buf, length, prot));
 			else {
 				// Since the Length class cannot use length variables
 				// we must not create an InsetMathSpace.
-				cell->push_back(MathAtom(new InsetMathMacro(buf, name)));
-				MathData ar(buf);
-				mathed_parse_cell(ar, '{' + arg + '}', mode_);
-				cell->append(ar);
+				cell->emplace_back(new InsetMathMacro(buf, name));
+				MathData md(buf);
+				mathed_parse_cell(md, '{' + arg + '}', mode_);
+				cell->append(md);
 			}
 		}
 
@@ -1911,13 +1934,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 				} else {
 					docstring const arg = parse_verbatim_item();
-					cell->push_back(MathAtom(new InsetMathMacro(buf, t.cs())));
-					MathData ar(buf);
-					mathed_parse_cell(ar, '[' + opt + ']', mode_);
-					cell->append(ar);
-					ar = MathData(buf);
-					mathed_parse_cell(ar, '{' + arg + '}', mode_);
-					cell->append(ar);
+					cell->emplace_back(new InsetMathMacro(buf, t.cs()));
+					MathData md(buf);
+					mathed_parse_cell(md, '[' + opt + ']', mode_);
+					cell->append(md);
+					md = MathData(buf);
+					mathed_parse_cell(md, '{' + arg + '}', mode_);
+					cell->append(md);
 				}
 			}
 			else {
@@ -1928,8 +1951,8 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 #if 0
 		else if (t.cs() == "infer") {
-			MathData ar(buf);
-			parse(ar, FLAG_OPTION, mode);
+			MathData md(buf);
+			parse(md, FLAG_OPTION, mode);
 			cell->push_back(createInsetMath(t.cs(), buf));
 			parse2(cell->back(), FLAG_ITEM, mode, false);
 		}
@@ -1963,10 +1986,10 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					Encodings::MATH_CMD | Encodings::TEXT_CMD,
 					termination, rem);
 				for (char_type c : cmd)
-					cell->push_back(MathAtom(new InsetMathChar(buf, c)));
+					cell->emplace_back(new InsetMathChar(buf, c));
 				if (!rem.empty()) {
 					char_type c = rem[0];
-					cell->push_back(MathAtom(new InsetMathChar(buf, c)));
+					cell->emplace_back(new InsetMathChar(buf, c));
 					cmd = rem.substr(1);
 					rem.clear();
 				} else
@@ -1989,10 +2012,27 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (l && !is_user_macro) {
 				if (l->inset == "big") {
 					skipSpaces();
+					// If we are parsing for an optional argument,
+					// we must not handle ']' as a delimiter (bug 8934).
+					// In standard LaTeX \foo[\Big] will give an error
+					// "Missing delimiter (. inserted)", however in
+					// combination with \DeclarePairedDelimiterX from
+					// the mathtools package \Big on its own inside an
+					// optional argument is valid:
+					// \documentclass{article}
+					// \usepackage{mathtools}
+					// \DeclarePairedDelimiterX{\COM}[2]{[}{]}{#1,#2}
+					// \newcommand{\Com}[3][]{\COM[#1]{#2}{#3}}
+					// \begin{document}
+					// $\Com[\Big]AB$
+					// \end{document}
+					// Therefore, if we are looking for FLAG_BRACK_LAST,
+					// and if the delimiter is ']', let FLAG_BRACK_LAST
+					// take precedence.
 					docstring const delim = getToken().asInput();
-					if (InsetMathBig::isBigInsetDelim(delim))
-						cell->push_back(MathAtom(
-											new InsetMathBig(buf, t.cs(), delim)));
+					if (InsetMathBig::isBigInsetDelim(delim) &&
+					    (delim != "]" || !(flags & FLAG_BRACK_LAST)))
+						cell->emplace_back(new InsetMathBig(buf, t.cs(), delim));
 					else {
 						cell->push_back(createInsetMath(t.cs(), buf));
 						// For some reason delim.empty()
@@ -2092,7 +2132,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 							}
 						}
 						is_unicode_symbol = true;
-						cell->push_back(MathAtom(new InsetMathChar(buf, c)));
+						cell->emplace_back(new InsetMathChar(buf, c));
 					} else {
 						while (num_tokens--)
 							putback();
@@ -2109,19 +2149,57 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						m = at->currentMode();
 					//lyxerr << "default creation: m2: " << m << endl;
 					idx_type start = 0;
-					// this fails on \bigg[...\bigg]
-					//MathData opt;
-					//parse(opt, FLAG_OPTION, InsetMath::VERBATIM_MODE);
-					//if (!opt.empty()) {
-					//	start = 1;
-					//	at.nucleus()->cell(0) = opt;
-					//}
-					for (idx_type i = start; i < at->nargs(); ++i) {
-						parse(at.nucleus()->cell(i), FLAG_ITEM, m);
-						if (mode == InsetMath::MATH_MODE)
+					// The code for handling arguments of at does not work
+					// together with the optional argument handling, therefore
+					// we must not use the latter if at->nargs() > 0.
+					size_t optionals = 0;
+					if (is_user_macro && at->nargs() == 0 && buf) {
+						if (mode_ & Parse::TRACKMACRO) {
+							Buffer::UserMacroSet::const_iterator it =
+								buf->usermacros.find(t.cs());
+							if (it != buf->usermacros.end())
+								optionals = it->second;
+						} else {
+							MacroData const * const user_macro =
+								buf->getMacro(t.cs(), false);
+							if (user_macro)
+								optionals = user_macro->optionals();
+
+						}
+					}
+					if (optionals > 0)
+					{
+						cell->push_back(at);
+						// Try to parse the optional arguments with FLAG_OPTION.
+						// Without this the end of the optional arg will not
+						// be handled correctly (bug 8934).
+						// We must know the number of allowed optional arguments
+						// of the macro template. Otherwise user defined macros
+						// similar to big operators like \mybigg[...\mybigg]
+						// would not be parsed correctly.
+						for (size_t i = 0; i < optionals; ++i) {
+							// We need to distinguish empty arguments []
+							// and missing arguments, since this makes a
+							// difference for some macros.
+							MathData opt(buf);
 							skipSpaces();
+							if (good() && nextToken().asInput() == "[") {
+								parse(opt, FLAG_OPTION, m);
+								// the optional arg will later be attached to the
+								// macro by MathData::collectOptionalParameters()
+								opt.insert(0, MathAtom(new InsetMathChar(buf, '[')));
+								opt.push_back(MathAtom(new InsetMathChar(buf, ']')));
+								cell->append(opt);
+							}
+						}
+					} else {
+						for (idx_type i = start; i < at->nargs(); ++i) {
+							parse(at.nucleus()->cell(i), FLAG_ITEM, m);
+							if (mode == InsetMath::MATH_MODE)
+								skipSpaces();
+						}
+						cell->push_back(at);
 					}
-					cell->push_back(at);
 				}
 			}
 		}
@@ -2140,7 +2218,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 
 // FIXME This will likely need some work.
-char const * latexkeys::MathMLtype() const
+std::string latexkeys::mathml_type() const
 {
 	// Consider as identifier (mml:mi): ordinary and alphabetical characters.
 	if (extra == "mathord" || extra == "mathalpha")
@@ -2150,16 +2228,16 @@ char const * latexkeys::MathMLtype() const
 }
 
 
-bool mathed_parse_cell(MathData & ar, docstring const & str, Parse::flags f)
+bool mathed_parse_cell(MathData & md, docstring const & str, Parse::flags f)
 {
-	return Parser(str, f, ar.buffer()).parse(ar, 0, (f & Parse::TEXTMODE) ?
+	return Parser(str, f, md.buffer()).parse(md, 0, (f & Parse::TEXTMODE) ?
 				InsetMath::TEXT_MODE : InsetMath::MATH_MODE);
 }
 
 
-bool mathed_parse_cell(MathData & ar, istream & is, Parse::flags f)
+bool mathed_parse_cell(MathData & md, istream & is, Parse::flags f)
 {
-	return Parser(is, f, ar.buffer()).parse(ar, 0, (f & Parse::TEXTMODE) ?
+	return Parser(is, f, md.buffer()).parse(md, 0, (f & Parse::TEXTMODE) ?
 				InsetMath::TEXT_MODE : InsetMath::MATH_MODE);
 }
 
