@@ -14,7 +14,6 @@
 #include "output_docbook.h"
 
 #include "Buffer.h"
-#include "buffer_funcs.h"
 #include "BufferParams.h"
 #include "Font.h"
 #include "InsetList.h"
@@ -25,7 +24,6 @@
 #include "Text.h"
 #include "TextClass.h"
 
-#include "insets/InsetBibtex.h"
 #include "insets/InsetBibitem.h"
 #include "insets/InsetLabel.h"
 #include "mathed/InsetMath.h"
@@ -389,7 +387,8 @@ void makeParagraph(
 			INCLUDE_CODE,
 			NOMENCL_PRINT_CODE,
 			TOC_CODE, // To be ignored in DocBook, the processor afterwards should deal with ToCs.
-			NOTE_CODE // Notes do not produce any output.
+			NOTE_CODE, // Notes do not produce any output.
+			INDEX_PRINT_CODE
 	};
 	auto isLyxCodeSpecialCase = [](InsetList::Element inset) {
 		return lyxCodeSpecialCases.find(inset.inset->lyxCode()) != lyxCodeSpecialCases.end();
@@ -1279,10 +1278,10 @@ void docbookParagraphs(Text const &text,
 					if (inset) {
 						if (auto label = dynamic_cast<InsetLabel const *>(inset)) {
 							// Generate the attributes for the section if need be.
-							id += "xml:id=\"" + xml::cleanID(label->screenLabel()) + "\"";
+							id += "xml:id=\"" + xml::cleanID(label->getParam("name")) + "\"";
 
 							// Don't output the ID as a DocBook <anchor>.
-							ourparams.docbook_anchors_to_ignore.emplace(label->screenLabel());
+							ourparams.docbook_anchors_to_ignore.emplace(label->getParam("name"));
 
 							// Cannot have multiple IDs per tag. If there is another ID inset in the document, it will
 							// be output as a DocBook anchor.
@@ -1417,4 +1416,4 @@ void docbookParagraphs(Text const &text,
 	}
 }
 
-} // namespace lyx
\ No newline at end of file
+} // namespace lyx
