@@ -13,7 +13,6 @@
 #include "output_plaintext.h"
 
 #include "Buffer.h"
-#include "BufferParams.h"
 #include "Layout.h"
 #include "output.h"
 #include "OutputParams.h"
@@ -24,7 +23,6 @@
 #include "insets/Inset.h"
 
 #include "support/debug.h"
-#include "support/gettext.h"
 #include "support/lstrings.h"
 
 using namespace std;
