@@ -12,15 +12,13 @@
 
 #import <AvailabilityMacros.h>
 
+#include <wchar.h>
+
 #include "support/AppleSpeller.h"
 
 typedef struct AppleSpellerRec {
 	NSSpellChecker * checker;
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	NSInteger doctag;
-#else
-	int doctag;
-#endif
 	NSArray * suggestions;
 	NSArray * misspelled;
 } AppleSpellerRec ;
@@ -52,16 +50,24 @@ void freeAppleSpeller(AppleSpeller speller)
 }
 
 
-static NSString * toString(const char * word)
+static NSString * toString(const char * lang)
 {
-	return [[NSString alloc] initWithBytes:word length:strlen(word) encoding:NSUTF8StringEncoding];
+	return [[NSString alloc] initWithBytes:lang length:strlen(lang) encoding:NSUTF8StringEncoding];
+}
+
+
+NSString * wcharToString(const wchar_t* text, NSUInteger length)
+{
+	BOOL lendian = NSHostByteOrder() == NS_LittleEndian;
+	NSUInteger bytes = length*sizeof(wchar_t);
+
+	return [[NSString alloc] initWithBytes:text length:bytes encoding:(lendian ? NSUTF32LittleEndianStringEncoding : NSUTF32BigEndianStringEncoding)];
 }
 
 
 static NSString * toLanguage(AppleSpeller speller, const char * lang)
 {
 	NSString * result = nil;
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 	NSString * lang_ = toString(lang);
 	if ([NSSpellChecker instancesRespondToSelector:@selector(availableLanguages)]) {
@@ -78,27 +84,74 @@ static NSString * toLanguage(AppleSpeller speller, const char * lang)
 	}
 	[lang_ release];
 	[pool release];
-#endif
 	return result;
 }
 
 
-SpellCheckResult AppleSpeller_check(AppleSpeller speller, const char * word, const char * lang)
+BOOL surrorate(unichar curr, unichar next) {
+	return 0xD800 <= curr && curr <= 0xDBFF &&
+		0xDC00 <= next && next <= 0xDFFF;
+}
+
+
+NSArray * AppleSpeller_adjustPositions(NSArray * misspelled,
+	NSString * text, NSUInteger length)
+{
+	NSUInteger r = 0;
+	NSUInteger i = 0;
+	NSUInteger rcount = [misspelled count];
+	NSRange range = [[misspelled objectAtIndex:r] rangeValue];
+	NSUInteger rstart = range.location;
+	NSUInteger rend   = range.location+range.length;
+	NSUInteger asurrogates = 0;
+	NSUInteger rsurrogates = 0;
+	NSMutableArray * result = [NSMutableArray arrayWithCapacity:rcount+1];
+	unichar curr = [text characterAtIndex:i];
+
+	while (i < length-1 && r < rcount) {
+		if (i == rstart) {
+			range.location -= asurrogates;
+			rsurrogates = 0;
+		} else if (i == rend) {
+			range.length -= rsurrogates;
+			[result addObject:[NSValue valueWithRange:range]];
+			if (++r < rcount) {
+				range = [[misspelled objectAtIndex:r] rangeValue];
+				rstart = range.location;
+				rend   = range.location+range.length;
+			}
+		}
+		unichar prev = curr;
+		curr = [text characterAtIndex:++i];
+		if (surrorate(prev, curr) && i < length-1) {
+			curr = [text characterAtIndex:++i];
+			asurrogates++;
+			rsurrogates++;
+		}
+	}
+	return result;
+}
+
+
+SpellCheckResult AppleSpeller_check(AppleSpeller speller,
+	const wchar_t * word, const char * lang)
 {
 	if (!speller->checker || !lang || !word)
 		return SPELL_CHECK_FAILED;
 
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-	NSString * word_ = toString(word);
+	NSUInteger wlength = wcslen(word);
+	NSString * word_ = wcharToString(word, wlength);
 	NSString * lang_ = toString(lang);
 	SpellCheckResult result = SPELL_CHECK_FAILED;
-	int start = 0;
-	int length = [word_ length];
+	NSUInteger start = 0;
+	NSUInteger ulength = [word_ length];
 
 	[speller->misspelled release];
 	speller->misspelled = nil;
+	BOOL surrogates = ulength > wlength;
 
-	while (result == SPELL_CHECK_FAILED && start < length) {
+	while (result == SPELL_CHECK_FAILED && start < ulength) {
 		NSRange match = [speller->checker
 			checkSpellingOfString:word_
 			startingAt:start
@@ -114,11 +167,7 @@ SpellCheckResult AppleSpeller_check(AppleSpeller speller, const char * word, con
 					result = SPELL_CHECK_LEARNED;
 			}
 		} else {
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 			NSUInteger capacity = [speller->misspelled count] + 1;
-#else
-			int capacity = [speller->misspelled count] + 1;
-#endif
 			NSMutableArray * misspelled = [NSMutableArray arrayWithCapacity:capacity];
 			[misspelled addObjectsFromArray:speller->misspelled];
 			[misspelled addObject:[NSValue valueWithRange:match]];
@@ -127,6 +176,11 @@ SpellCheckResult AppleSpeller_check(AppleSpeller speller, const char * word, con
 			start = match.location + match.length + 1;
 		}
 	}
+	if ([speller->misspelled count] > 0 && surrogates) {
+		NSArray * misspelled = AppleSpeller_adjustPositions(speller->misspelled, word_, ulength);
+		[speller->misspelled release];
+		speller->misspelled = [[NSArray arrayWithArray:misspelled] retain];
+	}
 
 	[word_ release];
 	[lang_ release];
@@ -136,10 +190,10 @@ SpellCheckResult AppleSpeller_check(AppleSpeller speller, const char * word, con
 }
 
 
-void AppleSpeller_ignore(AppleSpeller speller, const char * word)
+void AppleSpeller_ignore(AppleSpeller speller, const wchar_t * word)
 {
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-	NSString * word_ = toString(word);
+	NSString * word_ = wcharToString(word, wcslen(word));
 
 	[speller->checker ignoreWord:word_ inSpellDocumentWithTag:(speller->doctag)];
 
@@ -148,18 +202,16 @@ void AppleSpeller_ignore(AppleSpeller speller, const char * word)
 }
 
 
-size_t AppleSpeller_makeSuggestion(AppleSpeller speller, const char * word, const char * lang)
+size_t AppleSpeller_makeSuggestion(AppleSpeller speller, const wchar_t * word, const char * lang)
 {
 	if (!speller->checker || !word || !lang)
 		return 0;
 
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-	NSString * word_ = toString(word);
+	NSString * word_ = wcharToString(word, wcslen(word));
 	NSString * lang_ = toString(lang);
 	NSArray * result ;
 
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1060)
-	// Mac OS X 10.6 only
 	NSInteger slen = [word_ length];
 	NSRange range = { 0, slen };
 
@@ -167,10 +219,6 @@ size_t AppleSpeller_makeSuggestion(AppleSpeller speller, const char * word, cons
 		inString:word_
 		language:lang_
 		inSpellDocumentWithTag:speller->doctag];
-#else
-	[speller->checker setLanguage:lang_];
-	result = [speller->checker guessesForWord:word_];
-#endif
 
 	[word_ release];
 	[lang_ release];
@@ -193,33 +241,29 @@ const char * AppleSpeller_getSuggestion(AppleSpeller speller, size_t pos)
 }
 
 
-void AppleSpeller_learn(AppleSpeller speller, const char * word)
+void AppleSpeller_learn(AppleSpeller speller, const wchar_t * word)
 {
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-	NSString * word_ = toString(word);
+	NSString * word_ = wcharToString(word, wcslen(word));
 
 	if ([NSSpellChecker instancesRespondToSelector:@selector(learnWord:)])
 		[speller->checker learnWord:word_];
 
 	[word_ release];
 	[pool release];
-#endif
 }
 
 
-void AppleSpeller_unlearn(AppleSpeller speller, const char * word)
+void AppleSpeller_unlearn(AppleSpeller speller, const wchar_t * word)
 {
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-	NSString * word_ = toString(word);
+	NSString * word_ = wcharToString(word, wcslen(word));
 
 	if ([NSSpellChecker instancesRespondToSelector:@selector(unlearnWord:)])
 		[speller->checker unlearnWord:word_];
 
 	[word_ release];
 	[pool release];
-#endif
 }
 
 
@@ -231,11 +275,7 @@ int AppleSpeller_numMisspelledWords(AppleSpeller speller)
 
 void AppleSpeller_misspelledWord(AppleSpeller speller, int index, int * start, int * length)
 {
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	NSRange range = [[speller->misspelled objectAtIndex:(NSUInteger)index] rangeValue];
-#else
-	NSRange range = [[speller->misspelled objectAtIndex:index] rangeValue];
-#endif
 	*start = range.location;
 	*length = range.length;
 }
@@ -243,9 +283,5 @@ void AppleSpeller_misspelledWord(AppleSpeller speller, int index, int * start, i
 
 int AppleSpeller_hasLanguage(AppleSpeller speller, const char * lang)
 {
-#if defined(__MAC_OS_X_VERSION_MAX_ALLOWED) && (__MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
 	return toLanguage(speller, lang) != nil;
-#else
-	return true;
-#endif
 }
