@@ -20,8 +20,7 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 
-#include <boost/assert.hpp>
-
+#include <iostream>
 
 #ifdef LYX_CALLSTACK_PRINTING
 #include <cstdio>
@@ -37,12 +36,18 @@ namespace lyx {
 using namespace std;
 using namespace support;
 
+// This is implemented in LyX.cpp
+void lyx_exit(int);
 
 void doAssertWithCallstack(bool value)
 {
 	if (!value) {
 		printCallStack();
-		BOOST_ASSERT(false);
+		// FIXME: by default we exit here but we could also inform the
+		// user about the assertion and do the emergency cleanup
+		// without exiting.
+		// FIXME: do we have a list of exit codes defined somewhere?
+		lyx::lyx_exit(1);
 	}
 }
 
@@ -55,6 +60,16 @@ void doAssert(char const * expr, char const * file, long line)
 }
 
 
+void doAssertStatic(char const * expr, char const * file, long line)
+{
+	cerr << "ASSERTION " << expr << " VIOLATED IN " << file << ":" << line << endl;
+	// FIXME: by default we exit here but we could also inform the user
+	// about the assertion and do the emergency cleanup without exiting.
+	// FIXME: do we have a list of exit codes defined somewhere?
+	lyx::lyx_exit(1);
+}
+
+
 docstring formatHelper(docstring const & msg,
 	char const * expr, char const * file, long line)
 {
@@ -123,11 +138,11 @@ docstring printCallStack()
 			}
 		}
 		int status = 0;
-		const char* demangled =
+		char* demangled =
 			abi::__cxa_demangle(mangled, nullptr, nullptr, &status);
 		const QByteArray line = QString("(%1) %2: %3\n").arg(i, 3).arg(messages[i])
 								.arg(demangled ? demangled : orig.c_str()).toLocal8Bit();
-		free((void*)demangled);
+		free(demangled);
 
 		fprintf(stderr, "%s", line.constData());
 		bt += from_local8bit(line.constData());
