@@ -13,12 +13,14 @@
 #include "support/lyxtime.h"
 
 #include "support/debug.h"
-#include "support/environment.h"
 #include "support/lstrings.h"
 #include "support/qstring_helpers.h"
 
 #include <QDateTime>
 #include <QLocale>
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+#include <QTimeZone>
+#endif
 
 using namespace std;
 
@@ -34,20 +36,12 @@ time_t current_time()
 docstring formatted_datetime(time_t t, string const & fmt)
 {
 	QString qres;
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
 	if (fmt.empty())
 		qres = QLocale().toString(QDateTime::fromSecsSinceEpoch(t),
 		                          QLocale::ShortFormat);
 	else
 		qres = QLocale().toString(QDateTime::fromSecsSinceEpoch(t),
 					  toqstr(fmt));
-#else
-	if (fmt.empty())
-		qres = QLocale().toString(QDateTime::fromTime_t(t),
-		                          QLocale::ShortFormat);
-	else
-		qres = QLocale().toString(QDateTime::fromTime_t(t), toqstr(fmt));
-#endif
 	return qstring_to_ucs4(qres);
 }
 
@@ -69,12 +63,12 @@ time_t from_asctime_utc(string t)
 				<< "´ (invalid format)");
 		return static_cast<time_t>(-1);
 	}
-	loc_dt.setTimeSpec(Qt::UTC);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
-	return loc_dt.toSecsSinceEpoch();
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+	loc_dt.setTimeZone(QTimeZone(QTimeZone::UTC));
 #else
-	return loc_dt.toTime_t();
+	loc_dt.setTimeSpec(Qt::UTC);
 #endif
+	return loc_dt.toSecsSinceEpoch();
 }
 
 } // namespace support
