@@ -11,7 +11,6 @@
 #include <config.h>
 
 #include "support/userinfo.h"
-#include "support/environment.h"
 #include "support/docstring.h"
 
 #include "support/lassert.h"
@@ -35,40 +34,40 @@ using namespace std;
 namespace lyx {
 namespace support {
 
-docstring const user_name()
+string const user_name()
 {
 #if defined (_WIN32)
 
 	char name[UNLEN + 1];
 	DWORD size = UNLEN + 1;
 	if (!GetUserName(name, &size))
-		return _("Unknown user");
-	return from_local8bit(name);
+		return to_utf8(_("Unknown user"));
+	return to_utf8(from_local8bit(name));
 #else
 	struct passwd * pw = getpwuid(geteuid());
-	LASSERT(pw, return docstring());
+	LASSERT(pw, return string());
 
 	const string gecos = pw->pw_gecos;
 	const size_t pos = gecos.find(",");
 	string name = gecos.substr(0, pos);
 	if (name.empty())
 		name = pw->pw_name;
-	return from_local8bit(name);
+	return to_utf8(from_local8bit(name));
 #endif
 }
 
 
-docstring const user_email()
+string const user_email()
 {
 	//FIXME: quick fix wrt bug #3764; only Anonymous is detected now.
 	//The code after should be used only after user approval.
-	return docstring();
+	return string();
 
 #if 0
 	string email = getEnv("EMAIL_ADDRESS");
 	if (email.empty())
 		email = getEnv("EMAIL");
-	return from_local8bit(email);
+	return to_utf8(from_local8bit(email));
 #endif
 }
 
